#import "Platypus.h"
#include <stdio.h>

@implementation Platypus


- (id)init
{
	if (self = [super init]) 
	{
		//create file list
		fileList = [[FileList alloc] init];
		typesList = [[TypesList alloc] init];
		recentItems = [[NSMutableArray alloc] initWithCapacity: 10];
    }
    return self;
}

-(void)dealloc
{
    [fileList release];
	[typesList release];
	[recentItems release];
    [super dealloc];
}

+ (void)initialize 
{ 
	// create the user defaults here if none exists
    // create a dictionary
    NSMutableDictionary *defaultPrefs = [NSMutableDictionary dictionary];
    // put default prefs in the dictionary
	[defaultPrefs setObject: [NSNumber numberWithBool:YES] forKey:@"ParseShebangLineForInterpreter"];
	[defaultPrefs setObject: [NSNumber numberWithBool:NO]  forKey:@"RevealApplicationWhenCreated"];
	[defaultPrefs setObject: @"TextEdit"  forKey:@"DefaultEditor"];
	[defaultPrefs setObject: [NSNumber numberWithBool:NO] forKey:@"ShowAdvancedOptions"];
	
    // register the dictionary of defaults
    [[NSUserDefaults standardUserDefaults] registerDefaults: defaultPrefs];
}


- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
	//set up window
	[window center];
	if ([[NSUserDefaults standardUserDefaults] boolForKey:@"ShowAdvancedOptions"] == YES)
	{
		[showAdvancedArrow setState: YES];
		[self toggleAdvancedOptions: self];
	}
	[window registerForDraggedTypes: [NSArray arrayWithObjects:NSFilenamesPboardType, nil]];
	[window makeKeyAndOrderFront: NULL];
	
	//register file list for drag
	//[fileListDataBrowser registerForDraggedTypes: [NSArray arrayWithObjects:NSFilenamesPboardType, nil]];
	
	//load recent items
	[recentItems addObjectsFromArray: [[NSUserDefaults standardUserDefaults] objectForKey:@"RecentItems"]];
	[self constructOpenRecentMenu];
	
	//register the data source for the types list
	[typesWindow center];
	[self setDefaultTypes: self];
	[typesListDataBrowser setDataSource: typesList];
	[typesListDataBrowser reloadData];
}

- (void)applicationWillTerminate:(NSNotification *)aNotification
{
	//save recent items
	[[NSUserDefaults standardUserDefaults] setObject: recentItems  forKey:@"RecentItems"];
	
	//save window status
	[[NSUserDefaults standardUserDefaults] setBool: [showAdvancedArrow state]  forKey:@"ShowAdvancedOptions"];
	
}

- (BOOL)application:(NSApplication *)theApplication openFile:(NSString *)filename
{
	BOOL	isDir = NO;

	if([[NSFileManager defaultManager] fileExistsAtPath: filename isDirectory: &isDir] && !isDir)
	{
		[self loadScript: filename];
		return(YES);
	}
	else
		return(NO);
}

#pragma mark -


/*****************************************
 - Create a new script and open in external editor
*****************************************/

- (IBAction)newScript:(id)sender
{
	int			i = 0;
	NSString	*tempScript = @"/tmp/PlatypusScript";
	NSArray *defaultInterpreters = [NSArray arrayWithObjects: @"/bin/sh",@"/usr/bin/python",@"/usr/bin/osascript",@"/usr/bin/expect",@"/usr/bin/perl",@"/usr/bin/ruby",@"/usr/bin/tclsh",@"/usr/bin/php", nil];
	
	//put shebang line in the new script text file
	NSString	*shebangStr = [NSString stringWithFormat: @"#!%@\n\n", [defaultInterpreters objectAtIndex: [[scriptTypeRadioButtons selectedCell] tag]]];
	
	//if this is a perl or shell script, we add a commented list of paths to the bundled files 
	if (([[scriptTypeRadioButtons selectedCell] tag] == 0 || [[scriptTypeRadioButtons selectedCell] tag] == 4) && [fileList numFiles] > 0)
	{
		shebangStr = [shebangStr stringByAppendingString: @"# You can access your bundled files at the following paths:\n#\n"];
		for (i = 0; i < [fileList numFiles]; i++)
		{
			if ([[scriptTypeRadioButtons selectedCell] tag] == 0)//shell script
				shebangStr = [shebangStr stringByAppendingString: [NSString stringWithFormat:@"# \"$1/Contents/Resources/%@\"\n", [fileList getFileAtIndex: i]]];
			else if ([[scriptTypeRadioButtons selectedCell] tag] == 4)//perl script
				shebangStr = [shebangStr stringByAppendingString: [NSString stringWithFormat:@"# \"$ARGV[0]/Contents/Resources/%@\"\n", [fileList getFileAtIndex: i]]];
		}
		shebangStr = [shebangStr stringByAppendingString: @"#\n#\n\n"];
	}
	
	[shebangStr writeToFile: tempScript atomically: YES];

	[self loadScript: tempScript];
	[self editScript: self];

}

/*****************************************
 - Reveal script in Finder
*****************************************/

- (IBAction)revealScript:(id)sender
{
	if ([[scriptPathTextField stringValue] length] == 0)
		return;
		
	//see if file exists
	if ([[NSFileManager defaultManager] fileExistsAtPath: [scriptPathTextField stringValue] ])
	{
		[[NSWorkspace sharedWorkspace] selectFile:[scriptPathTextField stringValue] inFileViewerRootedAtPath:nil];
	}
	else
	{
		[STUtil alert:@"File does not exist" subText: @"No file exists at the specified path"];
	}
}


/*****************************************
 - Open script in external editor
*****************************************/

- (IBAction)editScript:(id)sender
{
	if ([[scriptPathTextField stringValue] length] == 0)
		return;
		
	//see if file exists
	if ([[NSFileManager defaultManager] fileExistsAtPath: [scriptPathTextField stringValue] ])
	{
		NSArray *args = [NSArray arrayWithObjects: @"-a", [[NSUserDefaults standardUserDefaults] stringForKey:@"DefaultEditor"], [scriptPathTextField stringValue], nil];
		[NSTask launchedTaskWithLaunchPath:@"/usr/bin/open" arguments: args];
	}
	else
		[STUtil alert:@"File does not exist" subText: @"No file exists at the specified path"];
}

/*****************************************
 - generate application bundle from data provided
*****************************************/

- (IBAction)createApp:(id)sender
{
	int		result, i;
	BOOL	isDir = FALSE;
	NSString *appPath, *contentsPath, *macosPath, *resourcesPath, *lprojPath;
	NSString *execDestinationPath, *infoPlistPath, *iconPath, *bundledFileDestPath, *progressBarNibDestPath;
	NSString *execPath, *bundledFilePath, *progressBarNibPath;
	NSString *scriptFilePath, *pkgInfoPath, *appSettingsPlistPath;
	NSString *appName;
	NSString *infoPlistStrings;
	NSBundle *platypusBundle = [NSBundle mainBundle];
	NSMutableDictionary	*appSettingsPlist;
	
	//file manager
	NSFileManager *fileManager = [NSFileManager defaultManager];
	
	/////////////**************verify that values in text fields are valid**********************//////////////
	
	if (([fileManager fileExistsAtPath: [scriptPathTextField stringValue] isDirectory: &isDir] == NO) && !isDir)//make sure script exists
	{
		[STUtil alert:@"Invalid Script Path" subText: @"No file exists at the specified script path"];
		return;
	}
	
	if ([fileManager fileExistsAtPath: [interpreterTextField stringValue]] == NO)//make sure interpreter exists
	{
		if (NO == [STUtil proceedWarning: @"Invalid Interpreter" subText: @"The specified interpreter does not exist on this system.  Do you wish to proceed anyway?"])
			return;
	}
	
	if ([typesList numTypes] <= 0 && [isDroppableCheckbox state] == YES)
	{
		[STUtil alert:@"Invalid Types List" subText: @"The app has been set to be droppable but no file types are set.  Please modify the Types list to correct this."];
		return;
	}
	
	/////////////**************init and run save panel**********************//////////////
	NSSavePanel *sPanel = [NSSavePanel savePanel];
	[sPanel setPrompt:@"Create"];
	[sPanel setTitle:@"Create App"];
	result = [sPanel runModalForDirectory:nil file: [appNameTextField stringValue]];
	if (result == NSOKButton) 
	{
		appPath = [sPanel filename];
		//[appPath stringByAppendingString:@".app"]
		//check if app already exists, and if so, prompt if to replace
		if ([fileManager fileExistsAtPath: [appPath stringByAppendingString:@".app"]])
		{
			if (NO == [STUtil proceedWarning: @"Application already exists" subText: @"An application with this name already exists in the location you specified.  Do you wish to overwrite it?"])
				return;
			else
				[fileManager removeFileAtPath: [appPath stringByAppendingString:@".app"] handler: NULL];
		}
	}
	else
		return;
	
	
	
	//////////********* Create the folder hierarchy ************////////////////
	
			//Application.app bundle
			if (![appPath hasSuffix:@".app"])
			{
				appPath = [appPath stringByAppendingString:@".app"];
			}
			[fileManager createDirectoryAtPath: appPath attributes:nil];
			
			//Contents
			contentsPath = [appPath stringByAppendingString:@"/Contents"];
			[fileManager createDirectoryAtPath: contentsPath attributes:nil];
			
			//MacOS
			macosPath = [contentsPath stringByAppendingString:@"/MacOS"];
			[fileManager createDirectoryAtPath: macosPath attributes:nil];
			
			//Resources
			resourcesPath = [contentsPath stringByAppendingString:@"/Resources"];
			[fileManager createDirectoryAtPath: resourcesPath attributes:nil];
			
			//English.lproj
			lprojPath = [resourcesPath stringByAppendingString:@"/English.lproj"];
			[fileManager createDirectoryAtPath: lprojPath attributes:nil];
			
			//PkgInfo file
			pkgInfoPath = [contentsPath stringByAppendingString:@"/PkgInfo"];
			[[NSString stringWithString:@"APPL????"] writeToFile: pkgInfoPath atomically: YES];
	
	//////////********* Copy files to app bundle ************////////////////
	
		//copy exec file
		appName = [STUtil cutSuffix: [appPath lastPathComponent]];
		execPath = [platypusBundle pathForResource:@"exec" ofType:nil];
		execDestinationPath = [macosPath stringByAppendingString:@"/"];
		execDestinationPath = [execDestinationPath stringByAppendingString: appName]; 
		[fileManager copyPath:execPath toPath:execDestinationPath handler:nil];
		
			//copy nib file to app bundle
			progressBarNibPath = [platypusBundle pathForResource:@"MainMenu.nib" ofType:nil];
			progressBarNibDestPath = [lprojPath stringByAppendingString:@"/MainMenu.nib"];
			[fileManager copyPath:progressBarNibPath toPath:progressBarNibDestPath handler:nil];
			
			//create InfoPlist.strings
			infoPlistStrings = [NSString stringWithFormat:
								@"CFBundleName = \"%@\";\nCFBundleShortVersionString = \"1.0\";\nCFBundleGetInfoString = \"%@ version 1.0 Copyright %d %@\";\nNSHumanReadableCopyright = \"Copyright %d %@.\";", appName, appName, [[NSCalendarDate calendarDate] yearOfCommonEra], NSFullUserName(), [[NSCalendarDate calendarDate] yearOfCommonEra], NSFullUserName()];
			[infoPlistStrings writeToFile:  [lprojPath stringByAppendingString:@"/InfoPlist.strings"] atomically: YES];

		//create AppSettings.plist file
		appSettingsPlist = [NSMutableDictionary dictionaryWithCapacity: 40];
		[appSettingsPlist setObject: [NSNumber numberWithBool:[rootPrivilegesCheckbox state]] forKey:@"RequiresAdminPrivileges"];
		[appSettingsPlist setObject: [NSNumber numberWithBool:[isDroppableCheckbox state]] forKey:@"IsDroppable"];
		[appSettingsPlist setObject: [NSNumber numberWithBool:[remainRunningCheckbox state]] forKey:@"RemainRunningAfterCompletion"];
		[appSettingsPlist setObject: [[outputTypeRadioButtons selectedCell] title] forKey:@"OutputType"];
		[appSettingsPlist setObject: [interpreterTextField stringValue] forKey:@"ScriptInterpreter"];
		appSettingsPlistPath = [resourcesPath stringByAppendingString:@"/AppSettings.plist"];
		[appSettingsPlist writeToFile: appSettingsPlistPath atomically: YES];
		
		// copy script file over to app bundle
		scriptFilePath = [resourcesPath stringByAppendingString:@"/script"];
		[fileManager copyPath: [scriptPathTextField stringValue] toPath: scriptFilePath handler:nil];
				
		//create icon
		iconPath = [resourcesPath stringByAppendingString:@"/appIcon.icns"];
		[self writeImageAsIcon: [appIconImageWell image] path: iconPath];
		
		//create Info.plist file
		infoPlistPath = [contentsPath stringByAppendingString:@"/Info.plist"];
		NSDictionary *infoPlist = [self getInfoPropertyListDictionary: appName isDroppable: [isDroppableCheckbox state] inBackground: [showInDockCheckbox state] ];
		[infoPlist writeToFile: infoPlistPath atomically: YES];
		
		//copy files in file list to the Resources folder
		for (i = 0; i < [fileList numFiles]; i++)
		{
			bundledFilePath = [fileList getFileAtIndex: i];
			bundledFileDestPath = [resourcesPath stringByAppendingString:@"/"];
			bundledFileDestPath = [bundledFileDestPath stringByAppendingString: [bundledFilePath lastPathComponent]];
			[fileManager copyPath: bundledFilePath toPath: bundledFileDestPath handler:nil];
		}
		
		//reveal app in Finder
		if ([[NSUserDefaults standardUserDefaults] boolForKey:@"RevealApplicationWhenCreated"] == YES)
		{
			BOOL		isDir;
			
			if ([[NSFileManager defaultManager] fileExistsAtPath:appPath isDirectory:&isDir]) 
			{
						[[NSWorkspace sharedWorkspace] selectFile:appPath inFileViewerRootedAtPath:nil];
			}
		}
}



/*****************************************
 - Called when script type radio button is pressed
*****************************************/

- (IBAction)scriptTypeSelected:(id)sender
{
	[self setScriptType: [[sender selectedCell] tag] ];
}

/*****************************************
 - Updates data in interpreter, icon and script type radio buttons
*****************************************/

- (void)setScriptType: (int)typeNum
{
	NSArray *defaultInterpreters = [NSArray arrayWithObjects: @"/bin/sh",@"/usr/bin/python",@"/usr/bin/osascript",@"/usr/bin/expect",@"/usr/bin/perl",@"/usr/bin/ruby",@"/usr/bin/tclsh",@"/usr/bin/php", nil];
	[interpreterTextField setStringValue: [defaultInterpreters objectAtIndex: typeNum ] ];
	[self setAppIconForType: typeNum];
	[scriptTypeRadioButtons selectCellWithTag: typeNum];
}

/*****************************************
 - Sets the app icon given the script type
*****************************************/

- (NSString *)getInterpreterFromShebang: (NSString *)path
{
	FILE			*file;
	static char		str[1024];
	char			filePath[1024];
	char			len,i;
	NSString		*interpreter;
	
	[path getCString: (char *)&filePath maxLength: sizeof(filePath)];
	
	//open file and read to the first newline character
	file = fopen(filePath, "r");
	fgets(str,1024,file);
	fclose(file);
	
	//doesn't seem to be a shebang line
	if (str[0] != '#' || str[1] != '!')
		return @"";
	
	//jump over the first two characters that indicate the shebang
	strcpy((char *)&str, (char *)&str[2]);
	
	len = strlen((char *)&str);

	for (i = 0; i < len; i++)
	{
		//terminate string when we reach a space or a newline
		if (str[i] == '\n'|| str[i] == ' ')
		{
			str[i] = NULL;
		}
	}
	interpreter = [NSString stringWithCString: (char *)&str];
	
	return interpreter;
}

- (void)setAppIconForType: (int)type
{
	switch(type)
	{
		case 0:
			[appIconImageWell setImage: [NSImage imageNamed:@"PlatypusDflt"]];
			[iconNameTextField setStringValue:@"Platypus Default"];
			break;
		case 1:
			[appIconImageWell setImage: [NSImage imageNamed:@"PythonSnake"]];
			[iconNameTextField setStringValue:@"Python Snake"];
			break;
		case 2:
			[appIconImageWell setImage: [NSImage imageNamed:@"AppleScript"]];
			[iconNameTextField setStringValue:@"AppleScript"];
			break;
		case 3:
			[appIconImageWell setImage: [NSImage imageNamed:@"ExpectMonkey"]];
			[iconNameTextField setStringValue:@"Hexley"];
			break;
		case 4:
			[appIconImageWell setImage: [NSImage imageNamed:@"PerlCamel"]];
			[iconNameTextField setStringValue:@"Perl Camel"];
			break;
		case 5:
			[appIconImageWell setImage: [NSImage imageNamed:@"RubyGoat"]];
			[iconNameTextField setStringValue:@"Ruby Goat"];
			break;
		case 6:
			[appIconImageWell setImage: [NSImage imageNamed:@"TclFlamingo"]];
			[iconNameTextField setStringValue:@"Tcl Flamingo"];
			break;
		case 7:
			[appIconImageWell setImage: [NSImage imageNamed:@"PHPBird"]];
			[iconNameTextField setStringValue:@"PHP Bird"];
			break;
		case 8:
			[appIconImageWell setImage: [NSImage imageNamed:@"PlatypusDroplet"]];
			[iconNameTextField setStringValue:@"Platypus Droplet"];
			break;
		case 9:
			[appIconImageWell setImage: [NSImage imageNamed:@"PlatypusPlate"]];
			[iconNameTextField setStringValue:@"Platypus Plate"];
			break;
	}
}

/*****************************************
 - Open panel to select script to load
*****************************************/

- (IBAction)selectScript:(id)sender
{
	int		result;
	
	//create open panel
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];
	[oPanel setTitle: @"Select Script"];
    [oPanel setAllowsMultipleSelection:NO];
	[oPanel setCanChooseDirectories: NO];
	
	//run open panel
    result = [oPanel runModalForDirectory:nil file:nil types:nil];
    if (result == NSOKButton) 
			[self loadScript: [oPanel filename]];
}

/*****************************************
 - Loads script data into platypus window
*****************************************/

- (void)loadScript:(NSString *)filename
{
	NSString	*shebangInterpreter;
	int			i;

	//make sure file we're loading actually exists
	if (![[NSFileManager defaultManager] fileExistsAtPath: filename])
		return;

	//set script path
	[scriptPathTextField setStringValue: filename];

	//set app name
	NSString *appName = [STUtil cutSuffix: [filename lastPathComponent]];
	[appNameTextField setStringValue: appName];
	
	//interpreter
	
	//read shebang line
	//if ([[NSUserDefaults standardUserDefaults] boolForKey:@"ParseShebangLineForInterpreter"] == 1)
	//{
		shebangInterpreter = [self getInterpreterFromShebang: filename];
	/*}
	else
	{
		shebangInterpreter = @"";
	}*/
	
	//if no interpreter can be retrieved from shebang line
	if ([shebangInterpreter caseInsensitiveCompare: @""] == NSOrderedSame)
	{
		//try to determine type from suffix
		[self setScriptType: (int)[self getFileTypeFromSuffix: filename] ];
	}
	else
	{
		//see if interpreter matches a preset
		NSArray *defaultInterpreters = [NSArray arrayWithObjects: @"/bin/sh",@"/usr/bin/python",@"/usr/bin/osascript",@"/usr/bin/expect",
																  @"/usr/bin/perl",@"/usr/bin/ruby",@"/usr/bin/tclsh",@"/usr/bin/php", nil];
		for (i = 0; i < [defaultInterpreters count]; i++)
		{
			if ([shebangInterpreter caseInsensitiveCompare: [defaultInterpreters objectAtIndex: i]] == NSOrderedSame)
			{
				[self setScriptType: i];
			}
		}
		
		//set shebang interpreter into interpreter field
		[interpreterTextField setStringValue: shebangInterpreter ];
	}
	
	//enable the edit/reveal/create buttons
	/*[createAppButton setEnabled: YES];
	[editScriptButton setEnabled: YES];
	[revealScriptButton setEnabled: YES];
	*/
	[self controlTextDidChange: NULL];
	
	//add to open recent menu
	if (![recentItems containsObject: filename])
	{
		if ([recentItems count] == 8)
			[recentItems removeObjectAtIndex: 0];

		[recentItems addObject: filename];
		[self constructOpenRecentMenu];
	}
}

/*****************************************
 - Toggles between advanced options mode
*****************************************/

- (IBAction)toggleAdvancedOptions:(id)sender
{
	#define		kWindowExpansionHeight  252
	NSRect winRect = [window frame];

	if ([showAdvancedArrow state] == NSOffState)
	{
		winRect.origin.y += kWindowExpansionHeight;
		winRect.size.height -= kWindowExpansionHeight;
		[showOptionsTextField setStringValue: @"Show Advanced Options"];
		[toggleAdvancedMenuItem setTitle: @"Show Advanced Options"];
		[window setFrame: winRect display:TRUE];
	}
	else if ([showAdvancedArrow state] == NSOnState)
	{
		winRect.origin.y -= kWindowExpansionHeight;
		winRect.size.height += kWindowExpansionHeight;
		[showOptionsTextField setStringValue: @"Hide advanced options"];
		[toggleAdvancedMenuItem setTitle: @"Hide Advanced Options"];
		[window setFrame: winRect display:TRUE];
	}
}

- (IBAction)isDroppableWasClicked:(id)sender
{
	[editTypesButton setHidden: ![sender state]];
	if ([sender state] == NO)
		[typesWindow performClose: self];
}

- (IBAction)outputTypeChanged: (id)sender
{
	if ([[sender selectedCell] tag] == 2)//Text output selected
	{
		[rootPrivilegesCheckbox setIntValue: 0];
		[rootPrivilegesCheckbox setEnabled: NO];
	}
	else
		[rootPrivilegesCheckbox setEnabled: YES];
}

#pragma mark -

/*****************************************
 - Flip between icons
*****************************************/
- (IBAction)switchIcons:(id)sender
{
	[self setAppIconForType: [sender intValue]];
}

- (IBAction)nextIcon:(id)sender
{
	if ([iconToggleButton intValue] + 1 > [iconToggleButton maxValue])
		[iconToggleButton setIntValue: [iconToggleButton minValue]];
	else
		[iconToggleButton setIntValue: [iconToggleButton intValue] + 1];
	
	[self setAppIconForType: [iconToggleButton intValue]];
}

- (IBAction)previousIcon:(id)sender
{
	if ([iconToggleButton intValue] - 1 < [iconToggleButton minValue])
		[iconToggleButton setIntValue: [iconToggleButton maxValue]];
	else
		[iconToggleButton setIntValue: [iconToggleButton intValue] - 1];
	
	[self setAppIconForType: [iconToggleButton intValue]];
}

/*****************************************
 - Write an NSImage as icon to a path
*****************************************/

- (void)writeImageAsIcon: (NSImage *)img path: (NSString *)path
{
	IconFamily *iconFam = [[IconFamily alloc] initWithThumbnailsOfImage: img];
	[iconFam writeToFile: path];
}

#pragma mark -

/*****************************************
 - Generate dictionary for Info.plist in app bundle
*****************************************/

- (NSDictionary *)getInfoPropertyListDictionary: (NSString *)appName isDroppable: (BOOL)droppable inBackground: (BOOL)background
{
	NSDictionary *infoPlist;
	NSString	 *lsui = [NSString stringWithFormat:@"%d", background];//LSUIElement value
	
	if (!droppable)
	{
		infoPlist = [NSDictionary dictionaryWithObjectsAndKeys: 
								@"English", @"CFBundleDevelopmentRegion",
								appName, @"CFBundleExecutable", 
								appName, @"CFBundleName",
								[NSString stringWithFormat: @"%@ 1.0 Copyright %d %@", appName, [[NSCalendarDate calendarDate] yearOfCommonEra], NSFullUserName() ], @"CFBundleGetInfoString", 
								@"appIcon.icns", @"CFBundleIconFile",  
								@"1.0", @"CFBundleVersion",  
								[self generateBundleIdentifier], @"CFBundleIdentifier",  
								lsui, @"LSUIElement",
								@"6.0", @"CFBundleInfoDictionaryVersion",
								@"APPL", @"CFBundlePackageType",
								@"????", @"CFBundleSignature",
								@"MainMenu", @"NSMainNibFile",  
								@"NSApplication", @"NSPrincipalClass",  nil];

	}
	else
	{	
		infoPlist = [NSDictionary dictionaryWithObjectsAndKeys:
								@"English", @"CFBundleDevelopmentRegion",
								appName, @"CFBundleExecutable",
								[NSArray arrayWithObject: [typesList getDictionaryForTypes: [[appFunctionRadioButtons selectedCell] title]]] ,@"CFBundleDocumentTypes",
								@"appIcon.icns", @"CFBundleIconFile",  
								@"1.0", @"CFBundleVersion", 
								@"org.platypus.app", @"CFBundleIdentifier",
								lsui, @"LSUIElement",
								@"6.0", @"CFBundleInfoDictionaryVersion",
								@"APPL", @"CFBundlePackageType",
								@"????", @"CFBundleSignature",
								@"MainMenu", @"NSMainNibFile",  
								@"NSApplication", @"NSPrincipalClass", nil];
	}
	return(infoPlist);
}


/*****************************************
 - Determine script type based on a file's suffix
*****************************************/

- (int)getFileTypeFromSuffix: (NSString *)fileName
{
	if ([fileName hasSuffix: @".sh"])
		return 0;
	else if ([fileName hasSuffix: @".py"])
		return 1;
	else if ([fileName hasSuffix: @".scpt"] || [fileName hasSuffix: @".applescript"])
		return 2;
	else if ([fileName hasSuffix: @".exp"] || [fileName hasSuffix: @".expect"])
		return 3;
	else if ([fileName hasSuffix: @".pl"] || [fileName hasSuffix: @".perl"])
		return 4;
	else if ([fileName hasSuffix: @".rb"] || [fileName hasSuffix: @".rbx"])
		return 5;
	else if ([fileName hasSuffix: @".tcl"])
		return 6;
	else if ([fileName hasSuffix: @".php"])
		return 7;
	else
		return 0;
}

/*****************************************
 - //return the bundle identifier for the application
*****************************************/
- (NSString *)generateBundleIdentifier
{
	NSString	*bundleId;
	//The format is "org.username.appname"
	bundleId = [NSString stringWithFormat: @"org.%@.%@", NSUserName(), [appNameTextField stringValue]];
	bundleId = [[bundleId componentsSeparatedByString:@" "] componentsJoinedByString:@""];//no spaces
	return(bundleId);
}

#pragma mark -

- (IBAction)showPrefs:(id)sender
{
	[parseShebangCheckbox setState: [[NSUserDefaults standardUserDefaults] boolForKey:@"ParseShebangLineForInterpreter"]];
	[revealAppCheckbox setState: [[NSUserDefaults standardUserDefaults] boolForKey:@"RevealApplicationWhenCreated"]];
	[defaultEditorMenu setTitle: [[NSUserDefaults standardUserDefaults] stringForKey:@"DefaultEditor"]];
	
	[prefsWindow center];
	[prefsWindow makeKeyAndOrderFront: sender];
}

- (IBAction)applyPrefs:(id)sender
{
	[[NSUserDefaults standardUserDefaults] setBool: [parseShebangCheckbox state] forKey:@"ParseShebangLineForInterpreter"];
	[[NSUserDefaults standardUserDefaults] setBool: [revealAppCheckbox state]  forKey:@"RevealApplicationWhenCreated"];
	[[NSUserDefaults standardUserDefaults] setObject: [defaultEditorMenu titleOfSelectedItem]  forKey:@"DefaultEditor"];
	
	[prefsWindow performClose: sender];
}

- (IBAction) selectScriptEditor:(id)sender
{
	int			result;
	NSString	*editorName;
	
	//create open panel
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];
	[oPanel setTitle: @"Select Editor"];
    [oPanel setAllowsMultipleSelection:NO];
	[oPanel setCanChooseDirectories: NO];
	
	//run open panel
    result = [oPanel runModalForDirectory:nil file:nil types: [NSArray arrayWithObject:@"app"]];
    if (result == NSOKButton) 
	{
		//set app name minus .app suffix
		editorName = [STUtil cutSuffix: [[oPanel filename] lastPathComponent]];
		[defaultEditorMenu setTitle: editorName];
	}
	else
		[defaultEditorMenu setTitle: [[NSUserDefaults standardUserDefaults] stringForKey:@"DefaultEditor"]];
}

#pragma mark -

/*****************************************
 - Dragging and dropping for Platypus window
*****************************************/

- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
{
	NSPasteboard	*pboard = [sender draggingPasteboard];
	NSString		*filename;
	BOOL			isDir = FALSE;
	int				i = 0;

    if ( [[pboard types] containsObject:NSFilenamesPboardType] ) 
	{
        NSArray *files = [pboard propertyListForType:NSFilenamesPboardType];

		//files were dropped in the file list table
		if (NSMouseInRect([sender draggingLocation], NSMakeRect(22, 34, 568, 145), NO) && [showAdvancedArrow state] == NSOnState)
		{
			for (i = 0; i < [files count]; i++)
			{
				//we don't add if it already exists in the array
				if ([fileList hasFile: [files objectAtIndex:i]] == NO)
					[fileList addFile: [files objectAtIndex:i]];
			}
			
			//reload data in databrowser
			[fileListDataBrowser setDataSource: fileList];
			[fileListDataBrowser reloadData];
			//update button status
			[self tableViewSelectionDidChange: NULL];
			//update size field
			[self updateFileSizeField];
			return YES;
		}
		else //files were dropped on the window
		{
			filename = [files objectAtIndex: 0];//we only load the first dragged item
			if ([[NSFileManager defaultManager] fileExistsAtPath: filename isDirectory:&isDir] && !isDir)
			{
				[self loadScript: filename];
				return YES;
			}
		}
    }

    
	return NO;
}

- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender 
{

    NSPasteboard *pboard;
    NSDragOperation sourceDragMask;

    sourceDragMask = [sender draggingSourceOperationMask];
    pboard = [sender draggingPasteboard];


    if ( [[pboard types] containsObject:NSFilenamesPboardType] ) 
	{
        if (sourceDragMask & NSDragOperationLink) 
            return NSDragOperationLink;
		else if (sourceDragMask & NSDragOperationCopy)
            return NSDragOperationCopy;
    }

    return NSDragOperationNone;
}

#pragma mark -

- (void)tableViewSelectionDidChange:(NSNotification *)aNotification
{
	int i;
	int selected = 0;
	NSIndexSet *selectedItems = [fileListDataBrowser selectedRowIndexes];
	
	for (i = 0; i < [fileList numFiles]; i++)
	{
		if ([selectedItems containsIndex: i])
		{
			selected++;
		}
	}
	
	//update button status
	if (selected == 0)
	{
		[removeFileButton setEnabled: NO];
		[revealFileButton setEnabled: NO];
	}
	else
	{
		[removeFileButton setEnabled: YES];
		[revealFileButton setEnabled: YES];
	}
	
	if ([fileList numFiles] == 0)
		[clearFileListButton setEnabled: NO];
	else
		[clearFileListButton setEnabled: YES];	
}

- (void)updateFileSizeField
{
	int			i;
	UInt64		totalSize = 0;
	NSString	*totalSizeString;

	//if there are no items, we just list it as 0 items
	if ([fileList numFiles] <= 0)
	{
		[bundleSizeTextField setStringValue: [NSString stringWithFormat: @"%d items", [fileList numFiles]]];
		return;
	}
	
	//otherwise, loop through all files, calculate size
	for (i = 0; i < [fileList numFiles]; i++)
	{		
		totalSize += [STUtil fileOrFolderSize: [fileList getFileAtIndex: i]];
	}
	
	totalSizeString = [STUtil sizeAsHumanReadable: totalSize];
	if ([fileList numFiles] > 1)
		[bundleSizeTextField setStringValue: [NSString stringWithFormat: @"%d items  ( %@ )", [fileList numFiles], totalSizeString]];
	else
		[bundleSizeTextField setStringValue: [NSString stringWithFormat: @"%d item  ( %@ )", [fileList numFiles], totalSizeString]];
}

- (void)controlTextDidChange:(NSNotification *)aNotification
{
	BOOL	isDir;
	
	//app name was changed
	if ([aNotification object] == appNameTextField || [aNotification object] == NULL)
	{
		if ([[appNameTextField stringValue] length] > 0)
			[bundleIdentifierTextField setStringValue: [NSString stringWithFormat: @"Bundle identifier will be %@",[self generateBundleIdentifier]]];
		else
			[bundleIdentifierTextField setStringValue: @""];
	}
	//scrip path changed
	if ([aNotification object] == scriptPathTextField || [aNotification object] == NULL)
	{
		if ([[NSFileManager defaultManager] fileExistsAtPath: [scriptPathTextField stringValue] isDirectory:&isDir] && !isDir)
		{
			[createAppButton setEnabled: YES];
			[editScriptButton setEnabled: YES];
			[revealScriptButton setEnabled: YES];
		}
		else
		{
			[createAppButton setEnabled: NO];
			[editScriptButton setEnabled: NO];
			[revealScriptButton setEnabled: NO];
		}
	}
}


- (BOOL)validateMenuItem:(NSMenuItem*)anItem 
{
	BOOL isDir;

	//edit script
	if ([[anItem title] isEqualToString:@"Edit Script"] && (![[NSFileManager defaultManager] fileExistsAtPath: [scriptPathTextField stringValue] isDirectory:&isDir] || isDir))
	{
		return NO;
	}
	//reveal script
	if ([[anItem title] isEqualToString:@"Reveal Script"] && (![[NSFileManager defaultManager] fileExistsAtPath: [scriptPathTextField stringValue] isDirectory:&isDir] || isDir))
	{
		return NO;
	}
	//create app menu
	if ([[anItem title] isEqualToString:@"Create App"] && (![[NSFileManager defaultManager] fileExistsAtPath: [scriptPathTextField stringValue] isDirectory:&isDir] || isDir))
	{
		return NO;
	}

	//File list menu items
	if ([[anItem title] isEqualToString:@"Add File To Bundle"] && [showAdvancedArrow state] == NSOffState)
		return NO;
		
    if ([[anItem title] isEqualToString:@"Remove File"] && ([fileList numFiles] == 0 || [showAdvancedArrow state] == NSOffState))
		return NO;

    if ([[anItem title] isEqualToString:@"Reveal File In File List"] && ([fileList numFiles] == 0 || [showAdvancedArrow state] == NSOffState))
		return NO;
	
	if ([[anItem title] isEqualToString:@"Clear File List"] && ([fileList numFiles] == 0 || [showAdvancedArrow state] == NSOffState))
		return NO;

    return YES;

}


-(void) openRecentMenuItemSelected: (id)sender
{
	BOOL	isDir = NO;

	if ([[NSFileManager defaultManager] fileExistsAtPath: [sender title] isDirectory: &isDir] && isDir == NO)
		[self loadScript: [sender title]];
	else
		[STUtil alert:@"Invalid item" subText: @"The file you selected no longer exists at the specified path."];
}

-(void) constructOpenRecentMenu
{
	int i;

	//clear out all menu itesm
	while ([openRecentMenu numberOfItems])
		[openRecentMenu removeItemAtIndex: 0];

	if ([recentItems count] > 0)
	{
		//populate with contents of array
		for (i = [recentItems count]-1; i >= 0 ; i--)
		{
			[openRecentMenu addItemWithTitle: [recentItems objectAtIndex: i] action: @selector(openRecentMenuItemSelected:) keyEquivalent:@""];
		}
		
		//add seperator and clear menu
		[openRecentMenu addItem: [NSMenuItem separatorItem]];
		[openRecentMenu addItemWithTitle: @"Clear" action: @selector(clearRecentItems) keyEquivalent:@""];
		
	}
	else
	{
		[openRecentMenu addItemWithTitle: @"Empty" action: NULL keyEquivalent:@""];
		[[openRecentMenu itemAtIndex: 0] setEnabled: NO];
	}
}

-(void) clearRecentItems
{
	[recentItems removeAllObjects];
	[self constructOpenRecentMenu];
}

#pragma mark -


/*****************************************
 - called when + button is pressed
*****************************************/

- (IBAction)addFileToFileList:(id)sender
{
	int result;
	
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];
	[oPanel setTitle: @"Add File To Bundle"];
	[oPanel setCanChooseDirectories:YES];
    [oPanel setAllowsMultipleSelection:YES];
    result = [oPanel runModalForDirectory:nil file:nil types:nil];
    if (result == NSOKButton) 
	{
		//add all the files to the file list
        NSArray *filesToOpen = [oPanel filenames];
        int i, count = [filesToOpen count];
        for (i=0; i<count; i++) 
		{
			//we don't add if it already exists in the array
			if ([fileList hasFile: [filesToOpen objectAtIndex:i]] == NO)
				[fileList addFile: [filesToOpen objectAtIndex:i]];
        }
		//reload data
		[fileListDataBrowser setDataSource: fileList];
		[fileListDataBrowser reloadData];
		//update button status
		[self tableViewSelectionDidChange: NULL];
		[self updateFileSizeField];
    }
}

/*****************************************
 - called when C button is pressed
*****************************************/

- (IBAction)clearFileList:(id)sender
{
	[fileList clearList];
	[fileListDataBrowser reloadData];
	//update button status
	[self tableViewSelectionDidChange: NULL];
	[self updateFileSizeField];
}


/*****************************************
 - called when R button is pressed
*****************************************/
- (IBAction)revealFileInFileList:(id)sender
{	
	int i;
	NSIndexSet *selectedItems = [fileListDataBrowser selectedRowIndexes];
	
	for (i = 0; i < [fileList numFiles]; i++)
	{
		if ([selectedItems containsIndex: i])
		{
			[fileList revealInFinder: i];
		}
	}
}

/*****************************************
 - called when - button is pressed
*****************************************/

- (IBAction)removeFileFromFileList:(id)sender
{
	int i, didRemove = FALSE;
	NSIndexSet *selectedItems = [fileListDataBrowser selectedRowIndexes];
	
	for (i = [fileList numFiles]; i >= 0; i--)
	{
		if ([selectedItems containsIndex: i])
		{
			[fileList removeFile: i];
			didRemove = TRUE;
		}
	}
	
	if (didRemove == TRUE)
		[fileListDataBrowser reloadData];
	
	//update size field
	[self updateFileSizeField];
}


#pragma mark -

/*****************************************
 - called when [+] button is pressed in Types List
*****************************************/

- (IBAction) addType:(id)sender;
{
	if (![typesList hasType: [suffixTextField stringValue]] && ([[suffixTextField stringValue] length] > 0))
	{
		[typesList addType: [suffixTextField stringValue]];
		[suffixTextField setStringValue: @""];
	}
	//update
	[typesListDataBrowser setDataSource: typesList];
	[typesListDataBrowser reloadData];
	
	if ([typesList hasAllTypes])
		[numTypesTextField setStringValue: @"- All types"];
	else
		[numTypesTextField setStringValue: [NSString stringWithFormat:@"- %d types", [typesList numTypes]]];
}

/*****************************************
 - called when [-] button is pressed in Types List
*****************************************/

- (IBAction) removeType:(id)sender;
{
	int i;
	NSIndexSet *selectedItems = [typesListDataBrowser selectedRowIndexes];
	
	for (i = [typesList numTypes]; i >= 0; i--)
	{
		if ([selectedItems containsIndex: i])
		{
			[typesList removeType: i];
			[typesListDataBrowser reloadData];
			break;
		}
	}
	
	if ([typesList hasAllTypes])
		[numTypesTextField setStringValue: @"- All types"];
	else
		[numTypesTextField setStringValue: [NSString stringWithFormat:@"- %d types", [typesList numTypes]]];
}

/*****************************************
 - called when [C] button is pressed in Types List
*****************************************/
- (IBAction) clearTypesList:(id)sender
{
	[typesList clearList];
	[typesListDataBrowser reloadData];
	[numTypesTextField setStringValue: [NSString stringWithFormat:@"- %d types", [typesList numTypes]]];
}

/*****************************************
 - called when "Default" button is pressed in Types List
*****************************************/
- (IBAction) setDefaultTypes:(id)sender
{
	[typesList clearList];
	[typesList addType: @"*"];
	[typesListDataBrowser reloadData];
	
	if ([typesList hasAllTypes])
		[numTypesTextField setStringValue: @"- All types"];
	else
		[numTypesTextField setStringValue: [NSString stringWithFormat:@"- %d types", [typesList numTypes]]];
}

#pragma mark -
- (IBAction) showHelp:(id)sender
{
	NSString	*thePath = [[NSBundle mainBundle] pathForResource:@"index.html" ofType:nil];
	[NSTask launchedTaskWithLaunchPath: @"/usr/bin/open" arguments: [NSArray arrayWithObject: thePath]];
}

@end
