#import "ScriptExecController.h"
#import "STUtil.h"
#include <Security/Authorization.h>
#include <sys/wait.h>

@implementation ScriptExecController

- (id)init
{
	if (self = [super init]) 
	{
		//create arrays
		arguments = [[NSMutableArray alloc] initWithCapacity: kMaxFileArguments+2];
		fileArgs = [[NSMutableArray alloc] initWithCapacity: kMaxFileArguments];
		isTaskDone = NO;
		isTaskStarted = NO;
    }
    return self;
}

-(void)dealloc
{
	[arguments release];
	[fileArgs release];
}

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
	//load settings from app bundle
	[self loadSettings];
	
	//put application name into the relevant menu items
	[quitMenuItem setTitle: [NSString stringWithFormat: @"Quit %@", appName]];
	[aboutMenuItem setTitle: [NSString stringWithFormat: @"About %@", appName]];
	[hideMenuItem setTitle: [NSString stringWithFormat: @"Hide %@", appName]];
	
	//create progress window if so specified
	if (outputType == kProgressBarOutput)
	{
		//prepare progress bar, then center and display window
		[progressBar setUsesThreadedAnimation: YES];
		[progressBar startAnimation: self];
		[progressWindow setTitle: appName];
		[progressWindow center];
		[progressWindow makeKeyAndOrderFront: NULL];
	}
	else if (outputType == kTextOutput)
	{   
		[textOutputTextField setFont:[NSFont userFixedPitchFontOfSize:0.0]];
		[textOutputProgressIndicator setUsesThreadedAnimation: YES];
		[textOutputProgressIndicator startAnimation: self];
		[textOutputWindow setTitle: appName];
		[textOutputWindow center];
		[textOutputWindow makeKeyAndOrderFront: NULL];
	}

	//create argument list
	[arguments addObject: scriptPath];
	[arguments addObject: [[NSBundle mainBundle] bundlePath]]; //$1 is path of application bundle
	if ([fileArgs count] > 0)//if there are any dropped files, we add them as arguments after $1
		[arguments addObjectsFromArray: fileArgs];

	//start new thread for executing script
	if (execStyle == kPrivilegedExecution) //Authentication mode
	{
		[NSThread detachNewThreadSelector: @selector(executeScriptWithPrivileges) toTarget: self withObject: NULL];
	}
	else //plain old regular
	{
		//[NSThread detachNewThreadSelector: @selector(executeScript) toTarget: self withObject: NULL];
		[self executeScript];
	}
}

#pragma mark -

//launch task with NSTask
- (void)executeScript
{	
	isTaskStarted = YES;

	//initalize task
	task = [[NSTask alloc] init];
	
	if (outputType == kTextOutput)
	{
		outputPipe = [NSPipe pipe];
		[task setStandardOutput: outputPipe];
		[task setStandardError: outputPipe];
		readHandle = [outputPipe fileHandleForReading];
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(getData:) name:NSFileHandleReadCompletionNotification object:readHandle];
		[readHandle readInBackgroundAndNotify];
	}
	
	//apply settings for task
	[task setLaunchPath: interpreter];
	[task setArguments: arguments];
	
	//set it off
	[task launch];

	[NSTimer scheduledTimerWithTimeInterval: 0.25 target: self selector:@selector(checkTaskStatus) userInfo: nil repeats: YES];
}

- (void)checkTaskStatus
{
	if (![task isRunning])
	{
		[self taskFinished];
	}
}

- (void)taskFinished
{
		if (!remainRunning)
		{
			[[NSApplication sharedApplication] terminate: self];
		}
		else if (outputType == kTextOutput)
		{
			[textOutputCancelButton setTitle: @"Quit"];
			[textOutputCancelButton setKeyEquivalent:@"\r"];
			[textOutputProgressIndicator stopAnimation: self];
		}
		else if (outputType == kProgressBarOutput)
		{
			[messageTextField setStringValue: @"Task completed"];
			[progressBar stopAnimation: self];
			[cancelButton setTitle: @"Quit"];
			[cancelButton setKeyEquivalent:@"\r"];
		}
		
		isTaskDone = YES;
}

- (void) getData: (NSNotification *)aNotification
{
	NSData *data;
	id notificationObject;

	//get the data
	notificationObject = [aNotification object];
	data = [[aNotification userInfo] objectForKey:NSFileHandleNotificationDataItem];
	
	//make sure there's actual data
	if ([data length]) 
	{
		//append the output
		outputStr = [[[NSString alloc] initWithData: data encoding: NSASCIIStringEncoding] autorelease];
		[textOutputTextField setString: [[textOutputTextField string] stringByAppendingString: outputStr] ];
		//scroll to it
		[textOutputTextField scrollRangeToVisible:NSMakeRange([[textOutputTextField string] length], 0)];
		
		// we schedule the file handle to go read more data in the background again.
		[notificationObject readInBackgroundAndNotify];
	}
}

#pragma mark -


//launch task using Authentication Manager
- (void)executeScriptWithPrivileges
{
	OSErr					err = noErr;
    AuthorizationRef 		authorizationRef;
    short					i;
    char					*args[kMaxFileArguments+2];
	char					interpreterStr[kMaxPathLength];
	NSAutoreleasePool		*pool = [[NSAutoreleasePool alloc] init];
	
	isTaskStarted = YES;
	
	//interpreter
	[interpreter getCString: (char *)&interpreterStr maxLength: kMaxPathLength];
	
	//arguments array
	for (i = 0; i < [arguments count]; i++)
	{
		args[i] = malloc(kMaxPathLength);
		[[arguments objectAtIndex:i] getCString: (char *)args[i] maxLength: kMaxPathLength];
		if (i == [arguments count]-1)
		{
			args[i+1] = NULL;
		}
	}
    
    // Use Apple's Authentication Manager APIs to run script with root privileges
    err = AuthorizationCreate(NULL, kAuthorizationEmptyEnvironment, kAuthorizationFlagDefaults, &authorizationRef);
    if (err != errAuthorizationSuccess)
	{
		NSLog(@"Authorization for script execution failed - Error %d", err);
        [[NSApplication sharedApplication] terminate: self];
	}
	
    if (!(err = AuthorizationExecuteWithPrivileges(authorizationRef,(char *)&interpreterStr, kAuthorizationFlagDefaults, args, nil)) != noErr)
	{
            //wait for child process to finish
            int child;
            wait(&child);
			
			//release memory for arguments
			for (i = 0; i < [arguments count]; i++)
				free(args[i]);
			
			[self taskFinished];
			[pool release];
    }
	else
	{
		NSLog(@"Error %d occurred when attempting to run AuthorizationExecuteWithPrivileges. Terminating...", err);
		[pool release];
		[[NSApplication sharedApplication] terminate: self];
	}
}

#pragma mark -

//respond to AEOpenDoc
- (BOOL)application:(NSApplication *)theApplication openFile:(NSString *)filename
{
	if (!isTaskStarted)
	{
		[fileArgs addObject: filename];
		return TRUE;
	}
	else
		return FALSE;
}

- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender
{
	//terminate task
	if (task != NULL) //&& [task isRunning])
	{
		if ([task isRunning])
			[task terminate];
		[task release];
	}
		
	return(YES);
}

#pragma mark -

//load configuration files from application bundle
- (void)loadSettings
{
	NSBundle		*appBundle = [NSBundle mainBundle];
	NSFileManager   *fmgr = [NSFileManager defaultManager];
	NSDictionary	*appSettingsPlist;
	
	//make sure all the config files are present
	if (![fmgr fileExistsAtPath: [appBundle pathForResource:@"AppSettings.plist" ofType:nil]] ||
		![fmgr fileExistsAtPath: [appBundle pathForResource:@"script" ofType:nil]])
	{
		[STUtil fatalAlert: @"Corrupted app bundle" subText: @"Some configuration files were missing from the application bundle."];
	}
	
	//get app name
	appName = [[[NSBundle mainBundle] executablePath] lastPathComponent];
	
	//get dictionary with app settings
	appSettingsPlist = [NSDictionary dictionaryWithContentsOfFile: [appBundle pathForResource:@"AppSettings.plist" ofType:nil]];
	
	//get interpreter
	interpreter = [appSettingsPlist objectForKey:@"ScriptInterpreter"];
		
	//get path to script
	scriptPath = [appBundle pathForResource:@"script" ofType:nil];
	
	//determine output type
	NSString *outputTypeStr = [appSettingsPlist objectForKey:@"OutputType"];
	if ([outputTypeStr isEqualToString: @"Progress Bar"])
		outputType = kProgressBarOutput;
	else if ([outputTypeStr isEqualToString: @"Text Window"])
		outputType = kTextOutput;
	else
		outputType = kNoOutput;
		
	//determine execution style
	execStyle = [[appSettingsPlist objectForKey:@"RequiresAdminPrivileges"] boolValue];

	//is droppable?
	//isDroppable = [[appSettingsPlist objectForKey:@"IsDroppable"] boolValue];
	
	//is droppable?
	remainRunning = [[appSettingsPlist objectForKey:@"RemainRunningAfterCompletion"] boolValue];
	
	//make sure there are no conflicting settings
	if (execStyle && outputType == kTextOutput)
	{
		[STUtil fatalAlert: @"Invalid Application Settings" subText: @"This Platypus application contains invalid application settings and thus cannot run."];
	}
}



// Respond to Cancel by exiting application
- (IBAction)cancel:(id)sender
{
	[[NSApplication sharedApplication] terminate: self];
}




@end
