//their stuff
#include <stdio.h>
#include <unistd.h>
#include <Carbon/Carbon.h>
#include <string.h>
#include <fcntl.h>
#include <string.h>

//my stuff
#include "platypus.h"





// Platypus windows and controls
WindowRef	platypusWindow, aboutWindow, prefsWindow;
ControlRef	appNameTextField, scriptPathTextField, typeRadioButtons, iconSelectIncrementor, appIcon, iconNameTextField;

//menus
Handle		menuBar;

// paths needed
char		execPath[kMaxPathLength] = "\0";
char		resourcesPath[kMaxPathLength] = "\0";

char		customIconPath[kMaxPathLength] = "\0";
char		customIconName[kMaxFileNameLength] = "\0";

// icons
short		currentIcon = 0;
PlatypusIconStruct	platypusIcons[9];


/*************************************
program start
*************************************/
int main(int argc, char* argv[])
{
	#pragma unused (argc)

    OSErr		err;

    // Get path to application resource from path to executable passed
    // as first argument.
    err = InitPaths(argv[0]);
    if (err)
        RedFatalAlert("\pInitialization error", "\pAn error occurred when getting resource paths.");

    err = InitIcons();
    if (err)
        RedFatalAlert("\pInitialization error", "\pAn error occurred when loading icons.");

    // Create window and menu bar from nib file
    err = InitNib(&platypusWindow);
    if (err)
        RedFatalAlert("\pInitialization error", "\pAn error occurred when loading window from Nib file.");
    
    // Get references to window controls
    err = InitControls();
    if (err)
        RedFatalAlert("\pInitialization error", "\pAn error occurred when getting control references.");
    
    // Install application event handlers
    err = InitHandlers();
    if (err)
        RedFatalAlert("\pInitialization error", "\pAn error occurred when initing application event handlers.");
    
    // Display
    NewPlatypusWindow(platypusWindow);
    
    // Run the event loop
    RunApplicationEventLoop();

    return 0;
}


/*************************************
Quit application
*************************************/
void QuitApp (void)
{
    ExitToShell();

}



#pragma mark -

/*************************************
Install all event handlers
*************************************/
OSErr InitHandlers (void)
{
    OSErr	err = noErr;

    const EventTypeSpec	commandEvents[] = { {kEventClassCommand, kEventProcessCommand} };
    const EventTypeSpec windowEvents[] = { {kEventClassWindow, kEventWindowClose}  };
    
    //command handler -- control events and menu events -- target is application
    
    //command handler
    err = InstallEventHandler(GetApplicationEventTarget(), (EventHandlerUPP)NewEventHandlerUPP(PlatypusCommandHandler), 1, commandEvents, NULL, NULL );
    
    //main window handler
    err += InstallWindowEventHandler(platypusWindow, (EventHandlerUPP)NewEventHandlerUPP(PlatypusWindowHandler), 1, windowEvents, NULL, NULL );
    
    //prefs and about windows
    err += InstallWindowEventHandler(aboutWindow, (EventHandlerUPP)NewEventHandlerUPP(PlatypusAboutWindowHandler), 1, windowEvents, (void *) aboutWindow , NULL );
    //err += InstallWindowEventHandler(prefsWindow, (EventHandlerUPP)NewEventHandlerUPP(PlatypusAboutWindowHandler), 1, windowEvents, (void *) prefsWindow, NULL );
    
    //AE Handlers
	err += AEInstallEventHandler( kCoreEventClass, kAEReopenApplication, NewAEEventHandlerUPP((AEEventHandlerProcPtr)PlatypusReopenAEHandler), 0, false );
	err += AEInstallEventHandler( kCoreEventClass, kAEOpenDocuments, NewAEEventHandlerUPP((AEEventHandlerProcPtr)PlatypusOpenDocAEHandler), 0, false );
	
	//Drag
	err += InstallReceiveHandler((DragReceiveHandlerUPP )NewDragReceiveHandlerUPP(PlatypusDragHandler), platypusWindow, NULL);
	
    return err;
}

/*************************************
Generate strings containing paths to
the components in the application package
required for creating the script app
*************************************/
OSErr InitPaths (char *argvZero)
{
    char	contentsPath[kMaxPathLength];
    
    if (argvZero == NULL)
        return true;

    //get path to Contents folder within application bundle
    strcpy((char *)&contentsPath, (char *)argvZero);
    CutFileFromPath((char *)&contentsPath);
    CutFileFromPath((char *)&contentsPath);
    
    //get path to executables i.e. MacOS folder
    strcpy((char *)&execPath, (char *)&contentsPath);
    strcat((char *)&execPath, (char *)&"/MacOS/");
    
    //get path to Resources folder
    strcpy((char *)&resourcesPath, (char *)&contentsPath);
    strcat((char *)&resourcesPath, (char *)&"/Resources/");
    
    return noErr;
}

/*************************************
Load window and menu bar from Nib file
*************************************/
OSErr InitNib (WindowRef *window)
{
    OSErr		err;
    IBNibRef 		nibRef;
    
    err = CreateNibReference(CFSTR("main"), &nibRef);
    if (err)
        return err;
    
    err += CreateMenuBarFromNib(nibRef, CFSTR("MenuBar"), &menuBar);
    if (err)
    	return err;

    //No prefs yet
    //EnableMenuCommand(NULL, kHICommandPreferences);
    
    SetMenuBar(menuBar);
   	SetMenuStatus();
    
    
    err = CreateWindowFromNib(nibRef, CFSTR("MainWindow"), window);
    if (err)
        return err;
    
    err = CreateWindowFromNib(nibRef, CFSTR("AboutWindow"), &aboutWindow);
    if (err)
        return err;
   	
   /*	err = CreateWindowFromNib(nibRef, CFSTR("PrefsWindow"), &prefsWindow);
    if (err)
        return err;*/

    // We don't need the nib reference anymore.
    DisposeNibReference(nibRef);

    return noErr;
}




/*************************************
Get references to window controls
*************************************/
OSErr InitControls (void)
{
    ControlID		controlID;
    
        //app name text field
        controlID.signature = 'Text';
        controlID.id = 0;
        GetControlByID (platypusWindow, &controlID, &appNameTextField);
        
        //script path text field
        controlID.signature = 'Text';
        controlID.id = 1;
        GetControlByID (platypusWindow, &controlID, &scriptPathTextField);

		//icon name text field
		controlID.signature = 'IcNm';
        controlID.id = 0;
        GetControlByID (platypusWindow, &controlID, &iconNameTextField);
		
		//icon image well
		controlID.signature = 'Icon';
        controlID.id = 0;
        GetControlByID (platypusWindow, &controlID, &appIcon);
        
        //get reference to type radio buttons
        controlID.signature = 'Type';
        controlID.id = 0;
        GetControlByID (platypusWindow, &controlID, &typeRadioButtons);
        
 	return noErr;
}

/*************************************
Load shipping icons from icns files
*************************************/
OSErr InitIcons (void)
{
	// Generic Application
	GetIconRef(kOnSystemDisk, 'macs', kGenericApplicationIcon, &platypusIcons[0].icon);
	strcpy((char *)&platypusIcons[0].iconName, (char *)&"Generic App");
	
	// Hexley Fork Icon
	platypusIcons[1].icon = RegisterIcon( 'pIcn', 'pIc1', "HexleyFork.icns" );
	strcpy((char *)&platypusIcons[1].iconName, (char *)&"Hexley Fork");
	strcpy((char *)&platypusIcons[1].fileName, (char *)&"HexleyFork.icns");
	
	// Hexley App Icon
	platypusIcons[2].icon = RegisterIcon( 'pIcn', 'pIc2', "HexleyApp.icns" );
	strcpy((char *)&platypusIcons[2].iconName, (char *)&"Hexley App");
	strcpy((char *)&platypusIcons[2].fileName, (char *)&"HexleyApp.icns");
	
	// Default Platypus App
	platypusIcons[3].icon = RegisterIcon( 'pIcn', 'pIc3', "platypusDefault.icns" );
	strcpy((char *)&platypusIcons[3].iconName, (char *)&"Platypus Default");
	strcpy((char *)&platypusIcons[3].fileName, (char *)&"platypusDefault.icns");
	
	// Perl Camel
	platypusIcons[4].icon = RegisterIcon( 'pIcn', 'pIc4', "PerlCamel.icns" );
	strcpy((char *)&platypusIcons[4].iconName, (char *)&"Perl Camel");
	strcpy((char *)&platypusIcons[4].fileName, (char *)&"PerlCamel.icns");
	
	// Python Snake Icon
	platypusIcons[5].icon = RegisterIcon( 'pIcn', 'pIc7', "PythonSnake.icns" );
	strcpy((char *)&platypusIcons[5].iconName, (char *)&"Python Snake");
	strcpy((char *)&platypusIcons[5].fileName, (char *)&"PythonSnake.icns");

	// Ruby Goat
	platypusIcons[6].icon = RegisterIcon( 'pIcn', 'pIc5', "RubyGoat.icns" );
	strcpy((char *)&platypusIcons[6].iconName, (char *)&"Ruby Goat");
	strcpy((char *)&platypusIcons[6].fileName, (char *)&"RubyGoat.icns");
	
	// AppleScript icon
	platypusIcons[7].icon = RegisterIcon( 'pIcn', 'pIc6', "AppleScript.icns" );
	strcpy((char *)&platypusIcons[7].iconName, (char *)&"AppleScript Icon");
	strcpy((char *)&platypusIcons[7].fileName, (char *)&"AppleScript.icns");
	
	// Tcl Flamingo
	platypusIcons[8].icon = RegisterIcon( 'pIcn', 'pIc8', "TclFlamingo.icns" );
	strcpy((char *)&platypusIcons[8].iconName, (char *)&"Tcl Flamingo");
	strcpy((char *)&platypusIcons[8].fileName, (char *)&"TclFlamingo.icns");
	
	
	// Default Platypus App
	// For About Window
	RegisterIcon( 'pIcn', 'Plat', "platypus.icns" );

    return noErr;
}


#pragma mark -



/*************************************
Set controls to default and show main window
*************************************/

OSErr NewPlatypusWindow (WindowRef window)
{
	//no type radio button selected
	SetControlValue(typeRadioButtons, 0);
	
	//set default app name
    SetControlData(appNameTextField, kControlNoPart, kControlEditTextTextTag, strlen(kDefaultAppName), kDefaultAppName);
    
    //clear script path field
    SetControlData(scriptPathTextField, kControlNoPart, kControlEditTextTextTag, 0, "");
    
    //select default icon
    SetIcon(kDefaultIcon);
    
    //reveal the window
    ShowWindow(window);
    SelectWindow(window);
    
    //update menus
    SetMenuStatus();
    
    return noErr;

}

/*************************************
Hide main window
*************************************/
void ClosePlatypusWindow (WindowRef window)
{
    HideWindow(window);
    SetMenuStatus();
}


#pragma mark -


/*************************************
Take data entered and create app with it
*************************************/
OSErr CreateScriptApplication (void)
{
	short		scriptType = 0;
	OSErr		err = noErr;
	char		command[4096] = "\0";
    char		appName[kMaxFileNameLength] = "\0";
    static char	appPath[kMaxPathLength] = "\0";
    char		scriptPath[kMaxPathLength] = "\0";
    Str255		pascalStr;
    
    

	//get path to script
    err = GetControlData (scriptPathTextField, kControlEntireControl, kControlEditTextTextTag, sizeof(scriptPath), &scriptPath, NULL);
	if (err)
	{
		RedAlert("\pControl Error", "\pError retrieving field value from editable text field scriptPathTextField.");
        return true;
	}

	//check if script path is valid
    if (access(scriptPath, F_OK) == -1)
    {
        YellowAlert("\pInvalid Script Path", "\pNo file exists at the specified path.");
        return true;
    }
    if (UnixIsFolder((char *)&scriptPath))
    {
    	YellowAlert("\pInvalid Script Path", "\pSpecified path points to a folder.");
        return true;
    }
    

	//get app name
    err = GetControlData (appNameTextField, kControlEntireControl, kControlEditTextTextTag, sizeof(appName), &appName, NULL);
    if (err)
	{
		RedAlert("\pControl Error", "\pError retrieving field value from editable text field appNameTextField.");
        return true;
	}
    if (strlen((char *)&appName) == 0 || strlen((char *)&appName) > kMaxFileNameLength)
    {
    	YellowAlert("\pInvalid Name", "\pYou must specify a name for the application to be created.");
        return true;
    }
	CopyCStringToPascal ((char *)&appName, pascalStr);
	
	// check suffix and warn if it isn't appropriate
	scriptType = GetControlValue(typeRadioButtons);
	
	if (scriptType == 0) // no script type set
	{
		YellowAlert("\pScript type not set", "\pYou must specify what kind of script is being used.");
        return true;
	}
	
	switch(scriptType)
	{
		case kShellScriptType:
			if (GetScriptType((char *)&scriptPath) != kShellScriptType)
			{
				YellowAlert("\pWarning", "\pThe script you specified does not appear to be a shell script.  This may result in a dysfunctional application.");
			}
			break;
		case kPerlScriptType:
			if (GetScriptType((char *)&scriptPath) != kPerlScriptType)
			{
				YellowAlert("\pWarning", "\pThe script you specified does not appear to be a Perl script.  This may result in a dysfunctional application.");
			}
			break;
		case kPythonScriptType:
			if (GetScriptType((char *)&scriptPath) != kPythonScriptType)
			{
				YellowAlert("\pWarning", "\pThe script you specified does not appear to be a Python script.  This may result in a dysfunctional application.");
			}
			break;
	}
	
	//get path to the app to be created
	err = NavigationPutPath(pascalStr, "\pSelect place to create app:", (char *)&appPath);
	if (err)
		return true;
	
	//get name from the path
	strcpy((char *)&appName, GetFileNameFromPath((char *)&appPath));
	CutSuffix((char *)&appName);
	
	//get path to the folder that is to contain the app
	
	CutFileFromPath((char *)&appPath);
	
	
	
	
	///////////// Generate command ///////////////
	
	
	//ok, add the path to the perl script that creates the app with perl defined as an interpreter
    strcpy((char *)&command, (char *)"/usr/bin/perl '");
    strcat((char *)&command, (char *)&execPath);
    strcat((char *)&command, (char *)"createapp.pl' ");
    
    //add the arguments
    
    // 1) Root
    strcat((char *)&command, (char *)"'");
    strcat((char *)&command, (char *)&appPath);
    strcat((char *)&command, (char *)"'");
    strcat((char *)&command, (char *)" ");
    
    // 2)
    strcat((char *)&command, (char *)"'");
    strcat((char *)&command, (char *)&appName);
    strcat((char *)&command, (char *)"'");
    strcat((char *)&command, (char *)" ");
    
    // 3)
    strcat((char *)&command, (char *)"'");
    strcat((char *)&command, (char *)&scriptPath);
    strcat((char *)&command, (char *)"'");
    strcat((char *)&command, (char *)" ");
    
    // 4)
    strcat((char *)&command, (char *)"'");
    strcat((char *)&command, (char *)&execPath);
    
    //what executable we copy to the new application bundle depends on
    // what kind of script is to be executed by it
    switch(scriptType)
    {
    	case kShellScriptType:
    		strcat((char *)&command, (char *)"shexec' ");
			break;
		case kPerlScriptType:
			strcat((char *)&command, (char *)"perlexec' ");
			break;
		case kPythonScriptType:
			strcat((char *)&command, (char *)"pythonexec' ");
			break;
		case kRubyScriptType:
			strcat((char *)&command, (char *)"rubyexec' ");
			break;
		case kAppleScriptType:
			strcat((char *)&command, (char *)"applescriptexec' ");
			break;
		case kTclScriptType:
			strcat((char *)&command, (char *)"tclexec' ");
			break;	
    }
    strcat((char *)&command, (char *)" ");
    
    if (currentIcon == kCustomIcon)
    {
    	// 5)
	    strcat((char *)&command, (char *)"'");
	    strcat((char *)&command, (char *)&customIconPath);
	    strcat((char *)&command, (char *)"'");
	    strcat((char *)&command, (char *)" ");
	    
	    // 6)
	    strcat((char *)&command, (char *)"'");
	    strcat((char *)&command, (char *)&customIconName);
	    strcat((char *)&command, (char *)"'");

    }
    else
    {
	    // 5)
	    strcat((char *)&command, (char *)"'");
	    strcat((char *)&command, (char *)&resourcesPath);
	    strcat((char *)&command, (char *)"'");
	    strcat((char *)&command, (char *)" ");
	    
	    // 6)
	    strcat((char *)&command, (char *)"'");
	    strcat((char *)&command, (char *)&platypusIcons[currentIcon].fileName);
	    strcat((char *)&command, (char *)"'");
	    
    }
    
    system((char *)&command);
    
    return noErr;
}








/*




OSErr CreateScriptApplicationFromFSSpec (FSSpec fileSpec)
{
	char		command[4096] = "\0";
	
	char		appName[kMaxFileNameLength] = "\0";
    char		appPath[kMaxPathLength] = "\0";
    char		scriptPath[kMaxPathLength] = "\0";
    
    char		str;
    
    short		scriptType;
    
    OSErr		err = noErr;



	//get path to script
	err = FSMakePath(fileSpec, (unsigned char *)&scriptPath, kMaxPathLength);
	if (err)
		return err;
	
	//get path to directory in which the script resides -- which incidentally is also the path where we create the app
	strcpy((char *)&appPath, (char *)&scriptPath);
	CutFileFromPath((char *)&appPath);

	// get name of app, which is the same as that of the script, minus the suffix in question
	strcpy((char *)&str, (char *)&scriptPath);
	strcpy((char *)&appName, (char *)GetFileNameFromPath(CutSuffix((char *)&str)));

	// get script type
	scriptType = GetScriptType((char *)&scriptPath);
	if (scriptType == 0) //no type could be discerned from script filename
	{
		return true;
	}
	
	
	///////////// Generate command ///////////////
	
	
	//ok, add the path to the perl script that creates the app with perl defined as an interpreter
    strcpy((char *)&command, (char *)"/usr/bin/perl '");
    strcat((char *)&command, (char *)&execPath);
    strcat((char *)&command, (char *)"createapp.pl' ");
    
    //add the arguments
    
    // 1) Root
    strcat((char *)&command, (char *)"'");
    strcat((char *)&command, (char *)&appPath);
    strcat((char *)&command, (char *)"'");
    strcat((char *)&command, (char *)" ");
    
    // 2)
    strcat((char *)&command, (char *)"'");
    strcat((char *)&command, (char *)&appName);
    strcat((char *)&command, (char *)"'");
    strcat((char *)&command, (char *)" ");
    
    // 3)
    strcat((char *)&command, (char *)"'");
    strcat((char *)&command, (char *)&scriptPath);
    strcat((char *)&command, (char *)"'");
    strcat((char *)&command, (char *)" ");
    
    // 4)
    strcat((char *)&command, (char *)"'");
    strcat((char *)&command, (char *)&execPath);
    
    //what executable we copy to the new application bundle depends on
    // what kind of script is to be executed by it
    switch(scriptType)
    {
    	case kShellScriptType:
    		strcat((char *)&command, (char *)"shexec' ");
			break;
		case kPerlScriptType:
			strcat((char *)&command, (char *)"perlexec' ");
			break;
		case kPythonScriptType:
			strcat((char *)&command, (char *)"pythonexec' ");
			break;
    }
    strcat((char *)&command, (char *)" ");
    
    // 5)
    strcat((char *)&command, (char *)"'");
    strcat((char *)&command, (char *)&resourcesPath);
    strcat((char *)&command, (char *)"'");
    strcat((char *)&command, (char *)" ");
    
    // 6)
    strcat((char *)&command, (char *)"'");//use the default icon
    strcat((char *)&command, (char *)&platypusIcons[0].fileName);
    strcat((char *)&command, (char *)"'");
    
    system((char *)&command);

	return noErr;
}
*/


#pragma mark -

/*************************************
Switch icon
*************************************/
OSErr SetIcon (short iconNum)
{
	OSErr	err = noErr;
	ControlButtonContentInfo contentInfo;

	currentIcon = iconNum;
	SetWindowProxyIcon(platypusWindow, platypusIcons[iconNum].icon);
	err = SetControlData(iconNameTextField, kControlNoPart, kControlEditTextTextTag, strlen((char *)&platypusIcons[iconNum].iconName), &platypusIcons[iconNum].iconName);
	
	contentInfo.contentType = kControlContentIconRef;
	contentInfo.u.iconRef = platypusIcons[iconNum].icon;
	
	err = SetControlData(appIcon, kControlNoPart, kControlImageWellContentTag , sizeof(contentInfo), &contentInfo);
	
	Draw1Control(appIcon);
	Draw1Control(iconNameTextField);
	 
	
	return noErr;
}

/*************************************
Set custom icon
*************************************/
OSErr SetCustomIcon (char *path, FSSpec fileSpec)
{
	OSErr						err = noErr;
	ControlButtonContentInfo 	contentInfo;
	IconRef						iconRef;
				
	//get icon reference from file
	err = RegisterIconRefFromIconFile('pIcn', 'Cust', &fileSpec, &iconRef);
	if (err)
		goto Err;
				
	//set it in the image well
	contentInfo.contentType = kControlContentIconRef;
	contentInfo.u.iconRef = iconRef;
	err = SetControlData(appIcon, kControlNoPart, kControlImageWellContentTag , sizeof(contentInfo), &contentInfo);
	if (err)
		goto Err;
	err = SetControlData(iconNameTextField, kControlNoPart, kControlEditTextTextTag, strlen((char *)&"Custom Icon"), "Custom Icon");
	if (err)
		goto Err;
	
	currentIcon = kCustomIcon;
	
	//get path to directory which contains the icon
	strcpy((char *)&customIconPath, path);
	CutFileFromPath((char *)&customIconPath);
	
	//get name of icon file
	strcpy((char *)&customIconName, GetFileNameFromPath(path));

	Draw1Control(appIcon);
	Draw1Control(iconNameTextField);
	
	return noErr;
	
	Err:
		if (currentIcon == kCustomIcon)
			return err;
		SetIcon(currentIcon);
		return err;
}

/*************************************
Enable/disable menu items appropriately
*************************************/
void SetMenuStatus (void)
{
   	if (IsWindowVisible(platypusWindow))
    {
        EnableMenuCommand((MenuRef)(MenuHandle *)(**menuBar), kHICommandClose);
        DisableMenuCommand((MenuRef)(MenuHandle *)(**menuBar), kHICommandNew);
    }
    else
    {
		DisableMenuCommand((MenuRef)(MenuHandle *)(**menuBar), kHICommandClose);
        EnableMenuCommand((MenuRef)(MenuHandle *)(**menuBar), kHICommandNew);
    }

}

#pragma mark -

/*************************************
Get path to a file via Navigation dialog
*************************************/

OSErr NavigationGetPath (char *path)
{
    FSSpec			fileSpec;
    OSErr			err;

    err = NavigationGetFSSpec(&fileSpec);
    if (err == userCanceledErr)
    {
       	return true;
    }
    else if (err)
    {
        RedAlert("\pNavigation Error", "\pFunction NavigationGetFSSpec() returned an error when called from function NavigationGetPath().");
        return true;
    }
    
    err = FSMakePath(fileSpec, (unsigned char *)path, kMaxPathLength);
    

    return noErr;
}

/*************************************
Get path to a location via Nav dialog
*************************************/
OSErr NavigationPutPath (Str255 defaultName, Str255 promptMessage, char *path)
{
	OSErr			err;
	FSSpec			fileSpec;

	//Get the location in which the app is to be created
	
	err = NavigationPutFSSpec (promptMessage, defaultName, &fileSpec);
	if (err == userCanceledErr)
	{
            return true;
    }
    else if (err)
    {
    		RedAlert("\pNavigation Error", "\pError getting file specification from NavigationLib");
    		return true;
    }       
	
	//create dest file to get path to it, to fool Apple's stupid function
	err = FSpCreate(&fileSpec, 'Plat', 'Plat', smSystemScript);
		
	//get path in which to create app from FSSpec
	err = FSMakePath(fileSpec, (unsigned char *)path, kMaxPathLength);
	
	//delete teh placeholder file
	HDelete(fileSpec.vRefNum, fileSpec.parID, fileSpec.name);
	
	if (err)
	{
		RedAlert("\pPath error", "\pUnable to create path from FSSpec");
		return true;
	}

	return noErr;
}

/*************************************
Get filespec via navigation
*************************************/
OSErr NavigationGetFSSpec (FSSpec *file)
{
    NavDialogOptions    dialogOptions;
    NavObjectFilterUPP  filterProc = NewNavObjectFilterUPP(NavFilterProc);
    OSErr               anErr = noErr;
    
    //  Specify default options for dialog box
    anErr = NavGetDefaultDialogOptions(&dialogOptions);
    if (anErr == noErr)
    {
        // Adjust the options to fit our needs
        //Set default location option
        dialogOptions.dialogOptionFlags |= kNavSelectDefaultLocation;
        dialogOptions.dialogOptionFlags |= kNavAllFilesInPopup;
        //Clear preview option
        dialogOptions.dialogOptionFlags ^= kNavAllowPreviews;
        
        if (anErr == noErr)
        {
            NavReplyRecord reply;
            
            // Call NavGetFile() with specified options
            anErr = NavGetFile (NULL, &reply, &dialogOptions,NULL, nil, filterProc, NULL, nil);
            
            if (anErr == noErr && reply.validRecord)
            {
                // Deal with multiple file selection
                long    count;
                
                anErr = AECountItems(&(reply.selection), &count);
                // Set up index for file list
                if (anErr == noErr)
                {
                    long index;
                    
                    //loop through all the files selected and open them in turn
                    for (index = 1; index <= count; index++)
                    {
                        AEKeyword   theKeyword;
                        DescType    actualType;
                        Size        actualSize;
                        FSSpec      documentFSSpec;
                        
                        // Get a pointer to selected file
                        anErr = AEGetNthPtr(&(reply.selection), index,
                                            typeFSS, &theKeyword,
                                            &actualType,&documentFSSpec,
                                            sizeof(documentFSSpec),
                                            &actualSize);
                        if (anErr == noErr)
                        {
                            *file = documentFSSpec;
                        }
                    }
                }
                //  Dispose of NavReplyRecord, resources, descriptors
                anErr = NavDisposeReply(&reply);
            }
        }
    }

    //DisposeRoutineDescriptor(filterProc);
    return anErr;
}



/*************************************
Get file spec for placement
*************************************/

OSErr NavigationPutFSSpec (Str255 message, Str255 defaultName, FSSpec *theFile)
{
	OSErr				theErr = noErr;
	NavReplyRecord			theReply;
	NavDialogOptions		dialogOptions;

	CGrafPtr			savePort = NULL;
	GDHandle			saveDevice = NULL;
	
	GetGWorld(&savePort, &saveDevice);
	
	//set the Navigation dialog options
	theErr = NavGetDefaultDialogOptions(&dialogOptions);
	if(theErr)
		return theErr;
	
	dialogOptions.dialogOptionFlags -= kNavDontAddTranslateItems;
	dialogOptions.dialogOptionFlags += kNavNoTypePopup;
	
	//set the default name in editable text field
	BlockMove(defaultName, dialogOptions.savedFileName, sizeof(Str255));
	
	//same goes for message
	if(message != nil)
            BlockMove(message, dialogOptions.message, sizeof(Str255));
	else
            dialogOptions.message[0] = 0;

	//call function that pops up the dialog
	theErr = NavPutFile(NULL, &theReply, &dialogOptions, NULL, 'TEXT', 'ttxt', NULL);
	
	if(theErr)
		goto Bail;
	
	//if user selects a valid location
	if(theReply.validRecord) 
	{
		FSSpec	finalFSSpec;	
		AEDesc 	resultDesc;
		
		//get AE description
		if((theErr = AEGetNthDesc(&(theReply.selection), 1, typeFSS, NULL, &resultDesc)) == noErr)
       	{
       		//retrive the file spec from it
			AEGetDescData(&resultDesc, &finalFSSpec, sizeof(FSSpec));
			*theFile = finalFSSpec;
            
			AEDisposeDesc(&resultDesc);
		}
	}
	
	//close
	NavDisposeReply(&theReply);
	
Bail:

	SetGWorld(savePort, saveDevice);
	return theErr;
}



/*************************************
Stupid filter for openable files in 
Navigation dialog
*************************************/
pascal Boolean NavFilterProc (AEDesc* theItem, void* info,  NavCallBackUserData callBackUD, NavFilterModes filterMode)
{
	#pragma unused (theItem, info, callBackUD, filterMode)

    //all files are valid for opening
    return true;
}







#pragma mark -

/*************************************
Application ommand handler
*************************************/
static pascal OSStatus PlatypusCommandHandler (EventHandlerCallRef inCallRef, EventRef inEvent, void * inUserData)
{
	#pragma unused (inCallRef, inUserData)

    OSStatus        	result  = eventNotHandledErr;
    HICommand       	command;
    OSErr				err;
    static char			path[kMaxPathLength];
    	
                                    
    GetEventParameter( inEvent, kEventParamDirectObject, typeHICommand, NULL, sizeof( HICommand ), NULL, &command );

        switch ( GetEventKind( inEvent ) )
        {
            //regular old menu bar command
            case kEventCommandProcess:
                
                switch (command.commandID)
                {
                    //create app
                    case 'Crte':
                    
                    	//try to create application
                    	if (!CreateScriptApplication())
                    	{
                    		//close the window if it was successful
                        	ClosePlatypusWindow(platypusWindow);
                        }
                        break;
                    
                    //select script
                    case 'SSel':
                    
                        err = NavigationGetPath((char *)&path);
                        if (err)
                        	return 0;
                        
                        SetControlData(scriptPathTextField, kControlNoPart, kControlEditTextTextTag, strlen((char *)&path), path);
                        SetControlValue(typeRadioButtons, GetScriptType((char *)&path));
                        
                        //set radio button type based on suffix
            			SetControlValue(typeRadioButtons, GetScriptType((char *)&path));
            			SetIcon(2+GetControlValue(typeRadioButtons));
                        
						DrawControls(platypusWindow);
                    break;
                    //forward icon arrow
                    case 'Forw':
                        currentIcon++;
                        if (currentIcon > kNumIcons)
                        {
                            currentIcon = 0;
                         }
                         SetIcon(currentIcon);
                        break;
                        
                    //backward icon arrow
                    case 'Back':
                        currentIcon--;
                        if (currentIcon < 0)
                        {
                            currentIcon = kNumIcons;
                         }
                         SetIcon(currentIcon);
                        break;
                    
                    //quit
                  	case 'quit':
                  		QuitApp();
                  		break;
                  	
                  	//New window
                  	case 'new ':
                  		NewPlatypusWindow(platypusWindow);
                  		break;
                  	
                  	//About Platypus
                  	case 'abou':
                  		ShowWindow(aboutWindow);
                  		SelectWindow(aboutWindow);
                  		break;
                  	
                  	//Platypus website
                  	case 'PlWb':
                  		OpenURL((char *)&kPlatypusWebsite);
                  		break;
                  	
                  	//Contact author
                  	case 'CoAu':
                  		SendMail((char *)&kAuthorEmail);
                  		break;
                  	//Preferences
                  	case 'pref':
                  		/*ShowWindow(prefsWindow);
                  		SelectWindow(prefsWindow);*/
                  		break;
                }
                break;
        }
        
        return noErr;
}

/*************************************
Handler for main window
*************************************/
static pascal OSStatus PlatypusWindowHandler (EventHandlerCallRef inCallRef, EventRef inEvent, void * inUserData)
{ 
    #pragma unused (inCallRef, inUserData)

    switch (GetEventKind(inEvent))
    {
        case kEventWindowClose:
            ClosePlatypusWindow(platypusWindow);
            break;
  	}
    
    return noErr;
}

/*************************************
Handler for about window
*************************************/
static pascal OSStatus PlatypusAboutWindowHandler (EventHandlerCallRef inCallRef, EventRef inEvent, void *inUserData)
{ 
	#pragma unused (inCallRef)

	switch (GetEventKind(inEvent))
    {
        case kEventWindowClose:
            HideWindow((WindowRef)inUserData);
            break;
        }
    
    return noErr;
}




/*************************************
Handler for when Dock icon is clicked
*************************************/

static OSErr PlatypusReopenAEHandler( const AppleEvent *appleEvt, AppleEvent* reply, UInt32 refcon )
{
	#pragma unused (appleEvt, reply, refcon)

    if (IsWindowVisible(platypusWindow))
    	SelectWindow(platypusWindow);
    else
    	NewPlatypusWindow(platypusWindow);

    return noErr;
}

/*************************************
Handler for docs dragged on app icon
*************************************/

static OSErr PlatypusOpenDocAEHandler(const AppleEvent *theAppleEvent, AppleEvent *reply, long refCon)
{
	#pragma unused (reply, refCon)
	
        OSErr	            err;
        AEDescList	    	fileSpecList;
        short	        	    i;
        long	            count;
        long	            actualSize;
		
        FSSpec	            fileSpec;
        
        AEKeyword	        keyword;
        DescType	        type;
        
        char				path[kMaxPathLength];
        char				appName[kMaxFileNameLength];	
        
        //Read the AppleEvent
        err = AEGetParamDesc(theAppleEvent, keyDirectObject, typeAEList, &fileSpecList);
		
        //Count number of files
        err = AECountItems(&fileSpecList, &count);
        
        //iteratively open each file
        for (i = 1; i <= count; i++) 
        {
                err = AEGetNthPtr(&fileSpecList, i, typeFSS, &keyword, &type, (Ptr)&fileSpec, sizeof(FSSpec), &actualSize);
                if (err == noErr) 
                {
                	//get path from file spec
		            err = FSMakePath(fileSpec, (unsigned char *)&path, kMaxPathLength);
		            if (err)
		            	return err;
                
                	//make sure it isn't a folder
		            if (UnixIsFolder((char *)&path))
		            	return err;

                	if (IsWindowVisible(platypusWindow))
                	{
                		ClosePlatypusWindow(platypusWindow);
                	}
                	
                	//reveal window with the new params in field
		            NewPlatypusWindow(platypusWindow);
                	
                	//set the path of dragged file into Script path text field
            		err = SetControlData(scriptPathTextField, kControlNoPart, kControlEditTextTextTag, strlen((char *)&path), path);
                	
                	//set radio button type based on suffix
		            SetControlValue(typeRadioButtons, GetScriptType((char *)&path));
		            SetIcon(2+GetControlValue(typeRadioButtons));
                	
                	//set app name as that of script, minus suffix
		            strcpy((char *)&appName, GetFileNameFromPath((char *)&path));
		            CutSuffix((char *)&appName);
		            err += SetControlData(appNameTextField, kControlNoPart, kControlEditTextTextTag, strlen((char *)&appName), appName);
		            
		            
		            
		            //CreateScriptApplicationFromFSSpec(fileSpec);
                }
        }
        return err;
}

/*************************************
Drag and drop handler for script path text field
*************************************/
static pascal OSErr PlatypusDragHandler (WindowPtr theWindow, void *handlerRefCon, DragReference theDrag)
{
	#pragma unused (handlerRefCon, theWindow)

    short 			items, index;
    ItemReference 	theItem;
    FlavorFlags 	theFlags;
    Size 			dataSize;
    OSErr 			err = noErr;
    char			path[kMaxPathLength];
    char			appName[kMaxFileNameLength];		
    HFSFlavor		hfsFlavor;

    CountDragItems(theDrag, (unsigned short *)&items);
    for (index = 1; index <= items; index++)
    {
        GetDragItemReferenceNumber(theDrag, index, &theItem);
        err = GetFlavorFlags(theDrag, theItem, flavorTypeHFS, &theFlags);
        if (err == noErr) 
        {
            //load HFS file info from drag
            GetFlavorDataSize(theDrag, theItem, flavorTypeHFS, &dataSize);
            GetFlavorData(theDrag, theItem, flavorTypeHFS, &hfsFlavor, &dataSize, NULL);
            
            //get path from file spec
            err = FSMakePath(hfsFlavor.fileSpec, (unsigned char *)&path, kMaxPathLength);
            if (err)
            	return err;
            
            //user has dragged an icon file, we deal with that and return
            if (!strcmp((char*)&kIconFileSuffix,  GetFileSuffix((char *)&path)))
			{
				err = SetCustomIcon((char *)&path, hfsFlavor.fileSpec);
				return err;
			}
            
            //make sure it isn't a folder
            if (UnixIsFolder((char *)&path))
            	continue;
            
            //set the path of dragged file into Script path text field
            err = SetControlData(scriptPathTextField, kControlNoPart, kControlEditTextTextTag, strlen((char *)&path), path);
            
            //set radio button type based on suffix
            SetControlValue(typeRadioButtons, GetScriptType((char *)&path));
            SetIcon(2+GetControlValue(typeRadioButtons));
            
            //set app name as that of script, minus suffix
            strcpy((char *)&appName, GetFileNameFromPath((char *)&path));
            CutSuffix((char *)&appName);
            err += SetControlData(appNameTextField, kControlNoPart, kControlEditTextTextTag, strlen((char *)&appName), appName);
            
            
            return err;
        }
        
        //reset the list and info stuff and draw controls again
        continue;
    }

    return(noErr);
}










