#include <Carbon/Carbon.h>
#include "platypus.h"
#include <sys/stat.h>

////////////////////////////////////////
// Routine to register an icns file.
///////////////////////////////////////
IconRef RegisterIcon( FourCharCode inCreator, FourCharCode inType, const char* inName )
{
	IconRef result = 0;
	OSErr err = fnfErr;
	
	if ( inName != NULL && *inName != 0 )
	{
		CFBundleRef appBundle = CFBundleGetMainBundle();
		if ( appBundle )
		{
			CFStringRef fileName = CFStringCreateWithCString( NULL, inName, kCFStringEncodingASCII );
			if (fileName != NULL)
			{
				CFURLRef iconFileURL = CFBundleCopyResourceURL( appBundle, fileName, NULL, NULL );
				if (iconFileURL != NULL)
				{
					FSRef		fileRef;
					
					if ( CFURLGetFSRef( iconFileURL, &fileRef ) )
					{
						FSSpec		iconFileSpec;
						
						err = FSGetCatalogInfo( &fileRef, kFSCatInfoNone, NULL, NULL, &iconFileSpec, NULL );
						if ( err == noErr )
							err = RegisterIconRefFromIconFile( inCreator, inType, &iconFileSpec, &result );
					}
					CFRelease( iconFileURL );
				}
				CFRelease( fileName );
			}
		}
	}
	
	return result;
}


////////////////////////////////////////
//Standard error alert
////////////////////////////////////////
void RedAlert (Str255 errorString, Str255 expStr)
{
    StandardAlert (kAlertStopAlert, errorString,  expStr, NULL, NULL);
}


////////////////////////////////////////
//Standard error alert, then exit application
////////////////////////////////////////
void RedFatalAlert (Str255 errorString, Str255 expStr)
{
    StandardAlert (kAlertStopAlert, errorString,  expStr, NULL, NULL);
    QuitApp();
}

////////////////////////////////////////
//Standard warning alert
////////////////////////////////////////
void YellowAlert (Str255 errorString, Str255 expStr)
{
    StandardAlert (kAlertCautionAlert, errorString,  expStr, NULL, NULL);
}


////////////////////////////////////////
// Open URL with default browser using
// MacOS X's 'open' CLI command
///////////////////////////////////////
void OpenURL (char *url)
{
   	char	command[2048] = "open \"";

	//add defined URL
        strcat((char *)&command, url);
        strcat((char *)&command, "\"");
	//pass command to the system
	system((char *)&command);
}

////////////////////////////////////////
// Send system a command to open mail
// client and point to specified address
///////////////////////////////////////
void SendMail (char *email)
{
	char	command[512] = "open \"mailto:";

	//add defined URL
        strcat((char *)&command, email);
        strcat((char *)&command, "\"");
	//pass command to the system
	system((char *)&command);

}


////////////////////////////////////////
// Check if FSRef is a folder
///////////////////////////////////////
/*Boolean IsFolder (FSRef *fileRef)
{
	FSCatalogInfo 	catalogInfo;
	OSErr			err;

	err = FSGetCatalogInfo(fileRef, kFSCatInfoNodeFlags, &catalogInfo, NULL, NULL, NULL);
	if ((catalogInfo.nodeFlags & kFSNodeIsDirectoryMask) == kFSNodeIsDirectoryMask )
		return true;
	else
		return false;
}*/

////////////////////////////////////////
// Check if file in designated path is folder
///////////////////////////////////////
short UnixIsFolder (char *path)
{
    struct stat filestat;
    short err;
    
    err = stat(path, &filestat);
    if (err == -1)
        return err;

    if(S_ISREG(filestat.st_mode) != 1)
        return true;
    else 
        return false;
}



////////////////////////////////////////
// Cut the last segment of a path name 
// away from the string
///////////////////////////////////////
void CutFileFromPath (char *path)
{
    short i;
    short len = strlen(path);
    
    for (i = len; i > 0; i--)
    {
        if (path[i] == '/')
        {
            path[i] = '\0';
            break;
        }
    }
}

////////////////////////////////////////
// Generate path string from FSSpec record
///////////////////////////////////////
OSStatus FSMakePath(FSSpec file, UInt8 *path, UInt32 maxPathSize)
{
        OSStatus result;
        FSRef  ref;
 
        FSpMakeFSRef(&file, &ref);
 
 	/* and then convert the FSRef to a path */
	result = FSRefMakePath(&ref, path, maxPathSize);
        return ( result );
}


////////////////////////////////////////
// Returns the suffix of a file name,
// provided it is no longer than 
// the specified maximum suffix length
///////////////////////////////////////

char* GetFileSuffix (char *name)
{
    short	i, len, nameEnd = 0, suffixMaxLength = 11;
    
    len = strlen(name);
    
    for (i = 1; i < suffixMaxLength+2; i++)
    {
        if (name[len-i] == '.')
        {
            nameEnd = (len - i);
            break;
        }
    }
    
    return ((char *)&name[nameEnd]);
}


////////////////////////////////////////
// Cuts suffix from a file name
///////////////////////////////////////

char* CutSuffix (char *name)
{
    short	i, len, suffixMaxLength = 11;
    
    len = strlen(name);
    
    for (i = 1; i < suffixMaxLength+2; i++)
    {
        if (name[len-i] == '.')
        {
            name[len-i] = NULL;
            return name;
        }
    }
    return name;
}


////////////////////////////////////////
// Cuts suffix from a file name
///////////////////////////////////////

char* GetFileNameFromPath (char *name)
{
    short	i, len;
    
    len = strlen(name);
    
    for (i = len; i > 0; i--)
    {
        if (name[i] == '/')
        {
            return((char *)&name[i+1]);
        }
    }
    return name;
}


////////////////////////////////////////
// Determine script type according to suffix
///////////////////////////////////////

short GetScriptType (char *name)
{
		//shell
		if (!strcmp((char*)&kShellScriptSuffix,  GetFileSuffix((char *)name)))
		{
			return kShellScriptType;
		}
		//perl
		else if (!strcmp((char*)&kPerlScriptSuffix,  GetFileSuffix((char *)name)))
		{
			return kPerlScriptType;
		}
		//python
		else if (!strcmp((char*)&kPythonScriptSuffix,  GetFileSuffix((char *)name)))
		{
			return kPythonScriptType;
		}
		//ruby
		else if (!strcmp((char*)&kRubyScriptSuffix,  GetFileSuffix((char *)name)) || !strcmp((char*)&kRubyScriptSuffix2,  GetFileSuffix((char *)name)))
		{
			return kRubyScriptType;
		}
		//applescript
		else if (!strcmp((char*)&kAppleScriptSuffix,  GetFileSuffix((char *)name)) || !strcmp((char*)&kAppleScriptSuffix2,  GetFileSuffix((char *)name)))
		{
			return kAppleScriptType;
		}
		//tcl
		else if (!strcmp((char*)&kTclScriptSuffix,  GetFileSuffix((char *)name)))
		{
			return kTclScriptType;
		}
		
		return 0;
}