#include <Carbon/Carbon.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#define	kMaxOutputLength	100000

//output types
#define	kNoOutputString		"None"
#define	kTextOutputString	"Text"
#define	kProgressOutputString	"ProgressBar"

#define	kNoOutput		1
#define kTextOutput		2
#define	kProgressOutput		3


ControlRef		textField;
WindowRef 		theWindow;


int main(int argc, char* argv[])
{
    OSStatus		err;
    
    char		command[2048];
    char		buff[1000];
    char		*str;
    char		appName[255];
    
    

    FILE		*output;
    
    
    
    

    
    
    //generate command
    strcpy(&command, GetInterpreter());
    strcat(&command, &" '");
    strcat(&command, GetScript());
    strcat(&command, &"'");
    
    switch(GetOutputType())
    {
        /////////////////
        // None
        /////////////////
        case kNoOutput:
            err = system(&command);
            if (err == -1)
                RedFatalAlert("\pExecution error.", "\pAn error occurred when running script.");
            break;
        
        /////////////////
        // stdout text
        /////////////////
        case kTextOutput:
            //get application name from argv
            strcpy(&appName, GetFileNameFromPath(argv[0]));
            
            err = LoadMenuBar(&appName);
            if (err)
                RedFatalAlert("\pNib error.", "\pAn error occurred when loading menu bar from nib file.");
            
            err = LoadTextWindow(&appName);
            if (err)
                RedFatalAlert("\pNib error.", "\pAn error occurred when loading window from nib file.");
            
            str = malloc(kMaxOutputLength);
            if (str == 0)
                RedFatalAlert("\pMemory allocation error.", "\pFailed to malloc memory for output.");
                
            if ((output = popen(&command,"r")) == NULL)
                RedFatalAlert("\pExecution error.", "\pAn error occurred when opening pipe to command.");
    
            while (!feof(output))
            {
                fgets(buff,1000,output);
                if (strlen(str) + strlen(buff) < kMaxOutputLength)
                {
                    strcat(str, &buff);
                }
        
            }
    
            //put character output into editable text field
            SetControlData(textField, kControlNoPart, kControlEditTextTextTag, strlen(str), str);
            
            free(str);
            
            RunApplicationEventLoop();
            
            break;
            
        /////////////////
        // progress bar
        /////////////////
        case kProgressOutput:
            err = LoadProgressWindow();
            if (err)
                RedFatalAlert("\pExecution error.", "\pAn error occurred when loading window from nib.");
            err = system(&command);
            if (err == -1)
            	RedFatalAlert("\pExecution error.", "\pAn error occurred when running script.");
            break;
    }

    return 0;
}

#pragma mark -

//////////////////////////////////////
//  Load menu bar
/////////////////////////////////////

OSErr LoadMenuBar (char *appName)
{
    OSErr		err;
    IBNibRef 		nibRef;
    
    err = CreateNibReference(CFSTR("MenuBar"), &nibRef);
    if (err)
        return err;
    
    err = SetMenuBarFromNib(nibRef, CFSTR("MenuBar"));
    if (err)
        return err;

    DisposeNibReference(nibRef);

    return noErr;
}

//////////////////////////////////////
//  Load window for text output
/////////////////////////////////////

OSErr LoadTextWindow (char *appName)
{
    OSErr		err;
    IBNibRef 		nibRef;
    ControlID		controlID;
    Str255		pName;
    
    err = CreateNibReference(CFSTR("TextWindow"), &nibRef);
    if (err)
        return err;
        
    err = CreateWindowFromNib(nibRef, CFSTR("Window"), &theWindow);
    if (err)
        return err;

    DisposeNibReference(nibRef);

    //app name text field
    controlID.signature = 'Text';
    controlID.id = 0;
    GetControlByID (theWindow, &controlID, &textField);
    
    //set window title
    CopyCStringToPascal (appName, pName);
    SetWTitle (theWindow, pName);
    
    // The window was created hidden so show it.
    ShowWindow( theWindow );

    return noErr;
}

//////////////////////////////////////
//  Load modal progress bar
/////////////////////////////////////

OSErr LoadProgressWindow (void)
{
    OSErr		err;
    IBNibRef 		nibRef;
    ControlID		controlID;
    
    err = CreateNibReference(CFSTR("ProgressWindow"), &nibRef);
    if (err)
        return err;
        
    err = CreateWindowFromNib(nibRef, CFSTR("Window"), &theWindow);
    if (err)
        return err;

    DisposeNibReference(nibRef);
    
    // The window was created hidden so show it.
    ShowWindow( theWindow );

    return noErr;
}

#pragma mark -




short GetOutputType (void)
{
    CFStringRef 	fileName;
    CFBundleRef 	appBundle;
    CFURLRef		scriptFileURL;
    FSRef		fileRef;
    FSSpec		fileSpec;
    char		command[1000];
    static char		path[2048];
    char		typeStr[16];
    FILE		*output;
    
    OSErr		err = noErr;

    appBundle = CFBundleGetMainBundle();
    if (!appBundle)
        return NULL;

    fileName = CFStringCreateWithCString( NULL, "outputType", kCFStringEncodingASCII );
    if (fileName == NULL)
        return NULL;
        
    scriptFileURL = CFBundleCopyResourceURL( appBundle, fileName, NULL, NULL );
    if (scriptFileURL == NULL)
        return NULL;
    
    if ( !CFURLGetFSRef( scriptFileURL, &fileRef ))
        return NULL;
                                    
    err = FSGetCatalogInfo( &fileRef, kFSCatInfoNone, NULL, NULL, &fileSpec, NULL );
    if (err)
        return NULL;
    
    err = FSMakePath(fileSpec, &path, 2048);
    if (err)
        return NULL;

    //create cat command on interpreter text file
    strcpy(&command, &"/bin/cat ");
    strcat(&command, &path);

    //run command
    if ((output = popen(&command,"r")) == NULL)
        return NULL;
        
    //get the c string output
    fgets(typeStr, 16,output);

    //close pipe
    pclose(output);

    //release CF values
    CFRelease( scriptFileURL );
    CFRelease( fileName );
    
    if (!strcmp(&typeStr, &kNoOutputString))
        return kNoOutput;
    else if (!strcmp(&typeStr, &kTextOutputString))
        return kTextOutput;
    else if (!strcmp(&typeStr, &kProgressOutputString))
        return kProgressOutput;
    
    return (kNoOutput);
}


char* GetScript (void)
{
    CFStringRef 	fileName;
    CFBundleRef 	appBundle;
    CFURLRef		scriptFileURL;
    FSRef		fileRef;
    FSSpec		fileSpec;
    static char		path[2048];
    OSErr		err = noErr;

    appBundle = CFBundleGetMainBundle();
    if (!appBundle)
        return NULL;

    fileName = CFStringCreateWithCString( NULL, "script", kCFStringEncodingASCII );
    if (fileName == NULL)
        return NULL;
        
    scriptFileURL = CFBundleCopyResourceURL( appBundle, fileName, NULL, NULL );
    if (scriptFileURL == NULL)
        return NULL;
    
    
    if ( !CFURLGetFSRef( scriptFileURL, &fileRef ))
        return NULL;
        
                                
    err = FSGetCatalogInfo( &fileRef, kFSCatInfoNone, NULL, NULL, &fileSpec, NULL );
    if (err)
        return NULL;
    
    err = FSMakePath(fileSpec, &path, 2048);
    if (err)
        return NULL;
    
    return (&path);

}


char* GetInterpreter (void)
{
    CFStringRef 	fileName;
    CFBundleRef 	appBundle;
    CFURLRef		interpreterFileURL;
    FSRef		fileRef;
    FSSpec		fileSpec;
    char		command[2048];
    static char 	interpreter[1000];
    char		path[2048];
    FILE		*output;
    OSErr		err;
    
    appBundle = CFBundleGetMainBundle();
    if (!appBundle)
        return NULL;

    fileName = CFStringCreateWithCString( NULL, "interpreter", kCFStringEncodingASCII );
    if (fileName == NULL)
        return NULL;
        
    interpreterFileURL = CFBundleCopyResourceURL( appBundle, fileName, NULL, NULL );
    if (interpreterFileURL == NULL)
        return NULL;
    
    
    if ( !CFURLGetFSRef( interpreterFileURL, &fileRef ))
        return NULL;
        
                                
    err = FSGetCatalogInfo( &fileRef, kFSCatInfoNone, NULL, NULL, &fileSpec, NULL );
    if (err)
        return NULL;
    
    err = FSMakePath(fileSpec, &path, 2048);
    if (err)
        return NULL;
    
    //create cat command on interpreter text file
    strcpy(&command, &"/bin/cat ");
    strcat(&command, &path);
    
    //run command
    if ((output = popen(&command,"r")) == NULL)
        return NULL;

    //retrive output
    /*while (!feof(output))
    {*/
        //get the c string output
        fgets(interpreter,1000,output);
    //}

    //cloase pipe
    pclose(output);

    CFRelease( interpreterFileURL );
    CFRelease( fileName );
    
    return ((char *)&interpreter);
}

#pragma mark -

////////////////////////////////////////
// Generate path string from FSSpec record
///////////////////////////////////////
OSStatus FSMakePath(FSSpec file, UInt8 *path, UInt32 maxPathSize)
{
        OSStatus result;
        FSRef  ref;
 
        FSpMakeFSRef(&file, &ref);
 
 	/* and then convert the FSRef to a path */
	result = FSRefMakePath(&ref, path, maxPathSize);
        return ( result );
}


////////////////////////////////////////
// Gets name of a file from its path
///////////////////////////////////////

char* GetFileNameFromPath (char *name)
{
    short	i, len;
    
    len = strlen(name);
    
    for (i = len; i > 0; i--)
    {
        if (name[i] == '/')
        {
            return((char *)&name[i+1]);
        }
    }
    return name;
}


////////////////////////////////////////
//Standard error alert, then exit application
////////////////////////////////////////
void RedFatalAlert (Str255 errorString, Str255 expStr)
{
    StandardAlert (kAlertStopAlert, errorString,  expStr, NULL, NULL);
    ExitToShell();
}


/*************************************
Handler for docs dragged on app icon
*************************************/
/*
static OSErr PlatypusOpenDocAEHandler(const AppleEvent *theAppleEvent, AppleEvent *reply, long refCon)
{
	#pragma unused (reply, refCon)
	
        OSErr	            err;
        AEDescList	    	fileSpecList;
        short	        	    i;
        long	            count;
        long	            actualSize;
		
        FSSpec	            fileSpec;
        
        AEKeyword	        keyword;
        DescType	        type;
        
        char				path[kMaxPathLength];
        
        //Read the AppleEvent
        err = AEGetParamDesc(theAppleEvent, keyDirectObject, typeAEList, &fileSpecList);
		
        //Count number of files
        err = AECountItems(&fileSpecList, &count);
        
        //iteratively open each file
        for (i = 1; i <= count; i++) 
        {
                err = AEGetNthPtr(&fileSpecList, i, typeFSS, &keyword, &type, (Ptr)&fileSpec, sizeof(FSSpec), &actualSize);
                if (err == noErr) 
                {
                	//get path from file spec
		            err = FSMakePath(fileSpec, (unsigned char *)&path, kMaxPathLength);
		            if (err)
		            	return err;
                
                	//make sure it isn't a folder
		            if (UnixIsFolder((char *)&path))
		            	return err;

                	if (IsWindowVisible(platypusWindow))
                	{
                		ClosePlatypusWindow(platypusWindow);
                	}
                	
                	//reveal window with the new params in field
		            NewPlatypusWindow(platypusWindow);
                	
                	//set the path of dragged file into Script path text field
            		err = SetControlData(scriptPathTextField, kControlNoPart, kControlEditTextTextTag, strlen((char *)&path), path);
                	
                	//set radio button type based on suffix
		            SetControlValue(typeRadioButtons, GetScriptType((char *)&path));
		            SetIcon(2+GetControlValue(typeRadioButtons));
                	
                	//set app name as that of script, minus suffix
		            strcpy((char *)&appName, GetFileNameFromPath((char *)&path));
		            CutSuffix((char *)&appName);
		            err += SetControlData(appNameTextField, kControlNoPart, kControlEditTextTextTag, strlen((char *)&appName), appName);
		            
		            
		            
		            //CreateScriptApplicationFromFSSpec(fileSpec);
                }
        }
        return err;
}

*/