
/*
    Platypus - create MacOS X application bundles that execute scripts
    Copyright (C) 2003 Sveinbjorn Thordarson <sveinbt@hi.is>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    main.c - main program file

*/



//their stuff
#include <unistd.h>
#include <Carbon/Carbon.h>
#include <string.h>
#include <fcntl.h>

//my stuff
#include "platypus.h"


// Platypus windows and controls
WindowRef			platypusWindow, aboutWindow; //prefsWindow
ControlRef			appNameTextField, scriptPathTextField, interpreterTextField;
ControlRef			typeRadioButtons, iconImageWell, iconNameTextField, privilegesCheckBox;

//menus
Handle				menuBar;

char				customIconPath[kMaxPathLength] = "\0";

// icons
short				currentIcon = 0;
PlatypusIconStruct	platypusIcons[9];

short				appRun = false;


/*************************************
program start
*************************************/
int main (void)
{
    OSErr		err;

	// load the cursor
    InitCursor();

	//load icons in app bundle
    err = InitIcons();
    if (err)
        RedFatalAlert("\pInitialization error", "\pAn error occurred when loading icons.");

    // Create windows and menu bar from nib file
    err = InitNib();
    if (err)
        RedFatalAlert("\pInitialization error", "\pAn error occurred when loading window from Nib file.");
    
    // Get references to window controls
    err = InitControls();
    if (err)
        RedFatalAlert("\pInitialization error", "\pAn error occurred when getting control references.");
    
    // Install application event handlers
    err = InitHandlers();
    if (err)
        RedFatalAlert("\pInitialization error", "\pAn error occurred when initing application event handlers.");
    
    //create new platypus window
    NewPlatypusWindow();
    
    // Run the event loop
    RunApplicationEventLoop();

    return 0;
}


/*************************************
Quit application
*************************************/
void QuitApp (void)
{
    ExitToShell();
}



#pragma mark -

/*************************************
Install all event handlers
*************************************/
static OSErr InitHandlers (void)
{
    OSErr	err = noErr;

    const EventTypeSpec	commandEvents[] = { {kEventClassCommand, kEventProcessCommand} };
    const EventTypeSpec windowEvents[] = { {kEventClassWindow, kEventWindowClose}  };

    //command handler
    err += InstallEventHandler(GetApplicationEventTarget(), (EventHandlerUPP)NewEventHandlerUPP(PlatypusCommandHandler), 1, commandEvents, NULL, NULL );
    
    //main window handler
    err += InstallWindowEventHandler(platypusWindow, (EventHandlerUPP)NewEventHandlerUPP(PlatypusWindowHandler), 1, windowEvents, NULL, NULL );
    
    //prefs and about windows
    err += InstallWindowEventHandler(aboutWindow, (EventHandlerUPP)NewEventHandlerUPP(PlatypusAboutWindowHandler), 1, windowEvents, (void *) aboutWindow , NULL );
    //err += InstallWindowEventHandler(prefsWindow, (EventHandlerUPP)NewEventHandlerUPP(PlatypusAboutWindowHandler), 1, windowEvents, (void *) prefsWindow, NULL );

    //AE Handlers
	err += AEInstallEventHandler( kCoreEventClass, kAEReopenApplication, NewAEEventHandlerUPP((AEEventHandlerProcPtr)PlatypusReopenAEHandler), 0, false );
	err += AEInstallEventHandler( kCoreEventClass, kAEOpenDocuments, NewAEEventHandlerUPP((AEEventHandlerProcPtr)PlatypusOpenDocAEHandler), 0, false );
	err += AEInstallEventHandler( kCoreEventClass, kAEOpenApplication, NewAEEventHandlerUPP((AEEventHandlerProcPtr)PlatypusOpenAppAEHandler), 0, false );
	
	//Drag
	err += InstallReceiveHandler((DragReceiveHandlerUPP )NewDragReceiveHandlerUPP(PlatypusDragHandler), platypusWindow, NULL);
	
    return err;
}




/*************************************
Load window and menu bar from Nib file
*************************************/
static OSErr InitNib (void)
{
    OSErr		err;
    IBNibRef 		nibRef;
    
    err = CreateNibReference(CFSTR("main"), &nibRef);
    if (err)
        return err;
    
    err += CreateMenuBarFromNib(nibRef, CFSTR("MenuBar"), &menuBar);
    if (err)
    	return err;

    //No prefs yet
    //EnableMenuCommand(NULL, kHICommandPreferences);
    
    SetMenuBar(menuBar);
   	SetMenuStatus();
    
    err = CreateWindowFromNib(nibRef, CFSTR("MainWindow"), &platypusWindow);
    if (err)
        return err;
    
    err = CreateWindowFromNib(nibRef, CFSTR("AboutWindow"), &aboutWindow);
    if (err)
        return err;
   	
   /*	err = CreateWindowFromNib(nibRef, CFSTR("PrefsWindow"), &prefsWindow);
    if (err)
        return err;*/

    // We don't need the nib reference anymore.
    DisposeNibReference(nibRef);

    return noErr;
}




/*************************************
Get references to window controls
*************************************/
static OSErr InitControls (void)
{
    ControlID		controlID;
    
        //app name text field
        controlID.signature = 'Text';
        controlID.id = 0;
        GetControlByID (platypusWindow, &controlID, &appNameTextField);
        
        //script path text field
        controlID.signature = 'Text';
        controlID.id = 1;
        GetControlByID (platypusWindow, &controlID, &scriptPathTextField);

		//interpreter text field
        controlID.signature = 'Text';
        controlID.id = 2;
        GetControlByID (platypusWindow, &controlID, &interpreterTextField);

		//icon name text field
		controlID.signature = 'IcNm';
        controlID.id = 0;
        GetControlByID (platypusWindow, &controlID, &iconNameTextField);
		
		//icon image well
		controlID.signature = 'Icon';
        controlID.id = 0;
        GetControlByID (platypusWindow, &controlID, &iconImageWell);
        
        //get reference to type radio buttons
        controlID.signature = 'Type';
        controlID.id = 0;
        GetControlByID (platypusWindow, &controlID, &typeRadioButtons);
        
        //get reference to type radio buttons
        controlID.signature = 'Chek';
        controlID.id = 0;
        GetControlByID (platypusWindow, &controlID, &privilegesCheckBox);
        
 	return noErr;
}

/*************************************
Load shipping icons from icns files
*************************************/
static OSErr InitIcons (void)
{
	// Generic Application
	GetIconRef(kOnSystemDisk, 'macs', kGenericApplicationIcon, &platypusIcons[0].icon);
	strcpy((char *)&platypusIcons[0].iconName, (char *)&"Generic App");
	
	// Hexley Fork Icon
	platypusIcons[1].icon = RegisterIcon( 'pIcn', 'pIc1', "HexleyFork.icns" );
	strcpy((char *)&platypusIcons[1].iconName, (char *)&"Hexley Fork");
	strcpy((char *)&platypusIcons[1].fileName, (char *)&"HexleyFork.icns");
	
	// Hexley App Icon
	platypusIcons[2].icon = RegisterIcon( 'pIcn', 'pIc2', "HexleyApp.icns" );
	strcpy((char *)&platypusIcons[2].iconName, (char *)&"Hexley App");
	strcpy((char *)&platypusIcons[2].fileName, (char *)&"HexleyApp.icns");
	
	// Default Platypus App
	platypusIcons[3].icon = RegisterIcon( 'pIcn', 'pIc3', "platypusDefault.icns" );
	strcpy((char *)&platypusIcons[3].iconName, (char *)&"Platypus Default");
	strcpy((char *)&platypusIcons[3].fileName, (char *)&"platypusDefault.icns");
	
	// Perl Camel
	platypusIcons[4].icon = RegisterIcon( 'pIcn', 'pIc4', "PerlCamel.icns" );
	strcpy((char *)&platypusIcons[4].iconName, (char *)&"Perl Camel");
	strcpy((char *)&platypusIcons[4].fileName, (char *)&"PerlCamel.icns");
	
	// Python Snake Icon
	platypusIcons[5].icon = RegisterIcon( 'pIcn', 'pIc7', "PythonSnake.icns" );
	strcpy((char *)&platypusIcons[5].iconName, (char *)&"Python Snake");
	strcpy((char *)&platypusIcons[5].fileName, (char *)&"PythonSnake.icns");

	// Ruby Goat
	platypusIcons[6].icon = RegisterIcon( 'pIcn', 'pIc5', "RubyGoat.icns" );
	strcpy((char *)&platypusIcons[6].iconName, (char *)&"Ruby Goat");
	strcpy((char *)&platypusIcons[6].fileName, (char *)&"RubyGoat.icns");
	
	// AppleScript icon
	platypusIcons[7].icon = RegisterIcon( 'pIcn', 'pIc6', "AppleScript.icns" );
	strcpy((char *)&platypusIcons[7].iconName, (char *)&"AppleScript Icon");
	strcpy((char *)&platypusIcons[7].fileName, (char *)&"AppleScript.icns");
	
	// Tcl Flamingo
	platypusIcons[8].icon = RegisterIcon( 'pIcn', 'pIc8', "TclFlamingo.icns" );
	strcpy((char *)&platypusIcons[8].iconName, (char *)&"Tcl Flamingo");
	strcpy((char *)&platypusIcons[8].fileName, (char *)&"TclFlamingo.icns");
	
	
	// Default Platypus App
	// For About Window
	RegisterIcon( 'pIcn', 'Plat', "platypus.icns" );

    return noErr;
}


#pragma mark -



/*************************************
Set controls to default and show main window
*************************************/

static void NewPlatypusWindow (void)
{
	//no type radio button selected
	SetControlValue(typeRadioButtons, 0);
	
	//uncheck root privileges check box
	SetControlValue(privilegesCheckBox, 0);
	
	//set default app name
    SetControlData(appNameTextField, kControlNoPart, kControlEditTextTextTag, strlen(kDefaultAppName), kDefaultAppName);
    
    //clear script path field
    SetControlData(scriptPathTextField, kControlNoPart, kControlEditTextTextTag, 0, "");
    
    //clear interpreter text field
    SetControlData(interpreterTextField, kControlNoPart, kControlEditTextTextTag, 0, "");
    
    //select default icon
    SetIcon(kDefaultIcon);
    
    //reposition at center of screen
    RepositionWindow (platypusWindow, NULL, kWindowCenterOnMainScreen); 
    
    //reveal the window
    ShowWindow(platypusWindow);
    SelectWindow(platypusWindow);
    
    //update menus
    SetMenuStatus();
   
}

/*************************************
Hide main window
*************************************/
static void ClosePlatypusWindow (void)
{
    HideWindow(platypusWindow);
    SetMenuStatus();
}


#pragma mark -

/*************************************
Load script data into window, given its path
*************************************/
static OSErr LoadScript (char *path)
{
	OSErr 			err = noErr;
	char			appName[kMaxFileNameLength];
	char			interpreter[255];
	short			scriptType = 0;

	//make sure it isn't a folder
    if (UnixIsFolder((char *)path))
    	return true;

	//get interpreter from shebang
	err = GetInterpreterFromShebang(path, (char *)&interpreter);
	
	if (err)
	{
		//set radio button type based on suffix
    	scriptType = GetScriptType((char *)path);
    	SetScriptType(scriptType);
	}
	else
	{
			SetControlData(interpreterTextField, kControlNoPart, kControlEditTextTextTag, strlen((char *)&interpreter), &interpreter);
			SetScriptType(0);
		
	}
	//set the path of dragged file into Script path text field
    err = SetControlData(scriptPathTextField, kControlNoPart, kControlEditTextTextTag, strlen((char *)path), path);
    
	
    
    //set app name as that of script, minus suffix
    strcpy((char *)&appName, GetFileNameFromPath((char *)path));
    CutSuffix((char *)&appName);
    err += SetControlData(appNameTextField, kControlNoPart, kControlEditTextTextTag, strlen((char *)&appName), &appName);

	return err;
}








/*************************************
Take data entered and create app with it
*************************************/
static OSErr CreateScriptApplication (void)
{
	OSErr		err = noErr;
	Str255		pascalStr;

	short		scriptType = 0;
	short		authorized = 0;
	
	char		command[4096] = "\0";
	
    char		appName[kMaxFileNameLength] = "\0";
    char		appPath[kMaxPathLength] = "\0";
  	
    char		interpreterPath[kMaxFileNameLength] = "\0";
    
    char		iconPath[kMaxPathLength] = "\0";
    
    char		scriptPath[kMaxPathLength] = "\0";
    
    char		executablePath[kMaxPathLength] = "\0";
    char		perlScriptPath[kMaxPathLength] = "\0";
    
    
    //////// PRELIMINARY STUFF TO MAKE SURE WE CAN PROCEED WITH THIS WHOLE THING ////////////////
    
		//get path to script
	    err = GetControlData (scriptPathTextField, kControlEntireControl, kControlEditTextTextTag, sizeof(scriptPath), &scriptPath, NULL);
		if (err)
		{
			RedAlert("\pControl Error", "\pError retrieving field value from editable text field scriptPathTextField.");
	        return true;
		}
		
		//get path to interpreter
	    err = GetControlData (interpreterTextField, kControlEntireControl, kControlEditTextTextTag, sizeof(interpreterPath), &interpreterPath, NULL);
		if (err)
		{
			RedAlert("\pControl Error", "\pError retrieving field value from editable text field interpreterTextField.");
	        return true;
		}

		//check if script path is valid
	    if (!DoesFileExist((char *)&scriptPath))
	    {
	        YellowAlert("\pInvalid Script Path", "\pNo file exists at the specified path.");
	        return true;
	    }
	    
	    //check if interpreter path is valid
	    if (!DoesFileExist((char *)&interpreterPath))
	    {
	        YellowAlert("\pInvalid Interpreter", "\pNo interpreter fround at the specified path");
	        return true;
	    }
	    if (strlen((char *)&interpreterPath) > kMaxFileNameLength)
	    {
	    	RedAlert("\pInvalid Interpreter", "\pPath to interpreter is too long.  It cannot exceed 255 characters.");
	        return true;
	    
	    }
	    
	    //check if a folder has been specified
	    if (UnixIsFolder((char *)&scriptPath))
	    {
	    	YellowAlert("\pInvalid Script Path", "\pSpecified path points to a folder.");
	        return true;
	    }
	    
    
    
    
    
    
    
    //determine icon
    if (currentIcon == kCustomIcon)
    {	
    	strcpy((char *)&iconPath, (char *)&customIconPath);
    }
    else
    {
    	err = GetResourcePath((char *)&platypusIcons[currentIcon].fileName, (char *)&iconPath);
    	if (err)
    	{
    	
    	}
    }
    
    //determine whether script requires Administrator privileges
    authorized = GetControlValue(privilegesCheckBox);
    
    //get path to Perl script which creates the app
    err = GetResourcePath((char *)&kCreateScriptName, (char *)&perlScriptPath);
    
    //get path to the executable file which will be copied over to the app
    err = GetResourcePath((char *)&kExecutableName, (char *)&executablePath);
   
   
   
	//get app name
    err = GetControlData (appNameTextField, kControlEntireControl, kControlEditTextTextTag, sizeof(appName), &appName, NULL);
    if (err)
	{
		RedAlert("\pControl Error", "\pError retrieving field value from editable text field appNameTextField.");
        return true;
	}
    if (strlen((char *)&appName) == 0 || strlen((char *)&appName) > kMaxFileNameLength)
    {
    	YellowAlert("\pInvalid Name", "\pYou must specify a name for the application to be created.");
        return true;
    }
	CopyCStringToPascal ((char *)&appName, pascalStr);
	
	//get path to the app to be created
	err = NavigationPutPath(pascalStr, "\pSelect place to create app:", (char *)&appPath);
	if (err)
		return true;
	
	//get name from the path
	strcpy((char *)&appName, GetFileNameFromPath((char *)&appPath));
	CutSuffix((char *)&appName);
	
	//get path to the folder that is to contain the app
	CutFileFromPath((char *)&appPath);
	
    
   /*
    	The createapp.pl perl script accepts arguments:
    	
    	$appPath = $ARGV[0];
		$appName = $ARGV[1];
		$scriptPath = $ARGV[2];
		$execPath = $ARGV[3];
		$iconPath = $ARGV[4];
		$output = $ARGV[5];
		$interpreter = $ARGV[6];
    	$authorized = $ARGV[7];
    
    
    */
   
    
	if (err)
	{
	
	}
    
    //create command string
    sprintf((char *)&command, "'%s' '%s' '%s' '%s' '%s' '%s' '%s' 'None' '%s' '%d'", 
    						(char *)&kCreateScriptInterpreter, (char *)&perlScriptPath, (char *)&appPath, (char *)&appName, (char *)&scriptPath, (char *)&executablePath, (char *)iconPath, (char *)&interpreterPath, authorized);
    
    
    //run command
    err = system((char *)&command);
    if (err)
    	YellowAlert("\pCreation failed.", "\pAn unexpected error occurred when creating application.");
    
    
    return err;
}





#pragma mark -

/*************************************
Switch icon
*************************************/
static OSErr SetIcon (short iconNum)
{
	OSErr	err = noErr;
	ControlButtonContentInfo contentInfo;

	currentIcon = iconNum;
	SetWindowProxyIcon(platypusWindow, platypusIcons[iconNum].icon);
	err = SetControlData(iconNameTextField, kControlNoPart, kControlEditTextTextTag, strlen((char *)&platypusIcons[iconNum].iconName), &platypusIcons[iconNum].iconName);
	
	contentInfo.contentType = kControlContentIconRef;
	contentInfo.u.iconRef = platypusIcons[iconNum].icon;
	
	err = SetControlData(iconImageWell, kControlNoPart, kControlImageWellContentTag , sizeof(contentInfo), &contentInfo);
	
	Draw1Control(iconImageWell);
	Draw1Control(iconNameTextField);
	 
	
	return noErr;
}

/*************************************
Set custom icon
*************************************/
static OSErr SetCustomIcon (char *path, FSSpec fileSpec)
{
	OSErr						err = noErr;
	ControlButtonContentInfo 	contentInfo;
	IconRef						iconRef;
				
	//get icon reference from file
	err = RegisterIconRefFromIconFile('pIcn', 'Cust', &fileSpec, &iconRef);
	if (err)
		goto Err;
				
	//set it in the image well
	contentInfo.contentType = kControlContentIconRef;
	contentInfo.u.iconRef = iconRef;
	err = SetControlData(iconImageWell, kControlNoPart, kControlImageWellContentTag , sizeof(contentInfo), &contentInfo);
	if (err)
		goto Err;
	err = SetControlData(iconNameTextField, kControlNoPart, kControlEditTextTextTag, strlen((char *)&"Custom Icon"), "Custom Icon");
	if (err)
		goto Err;
	
	SetWindowProxyIcon(platypusWindow, iconRef);
	
	currentIcon = kCustomIcon;
	
	//get path to the icon into global string
	strcpy((char *)&customIconPath, path);

	//update these controls
	Draw1Control(iconImageWell);
	Draw1Control(iconNameTextField);
	
	return noErr;
	
	Err:
		if (currentIcon == kCustomIcon)
			return err;
		SetIcon(currentIcon);
		return err;
}

/*************************************
Enable/disable menu items appropriately
*************************************/
static void SetMenuStatus (void)
{
	short	platWinVis = IsWindowVisible(platypusWindow);
	short	aboutWinVis = IsWindowVisible(aboutWindow);

   	if (platWinVis == true || aboutWinVis == true)
        EnableMenuCommand((MenuRef)(MenuHandle *)(**menuBar), kHICommandClose);
   	else
   		DisableMenuCommand((MenuRef)(MenuHandle *)(**menuBar), kHICommandClose);
   	
   	if (platWinVis)
   	{
        DisableMenuCommand((MenuRef)(MenuHandle *)(**menuBar), kHICommandNew);
        EnableMenuCommand((MenuRef)(MenuHandle *)(**menuBar),  kPlatypusCreateCommand);
    }
    else
    {
        EnableMenuCommand((MenuRef)(MenuHandle *)(**menuBar), kHICommandNew);
        DisableMenuCommand((MenuRef)(MenuHandle *)(**menuBar),  kPlatypusCreateCommand);
    }

}

/*************************************
Set Icon, Radio button type and interpreter
according to script type
*************************************/
static OSErr SetScriptType (short type)
{
	OSErr	err = noErr;

	SetControlValue(typeRadioButtons, type);
	
	if (type != 0)
    	SetIcon(2 + type);
            
    switch(type)
    {
    	case kShellScriptType:
    		err = SetControlData(interpreterTextField, kControlNoPart, kControlEditTextTextTag, strlen((char *)&kShellScriptExec), kShellScriptExec);
    		break;
    	case kPerlScriptType:
    		err = SetControlData(interpreterTextField, kControlNoPart, kControlEditTextTextTag, strlen((char *)&kPerlScriptExec), kPerlScriptExec);
    		break;
    	case kPythonScriptType:
			err = SetControlData(interpreterTextField, kControlNoPart, kControlEditTextTextTag, strlen((char *)&kPythonScriptExec), kPythonScriptExec);
    		break;
    	case kRubyScriptType:
			err = SetControlData(interpreterTextField, kControlNoPart, kControlEditTextTextTag, strlen((char *)&kRubyScriptExec), kRubyScriptExec);
    		break;
    	case kAppleScriptType:
			err = SetControlData(interpreterTextField, kControlNoPart, kControlEditTextTextTag, strlen((char *)&kAppleScriptExec), kAppleScriptExec);
    		break;
    	case kTclScriptType:
			err = SetControlData(interpreterTextField, kControlNoPart, kControlEditTextTextTag, strlen((char *)&kTclScriptExec), kTclScriptExec);
    		break;
    }
    
    return err;
}

////////////////////////////////////////
// Determine script type according to suffix
///////////////////////////////////////

static short GetScriptType (char *name)
{
		//shell
		if (!strcmp((char*)&kShellScriptSuffix,  GetFileSuffix((char *)name)))
		{
			return kShellScriptType;
		}
		//perl
		else if (!strcmp((char*)&kPerlScriptSuffix,  GetFileSuffix((char *)name)))
		{
			return kPerlScriptType;
		}
		//python
		else if (!strcmp((char*)&kPythonScriptSuffix,  GetFileSuffix((char *)name)))
		{
			return kPythonScriptType;
		}
		//ruby
		else if (!strcmp((char*)&kRubyScriptSuffix,  GetFileSuffix((char *)name)) || !strcmp((char*)&kRubyScriptSuffix2,  GetFileSuffix((char *)name)))
		{
			return kRubyScriptType;
		}
		//applescript
		else if (!strcmp((char*)&kAppleScriptSuffix,  GetFileSuffix((char *)name)) || !strcmp((char*)&kAppleScriptSuffix2,  GetFileSuffix((char *)name)))
		{
			return kAppleScriptType;
		}
		//tcl
		else if (!strcmp((char*)&kTclScriptSuffix,  GetFileSuffix((char *)name)))
		{
			return kTclScriptType;
		}
		
		return 0;
}


/*************************************
Determine script type based on text in interpreter text field
*************************************/
/*static short GetScriptTypeFromInterpreterString (char *interpreterString)
{
	char			str[1000];
	short			len,i;

	strcpy((char *)&str, interpreterString);
	
	len = strlen((char *)&str);
	
	for (i = 0; i < len; i++)
	{
		if (str[i] == ' ')
			str[i] = NULL;
	}
	
	//shell
	if (!strcmp((char*)&kShellScriptExec,  (char *)&str))
	{
		return kShellScriptType;
	}
	//perl
	else if (!strcmp((char*)&kPerlScriptExec,  (char *)&str))
	{
		return kPerlScriptType;
	}
	//python
	else if (!strcmp((char*)&kPythonScriptExec,  (char *)&str))
	{
		return kPythonScriptType;
	}
	//ruby
	else if (!strcmp((char*)&kRubyScriptExec,  (char *)&str))
	{
		return kRubyScriptType;
	}
	//applescript
	else if (!strcmp((char*)&kAppleScriptExec,  (char *)&str))
	{
		return kAppleScriptType;
	}
	//tcl
	else if (!strcmp((char*)&kTclScriptExec,  (char *)&str))
	{
		return kTclScriptType;
	}

	return(0);
}
*/
/*************************************
Parse shebang line and get interpreter string
*************************************/
static OSErr GetInterpreterFromShebang (char *filePath, char *interpreter)
{
	FILE			*file;
	static char		str[1000];
	char			len,i;
	
	//open file and read to the first newline
	file = fopen(filePath, "r");
	fgets(str,1000,file);
	fclose(file);
	
	//doesn't seem to be a shebang line
	if (str[0] != '#' || str[1] != '!')
		return true;
	
	//jump over the first two characters that indicate the shebang
	strcpy((char *)&str, (char *)&str[2]);
	
	len = strlen((char *)&str);

	for (i = 0; i < len; i++)
	{
		//terminate string when we reach a space or a newline
		if (str[i] == '\n'|| str[i] == ' ')
			str[i] = NULL;
	}
	
	strcpy(interpreter, (char *)&str);
	
	//return pointer to the interpreter string
	return noErr;
}


#pragma mark -

/*************************************
Get path to a file via Navigation dialog
*************************************/

static OSErr NavigationGetPath (char *path)
{
    FSSpec			fileSpec;
    OSErr			err;

    err = NavigationGetFSSpec(&fileSpec);
    if (err == userCanceledErr)
    {
       	return true;
    }
    else if (err)
    {
        RedAlert("\pNavigation Error", "\pFunction NavigationGetFSSpec() returned an error when called from function NavigationGetPath().");
        return true;
    }
    
    err = FSMakePath(fileSpec, (unsigned char *)path, kMaxPathLength);
    

    return noErr;
}

/*************************************
Get path to a location via Nav dialog
*************************************/
static OSErr NavigationPutPath (Str255 defaultName, Str255 promptMessage, char *path)
{
	OSErr			err;
	FSSpec			fileSpec; //, defaultFolderSpec;
	//FSRef			defaultFolderRef;

	/*char			scriptPath[kMaxPathLength] = "\0";
    
    
	//get path to script
	err = GetControlData (scriptPathTextField, kControlEntireControl, kControlEditTextTextTag, sizeof(scriptPath), &scriptPath, NULL);
	if (err)
		return 0;
		
	CutFileFromPath((char *)&scriptPath);
	err = FSPathMakeRef((unsigned char *)&scriptPath, &defaultFolderRef, NULL);
	if (err)
		return 0;
		
	//convert FSRef to FSSpec
    err = FSGetCatalogInfo( &defaultFolderRef, kFSCatInfoNone, NULL, NULL, &defaultFolderSpec, NULL );
    if (err)
        return err;
	
	//Get default folder == folder which contains the script
	*/
	
	//Get the location in which the app is to be created
	err = NavigationPutFSSpec (promptMessage, defaultName, NULL, &fileSpec);
	if (err == userCanceledErr)
	{
            return true;
    }
    else if (err)
    {
    		RedAlert("\pNavigation Error", "\pError getting file specification from NavigationLib");
    		return true;
    }       
	
	//create dest file to get path to it, to fool Apple's stupid function
	err = FSpCreate(&fileSpec, 'Plat', 'Plat', smSystemScript);
		
	//get path in which to create app from FSSpec
	err = FSMakePath(fileSpec, (unsigned char *)path, kMaxPathLength);
	
	//delete teh placeholder file
	HDelete(fileSpec.vRefNum, fileSpec.parID, fileSpec.name);
	
	if (err)
	{
		RedAlert("\pPath error", "\pUnable to create path from FSSpec");
		return true;
	}

	return noErr;
}

/*************************************
Get filespec via navigation
*************************************/
static OSErr NavigationGetFSSpec (FSSpec *file)
{
    NavDialogOptions    dialogOptions;
    NavObjectFilterUPP  filterProc = NewNavObjectFilterUPP(NavFilterProc);
    OSErr               anErr = noErr;
    
    //  Specify default options for dialog box
    anErr = NavGetDefaultDialogOptions(&dialogOptions);
    if (anErr == noErr)
    {
        // Adjust the options to fit our needs
        //Set default location option
        dialogOptions.dialogOptionFlags |= kNavSelectDefaultLocation;
        dialogOptions.dialogOptionFlags |= kNavAllFilesInPopup;
        //Clear preview option
        dialogOptions.dialogOptionFlags ^= kNavAllowPreviews;
        
        if (anErr == noErr)
        {
            NavReplyRecord reply;
            
            // Call NavGetFile() with specified options
            anErr = NavGetFile (NULL, &reply, &dialogOptions,NULL, nil, filterProc, NULL, nil);
            
            if (anErr == noErr && reply.validRecord)
            {
                // Deal with multiple file selection
                long    count;
                
                anErr = AECountItems(&(reply.selection), &count);
                // Set up index for file list
                if (anErr == noErr)
                {
                    long index;
                    
                    //loop through all the files selected and open them in turn
                    for (index = 1; index <= count; index++)
                    {
                        AEKeyword   theKeyword;
                        DescType    actualType;
                        Size        actualSize;
                        FSSpec      documentFSSpec;
                        
                        // Get a pointer to selected file
                        anErr = AEGetNthPtr(&(reply.selection), index,
                                            typeFSS, &theKeyword,
                                            &actualType,&documentFSSpec,
                                            sizeof(documentFSSpec),
                                            &actualSize);
                        if (anErr == noErr)
                        {
                            *file = documentFSSpec;
                        }
                    }
                }
                //  Dispose of NavReplyRecord, resources, descriptors
                anErr = NavDisposeReply(&reply);
            }
        }
    }

    //DisposeRoutineDescriptor(filterProc);
    return anErr;
}



/*************************************
Get file spec for placement
*************************************/

static OSErr NavigationPutFSSpec (Str255 message, Str255 defaultName, FSSpec *defaultFolder, FSSpec *theFile)
{
	#pragma unused (defaultFolder)

	OSErr				theErr = noErr;
	NavReplyRecord			theReply;
	NavDialogOptions		dialogOptions;

	AEDesc 				aeDescriptor = { 'typeFSS', NULL };

	CGrafPtr			savePort = NULL;
	GDHandle			saveDevice = NULL;
	
	
	//AEReplaceDescData ( 'typeFSS', defaultFolder, sizeof(FSSpec), &aeDescriptor); 
	
	GetGWorld(&savePort, &saveDevice);
	
	//set the Navigation dialog options
	theErr = NavGetDefaultDialogOptions(&dialogOptions);
	if(theErr)
		return theErr;
	
	dialogOptions.dialogOptionFlags -= kNavDontAddTranslateItems;
	dialogOptions.dialogOptionFlags += kNavNoTypePopup;
	
	//set the default name in editable text field
	BlockMove(defaultName, dialogOptions.savedFileName, sizeof(Str255));
	
	//same goes for message
	if(message != nil)
            BlockMove(message, dialogOptions.message, sizeof(Str255));
	else
            dialogOptions.message[0] = 0;

	//call function that pops up the dialog
	theErr = NavPutFile(NULL, &theReply, &dialogOptions, NULL, 'TEXT', 'ttxt', NULL);
	
	if(theErr)
		goto Bail;
	
	//if user selects a valid location
	if(theReply.validRecord) 
	{
		FSSpec	finalFSSpec;	
		AEDesc 	resultDesc;
		
		//get AE description
		if((theErr = AEGetNthDesc(&(theReply.selection), 1, typeFSS, NULL, &resultDesc)) == noErr)
       	{
       		//retrive the file spec from it
			AEGetDescData(&resultDesc, &finalFSSpec, sizeof(FSSpec));
			*theFile = finalFSSpec;
            
			AEDisposeDesc(&resultDesc);
		}
	}
	
	//close
	NavDisposeReply(&theReply);
	
Bail:

	SetGWorld(savePort, saveDevice);
	return theErr;
}



/*************************************
Stupid filter for openable files in 
Navigation dialog
*************************************/
static pascal Boolean NavFilterProc (AEDesc* theItem, void* info,  NavCallBackUserData callBackUD, NavFilterModes filterMode)
{
	#pragma unused (theItem, info, callBackUD, filterMode)

    //all files are valid for opening
    return true;
}



#pragma mark -

/*************************************
Application ommand handler
*************************************/
static pascal OSStatus PlatypusCommandHandler (EventHandlerCallRef inCallRef, EventRef inEvent, void * inUserData)
{
	#pragma unused (inCallRef, inUserData)

    OSStatus        	result  = eventNotHandledErr;
    HICommand       	command;
    OSErr				err = noErr;
    static char			path[kMaxPathLength];
    	
                                    
    GetEventParameter( inEvent, kEventParamDirectObject, typeHICommand, NULL, sizeof( HICommand ), NULL, &command );

        switch ( GetEventKind( inEvent ) )
        {
            //regular old menu bar command
            case kEventCommandProcess:
                
                switch (command.commandID)
                {
                    //create app
                    case kPlatypusCreateCommand:
                    
                    	//try to create application
                    	if (!CreateScriptApplication())
                    	{
                    		//close the window if it was successful
                        	ClosePlatypusWindow();
                        }
                        break;
                    
                    //select script
                    case kPlatypusScriptSelectCommand:
                    
                        err = NavigationGetPath((char *)&path);
                        if (err)
                        	return err;
                        
                        err = LoadScript((char *)&path);
  
                    break;
                    
                    //script type selected from radio buttons
                    case kPlatypusTypeSetCommand:
				            SetScriptType(GetControlValue(typeRadioButtons));
                    	break;
                    
                    
                    //forward icon arrow
                    case kPlatypusNextIcon:
                        currentIcon++;
                        if (currentIcon > kNumIcons)
                        {
                            currentIcon = 0;
                         }
                         SetIcon(currentIcon);
                        break;
                        
                    //backward icon arrow
                    case kPlatypusPreviousIcon:
                        currentIcon--;
                        if (currentIcon < 0)
                        {
                            currentIcon = kNumIcons;
                         }
                         SetIcon(currentIcon);
                        break;
                    
                    //quit
                  	case kHICommandQuit:
                  		QuitApp();
                  		break;
                  	
                  	case kHICommandOpen:
                  		err = NavigationGetPath((char *)&path);
                        if (err)
                        	return err;
                        
                        if (IsWindowVisible(platypusWindow))
                        	ClosePlatypusWindow();
                        
                        NewPlatypusWindow();
                        
                       	err = LoadScript((char *)&path);
                       		return err;
                  		break;
                  	
                  	//New window
                  	case kHICommandNew :
                  		NewPlatypusWindow();
                  		break;
                  	
                  	//About Platypus
                  	case kHICommandAbout :
                  		//reposition at center of screen
                  		
    					RepositionWindow (aboutWindow, NULL, kWindowCenterOnMainScreen); 
                  		ShowWindow(aboutWindow);
                  		SelectWindow(aboutWindow);
                  		SetMenuStatus();
                  		break;
                  	
                  	//Platypus website
                  	case kPlatypusVisitWebsite:
                  		OpenURL((char *)&kPlatypusWebsite);
                  		break;
                  	
                  	//Contact author
                  	case kPlatypusContactAuthor:
                  		SendMail((char *)&kAuthorEmail);
                  		break;
                  	//Preferences
                  	case 'pref':
                  		/*ShowWindow(prefsWindow);
                  		SelectWindow(prefsWindow);*/
                  		break;
                }
                break;
        }
        
        return err;
}

/*************************************
Handler for main window
*************************************/
static pascal OSStatus PlatypusWindowHandler (EventHandlerCallRef inCallRef, EventRef inEvent, void * inUserData)
{ 
    #pragma unused (inCallRef, inUserData)

    switch (GetEventKind(inEvent))
    {
        case kEventWindowClose:
            ClosePlatypusWindow();
            break;
  	}
    
    return noErr;
}

/*************************************
Handler for about window
*************************************/
static pascal OSStatus PlatypusAboutWindowHandler (EventHandlerCallRef inCallRef, EventRef inEvent, void *inUserData)
{ 
	#pragma unused (inCallRef)

	switch (GetEventKind(inEvent))
    {
        case kEventWindowClose:
            HideWindow((WindowRef)inUserData);
            SetMenuStatus();
            break;
        }
    
    return noErr;
}

/*************************************
Handler for interpreter text field
*************************************/
/*static pascal OSStatus PlatypusInterpreterTextFieldHandler (EventHandlerCallRef inCallRef, EventRef inEvent, void *inUserData)
{ 
	#pragma unused (inCallRef, inEvent, inUserData)

	char		interpreter[1000];
	OSErr		err = noErr;
		
	err = GetControlData (interpreterTextField, kControlEntireControl, kControlEditTextTextTag, sizeof(interpreter), &interpreter, NULL);

   	SetControlValue(typeRadioButtons, GetScriptTypeFromInterpreterString((char *)&interpreter));
    
    return eventNotHandledErr;
}
*/


 
#pragma mark -


/*************************************
Handler for when Dock icon is clicked
*************************************/

static OSErr PlatypusReopenAEHandler( const AppleEvent *appleEvt, AppleEvent* reply, UInt32 refcon )
{
	#pragma unused (appleEvt, reply, refcon)

    if (IsWindowVisible(platypusWindow))
    	SelectWindow(platypusWindow);
    else
    	NewPlatypusWindow();

    return noErr;
}

/*************************************
Handler for docs dragged on app icon
*************************************/

static OSErr PlatypusOpenDocAEHandler(const AppleEvent *theAppleEvent, AppleEvent *reply, long refCon)
{
	#pragma unused (reply, refCon)
	
        OSErr	            err;
        AEDescList	    	fileSpecList;
        short	        	    i;
        long	            count;
        long	            actualSize;
		
        FSSpec	            fileSpec;
        
        AEKeyword	        keyword;
        DescType	        type;
        
        char				path[kMaxPathLength];
        
       
        
        //Read the AppleEvent
        err = AEGetParamDesc(theAppleEvent, keyDirectObject, typeAEList, &fileSpecList);
		
        //Count number of files
        err = AECountItems(&fileSpecList, &count);
        
        //iteratively open each file
        for (i = 1; i <= count; i++) 
        {
                err = AEGetNthPtr(&fileSpecList, i, typeFSS, &keyword, &type, (Ptr)&fileSpec, sizeof(FSSpec), &actualSize);
                if (err == noErr) 
                {
                	//get path from file spec
		            err = FSMakePath(fileSpec, (unsigned char *)&path, kMaxPathLength);
		            if (err)
		            	return err;
                
                	//make sure it isn't a folder
		            if (UnixIsFolder((char *)&path))
		            	return err;

                	if (IsWindowVisible(platypusWindow) && appRun == true)
                	{
                		ClosePlatypusWindow();
                		NewPlatypusWindow();
                		appRun = false;
                	}
                	else
                	{
                		NewPlatypusWindow();
                	}
                	
                	//reveal window with the new params in field
                	/*if (appRun != false)
                	{*/
		            	

                	//}
                	
                	err = LoadScript((char *)&path);
		            
                }
        }
        return err;
}

static OSErr PlatypusOpenAppAEHandler(const AppleEvent *theAppleEvent, AppleEvent *reply, long refCon)
{
	#pragma unused (reply, refCon, theAppleEvent)
	
	//NewPlatypusWindow();
  	
  	appRun = true;
  	
  	return noErr;
}

/*************************************
Drag and drop handler for script path text field
*************************************/
static pascal OSErr PlatypusDragHandler (WindowPtr theWindow, void *handlerRefCon, DragReference theDrag)
{
	#pragma unused (handlerRefCon, theWindow)

    short 			items, index;
    ItemReference 	theItem;
    FlavorFlags 	theFlags;
    Size 			dataSize;
    OSErr 			err = noErr;
    char			path[kMaxPathLength];
    HFSFlavor		hfsFlavor;

    CountDragItems(theDrag, (unsigned short *)&items);
    for (index = 1; index <= items; index++)
    {
        GetDragItemReferenceNumber(theDrag, index, &theItem);
        err = GetFlavorFlags(theDrag, theItem, flavorTypeHFS, &theFlags);
        if (err == noErr) 
        {
            //load HFS file info from drag
            GetFlavorDataSize(theDrag, theItem, flavorTypeHFS, &dataSize);
            GetFlavorData(theDrag, theItem, flavorTypeHFS, &hfsFlavor, &dataSize, NULL);
            
            //get path from file spec
            err = FSMakePath(hfsFlavor.fileSpec, (unsigned char *)&path, kMaxPathLength);
            if (err)
            	return err;
            
            //user has dragged an icon file, we deal with that and return
            if (!strcmp((char*)&kIconFileSuffix,  GetFileSuffix((char *)&path)))
			{
				err = SetCustomIcon((char *)&path, hfsFlavor.fileSpec);
				return err;
			}
            
            //ok, he loaded something other than an icon
            err = LoadScript((char *)&path);
            
            return err;
        }
        
        //reset the list and info stuff and draw controls again
        continue;
    }

    return(noErr);
}



