
/*
    Platypus - create MacOS X application bundles that execute scripts
    Copyright (C) 2003 Sveinbjorn Thordarson <sveinbt@hi.is>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    platypus.h - main header file

*/

#include <Carbon/Carbon.h>


/************************************************************************************
**  DEFINITIONS
************************************************************************************/
#pragma mark - definitions


//application info
#define		kAppName			"Platypus"
#define		kAppVersion			"1.5"
#define		kPlatypusComment	"Created with Platypus 1.5"


#define		kExecutableName				"exec"
#define		kCreateScriptName			"createapp.pl"
#define		kCreateScriptInterpreter	"/usr/bin/perl"

//application commands

#define		kPlatypusCreateCommand			'Crte'
#define		kPlatypusScriptSelectCommand	'SSel'
#define		kPlatypusTypeSetCommand			'Type'
#define		kPlatypusPreviousIcon			'Back'
#define		kPlatypusNextIcon				'Forw'
#define		kPlatypusVisitWebsite			'PlWb'
#define		kPlatypusContactAuthor			'CoAu'


//script types
#define		kShellScriptType	1
#define		kPerlScriptType		2
#define		kPythonScriptType	3
#define		kRubyScriptType		4
#define		kAppleScriptType	5
#define		kTclScriptType		6

// interpreters for different script types

#define		kShellScriptExec	"/bin/sh"
#define		kPerlScriptExec		"/usr/bin/perl"
#define		kPythonScriptExec	"/usr/bin/python"
#define		kRubyScriptExec		"/usr/bin/ruby"
#define		kAppleScriptExec	"/usr/bin/osascript"
#define		kTclScriptExec		"/usr/bin/tclsh"

// output types

#define		kNoOutput			"None"
#define		kTextOutput			"Text"
#define		kProgressOutput		"ProgressBar"


//script type suffices
#define		kShellScriptSuffix	".sh"
#define		kPerlScriptSuffix	".pl"
#define		kPythonScriptSuffix	".py"
#define		kRubyScriptSuffix	".rb"
#define		kRubyScriptSuffix2	".rbx"
#define		kAppleScriptSuffix	".applescript"
#define		kAppleScriptSuffix2	".osa"
#define		kTclScriptSuffix	".tcl"



#define		kIconFileSuffix		".icns"


//number of icons available
#define		kNumIcons		8
#define		kCustomIcon		-1
#define		kDefaultIcon	3

//author
#define		kAuthorEmail		"sveinbt@hi.is"
#define		kPlatypusWebsite	"http://raunvis.hi.is/~ssv/software.html"

//default settings
#define		kDefaultAppName		"MyApp"

//limits
#define		kMaxFileNameLength	255
#define		kMaxPathLength		1024//this seems to be the max path length in MacOS X 10.2



/************************************************************************************
**  PROTOTYPES
************************************************************************************/
#pragma mark - prototypes


	// main.c
	void QuitApp (void);

		// initialization routines
		static OSErr InitHandlers (void);
		static OSErr InitNib (void);
		static OSErr InitControls (void);
		static OSErr InitIcons (void);
		static OSErr GetResourcesPath (char *appPath);

		// Window functions
		static void NewPlatypusWindow (void);
		static void ClosePlatypusWindow (void);
		
		// The good stuff
		static OSErr CreateScriptApplication (void);
		static OSErr LoadScript (char *path);
		 
		// Utility functions
		static OSErr SetIcon (short iconNum);
		static OSErr SetCustomIcon (char *path, FSSpec fileSpec);
		static void SetMenuStatus (void);
		static OSErr SetScriptType (short type);
		static short GetScriptType (char *name);
		static OSErr GetInterpreterFromShebang (char *filePath, char *interpreter);
		static OSErr SetPlatypusComment (char *filePath);
		
		//Navigation
		//static OSErr NavigationPutPath (Str255 defaultName, Str255 promptMessage, char *path);
		static OSErr NavigationPutPath (Str255 message, Str255 defaultName, char *path);
		static OSErr NavigationGetPath (char *navPath);
		static OSErr NavigationGetFSSpec (FSSpec *file);
		//static OSErr NavigationPutFSSpec (Str255 message, Str255 defaultName, FSSpec *defaultFolder, FSSpec *theFile);
		//static OSErr NavigationPutFSSpec (Str255 message, Str255 defaultName, FSSpec *file);
		//static OSErr NavigationPutFSSpec (Str255 message, Str255 defaultName, FSSpec *file, CFStringRef name);
		pascal Boolean NavFilterProc (AEDesc* theItem, void* info,  NavCallBackUserData callBackUD, NavFilterModes filterMode);

		// Application handlers
		static pascal OSStatus PlatypusCommandHandler (EventHandlerCallRef inCallRef, EventRef inEvent, void * inUserData);
		static pascal OSStatus PlatypusWindowHandler (EventHandlerCallRef inCallRef, EventRef inEvent, void * inUserData);
		static pascal OSStatus PlatypusAboutWindowHandler (EventHandlerCallRef inCallRef, EventRef inEvent, void *inUserData);
		static pascal OSStatus PlatypusInterpreterTextFieldHandler (EventHandlerCallRef inCallRef, EventRef inEvent, void *inUserData);
		
		// Apple Event Handlers
		static OSErr PlatypusReopenAEHandler( const AppleEvent *appleEvt, AppleEvent* reply, UInt32 refcon );
		static OSErr PlatypusOpenDocAEHandler(const AppleEvent *theAppleEvent, AppleEvent *reply, long refCon);
		static OSErr PlatypusOpenAppAEHandler(const AppleEvent *theAppleEvent, AppleEvent *reply, long refCon);
		static pascal OSErr PlatypusDragHandler (WindowPtr theWindow, void *handlerRefCon, DragReference theDrag);
	
	// util.c

		IconRef RegisterIcon( FourCharCode inCreator, FourCharCode inType, const char* inName );
		void RedAlert (Str255 errorString, Str255 expStr);
		void RedFatalAlert (Str255 errorString, Str255 expStr);
		void DoCautionAlert (Str255 errorString, Str255 expStr);
		void YellowAlert (Str255 errorString, Str255 expStr);
		void OpenURL (char *url);
		void SendMail (char *email);
		short UnixIsFolder (char *path);
		void CutFileFromPath (char *path);
		OSStatus FSMakePath(FSSpec file, UInt8 *path, UInt32 maxPathSize);
		char* GetFileSuffix (char *name);
		short GetScriptType (char *name);
		char* CutSuffix (char *name);
		char* GetFileNameFromPath (char *name);
		short DoesFileExist (char *path);
		OSErr GetResourcePath (char *name, char *filePath);
	



		pascal void MyNavEventProc ( NavEventCallbackMessage callBackSelector, NavCBRecPtr callBackParms, void * callBackUD);


/************************************************************************************
**  STRUCTURES
************************************************************************************/
#pragma mark - structures

/////////////////////////////////
// For icons used by Platypus
/////////////////////////////////
typedef struct
{
    IconRef	icon;
    char	iconName[255];
    char	fileName[2048];

} PlatypusIconStruct;




