#import "Platypus.h"
#include <stdio.h>
#include "suffix.h"

@implementation Platypus


- (id)init
{
	if (self = [super init]) 
	{
		//create file list
		fileList = [[FileList alloc] init];
    }
    return self;
}

+ (void)initialize 
{ 
	// create the user defaults here if none exists
    // create a dictionary
    NSMutableDictionary *defaultPrefs = [NSMutableDictionary dictionary];
    // put default prefs in the dictionary
	[defaultPrefs setObject: [NSNumber numberWithBool:YES] forKey:@"ParseShebangLineForInterpreter"];
	[defaultPrefs setObject: [NSNumber numberWithBool:NO]  forKey:@"RevealApplicationWhenCreated"];
	[defaultPrefs setObject: @"TextEdit"  forKey:@"DefaultEditor"];
	
    // register the dictionary of defaults
    [[NSUserDefaults standardUserDefaults] registerDefaults: defaultPrefs];
}


- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
	[window center];
	[window registerForDraggedTypes: [NSArray arrayWithObjects:NSFilenamesPboardType, nil]];
	[window makeKeyAndOrderFront: NULL];
}

- (BOOL)application:(NSApplication *)theApplication openFile:(NSString *)filename
{
	[self loadScript: filename];
	return(YES);
}

#pragma mark -

/*****************************************
 - called when + button is pressed
*****************************************/

- (IBAction)addFileToFileList:(id)sender
{
	int result;
	
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];
	
	[oPanel setCanChooseDirectories:YES];
    [oPanel setAllowsMultipleSelection:YES];
    result = [oPanel runModalForDirectory:nil file:nil types:nil];
    if (result == NSOKButton) 
	{
        NSArray *filesToOpen = [oPanel filenames];
        int i, count = [filesToOpen count];
        for (i=0; i<count; i++) 
		{
			[fileList addFile: [filesToOpen objectAtIndex:i]];
        }
		
		[fileListDataBrowser setDataSource: fileList];
		[fileListDataBrowser reloadData];
    }
}

/*****************************************
 - called when C button is pressed
*****************************************/

- (IBAction)clearFileList:(id)sender
{
	[fileList clearList];
	[fileListDataBrowser reloadData];
}


/*****************************************
 - called when R button is pressed
*****************************************/
- (IBAction)revealFileInFileList:(id)sender
{	
	int i;
	NSIndexSet *selectedItems = [fileListDataBrowser selectedRowIndexes];
	
	for (i = 0; i < [fileList numFiles]; i++)
	{
		if ([selectedItems containsIndex: i])
		{
			[fileList revealInFinder: i];
		}
	}
}

/*****************************************
 - called when - button is pressed
*****************************************/

- (IBAction)removeFileFromFileList:(id)sender
{
	int i, didRemove = FALSE;
	NSIndexSet *selectedItems = [fileListDataBrowser selectedRowIndexes];
	
	for (i = [fileList numFiles]; i >= 0; i--)
	{
		if ([selectedItems containsIndex: i])
		{
			[fileList removeFile: i];
			didRemove = TRUE;
		}
	}
	
	if (didRemove == TRUE)
		[fileListDataBrowser reloadData];
}


#pragma mark -

/*****************************************
 - Create a new script and open in external editor
*****************************************/

- (IBAction)newScript:(id)sender
{
	NSString	*tempScript = @"/tmp/platypusScript";
	NSArray *defaultInterpreters = [NSArray arrayWithObjects: @"/bin/sh",@"/usr/bin/python",@"/usr/bin/osascript",@"/usr/bin/expect",@"/usr/bin/perl",@"/usr/bin/ruby",@"/usr/bin/tclsh",@"/usr/bin/php", nil];
	
	NSString	*shebangStr = [NSString stringWithFormat: @"#!%@\n\n", [defaultInterpreters objectAtIndex: [[scriptTypeRadioButtons selectedCell] tag]]];
	[shebangStr writeToFile: tempScript atomically: YES];

	[self loadScript: tempScript];
	[self editScript: self];

}

/*****************************************
 - Reveal script in Finder
*****************************************/

- (IBAction)revealScript:(id)sender
{
	if ([[scriptPathTextField stringValue] length] == 0)
		return;
		
	//see if file exists
	if ([[NSFileManager defaultManager] fileExistsAtPath: [scriptPathTextField stringValue] ])
	{
		[[NSWorkspace sharedWorkspace] selectFile:[scriptPathTextField stringValue] inFileViewerRootedAtPath:nil];
	}
	else
	{
		[STUtil alert:@"File does not exist" subText: @"No file exists at the specified path"];
	}
}


/*****************************************
 - Open script in external editor
*****************************************/

- (IBAction)editScript:(id)sender
{
	if ([[scriptPathTextField stringValue] length] == 0)
		return;
		
	//see if file exists
	if ([[NSFileManager defaultManager] fileExistsAtPath: [scriptPathTextField stringValue] ])
	{
		NSArray *args = [NSArray arrayWithObjects: @"-a", [[NSUserDefaults standardUserDefaults] stringForKey:@"DefaultEditor"], [scriptPathTextField stringValue], nil];
		[NSTask launchedTaskWithLaunchPath:@"/usr/bin/open" arguments: args];
	}
	else
	{
		[STUtil alert:@"File does not exist" subText: @"No file exists at the specified path"];
	}
}

/*****************************************
 - generate application bundle from data provided
*****************************************/

- (IBAction)createApp:(id)sender
{
	int result, i;
	NSString *appPath, *contentsPath, *macosPath, *resourcesPath;
	NSString *execDestinationPath, *infoPlistPath, *iconPath, *bundledFileDestPath, *progressBarNibDestPath;
	NSString *execPath, *bundledFilePath, *progressBarNibPath;
	NSString *interpreterFilePath, *droppableFilePath, *scriptFilePath, *outputTypeFilePath, *authFilePath;
	NSString *appName;
	NSBundle *platypusBundle = [NSBundle mainBundle];
	char	cStr[255];
	
	//file manager
	NSFileManager *fileManager = [NSFileManager defaultManager];
	
	/////////////**************verify that values in text fields are valid**********************//////////////
	
	if ([fileManager fileExistsAtPath: [scriptPathTextField stringValue]] == NO)
	{
		[STUtil alert:@"Invalid Script Path" subText: @"No file exists at the specified script path"];
		return;
	}
	
	if ([fileManager fileExistsAtPath: [interpreterTextField stringValue]] == NO)
	{
		if (NO == [STUtil proceedWarning: @"Invalid Interpreter" subText: @"The specified interpreter does not exist on this system.  Do you wish to proceed anyway?"])
			return;
	}
	
	
	
	/////////////**************init and run save panel**********************//////////////
	NSSavePanel *sPanel = [NSSavePanel savePanel];
	[sPanel setPrompt:@"Create"];
	[sPanel setTitle:@"Create App"];
	result = [sPanel runModalForDirectory:nil file: [appNameTextField stringValue]];
	if (result == NSOKButton) 
	{
		appPath = [sPanel filename];
    }
	else
		return;
	
	
	
	//////////********* Create the folder hierarchy ************////////////////
	
			//Application.app bundle
			if (![appPath hasSuffix:@".app"])
			{
				appPath = [appPath stringByAppendingString:@".app"];
			}
			[fileManager createDirectoryAtPath: appPath attributes:nil];
			
			//Contents
			contentsPath = [appPath stringByAppendingString:@"/Contents"];
			[fileManager createDirectoryAtPath: contentsPath attributes:nil];
			
			//MacOS
			macosPath = [contentsPath stringByAppendingString:@"/MacOS"];
			[fileManager createDirectoryAtPath: macosPath attributes:nil];
			
			//Resources
			resourcesPath = [contentsPath stringByAppendingString:@"/Resources"];
			[fileManager createDirectoryAtPath: resourcesPath attributes:nil];
	
	//////////********* Copy files to app bundle ************////////////////
	
		//exec file
		[[appPath lastPathComponent] getCString: cStr maxLength: sizeof(cStr)];
		CutSuffix((char *)&cStr);
		appName = [NSString stringWithCString: cStr length: strlen(cStr)];
		
		execPath = [platypusBundle pathForResource:@"exec" ofType:nil];
		execDestinationPath = [macosPath stringByAppendingString:@"/"];
		execDestinationPath = [execDestinationPath stringByAppendingString: appName]; 
		//NSLog(@"%@", execDestinationPath);
		[fileManager copyPath:execPath toPath:execDestinationPath handler:nil];
		
			//menu nib file
			progressBarNibPath = [platypusBundle pathForResource:@"MainMenu.nib" ofType:nil];
			progressBarNibDestPath = [resourcesPath stringByAppendingString:@"/MainMenu.nib"];
			[fileManager copyPath:progressBarNibPath toPath:progressBarNibDestPath handler:nil];
		
		//NSString *interpreterFilePath, *droppableFilePath, *scriptFilePath, *outputTypeFilePath, *authFilePath;
		
		//interpreter file
		interpreterFilePath = [resourcesPath stringByAppendingString:@"/interpreter"];
		[[interpreterTextField stringValue] writeToFile: interpreterFilePath atomically: YES];
		
		//script file
		scriptFilePath = [resourcesPath stringByAppendingString:@"/script"];
		[fileManager copyPath: [scriptPathTextField stringValue] toPath: scriptFilePath handler:nil];
		
		//auth file
		authFilePath = [resourcesPath stringByAppendingString:@"/auth"];
		if ([rootPrivilegesCheckbox state] == NSOnState)
			[@"Auth" writeToFile: authFilePath atomically: YES];
		else
			[@"NoAuth" writeToFile: authFilePath atomically: YES];
			
		//droppable file
		droppableFilePath = [resourcesPath stringByAppendingString:@"/droppable"];
		if ([isDroppableCheckbox state] == NSOnState)
			[@"noDrop" writeToFile: droppableFilePath atomically: YES];
		else
			[@"drop" writeToFile: droppableFilePath atomically: YES];
		
		//outputType file
		outputTypeFilePath = [resourcesPath stringByAppendingString:@"/outputType"];
		if ([[outputTypeRadioButtons selectedCell] tag] == 0)
			[@"None" writeToFile: outputTypeFilePath atomically: YES];
		else if ([[outputTypeRadioButtons selectedCell] tag] == 1)
			[@"ProgressBar" writeToFile: outputTypeFilePath atomically: YES];
		else if ([[outputTypeRadioButtons selectedCell] tag] == 2)
			[@"Text" writeToFile: outputTypeFilePath atomically: YES];
		
		
		//create icon
		iconPath = [resourcesPath stringByAppendingString:@"/appIcon.icns"];
		[self writeImageAsIcon: [appIconImageWell image] path: iconPath];
		
		//create Info.plist file
		infoPlistPath = [contentsPath stringByAppendingString:@"/Info.plist"];
		NSDictionary *infoPlist = [self getInfoPropertyListDictionary: appName isDroppable: [isDroppableCheckbox state]];
		[infoPlist writeToFile: infoPlistPath atomically: YES];
		
		//copy files in file list to the Resources folder
		for (i = 0; i < [fileList numFiles]; i++)
		{
			bundledFilePath = [fileList getFileAtIndex: i];
			bundledFileDestPath = [resourcesPath stringByAppendingString:@"/"];
			bundledFileDestPath = [bundledFileDestPath stringByAppendingString: [bundledFilePath lastPathComponent]];
			[fileManager copyPath: bundledFilePath toPath: bundledFileDestPath handler:nil];
		}
		
		//reveal app in Finder
		if ([[NSUserDefaults standardUserDefaults] boolForKey:@"RevealApplicationWhenCreated"] == 1)
		{
			BOOL		isDir;
			
			if ([[NSFileManager defaultManager] fileExistsAtPath:appPath isDirectory:&isDir]) 
			{
						[[NSWorkspace sharedWorkspace] selectFile:appPath inFileViewerRootedAtPath:nil];
			}
		}
}



/*****************************************
 - Called when script type radio button is pressed
*****************************************/

- (IBAction)scriptTypeSelected:(id)sender
{
	[self setScriptType: [[sender selectedCell] tag] ];
}

/*****************************************
 - Updates data in interpreter, icon and script type radio buttons
*****************************************/

- (void)setScriptType: (int)typeNum
{
	NSArray *defaultInterpreters = [NSArray arrayWithObjects: @"/bin/sh",@"/usr/bin/python",@"/usr/bin/osascript",@"/usr/bin/expect",@"/usr/bin/perl",@"/usr/bin/ruby",@"/usr/bin/tclsh",@"/usr/bin/php", nil];
	[interpreterTextField setStringValue: [defaultInterpreters objectAtIndex: typeNum ] ];
	[self setAppIconForType: typeNum];
	[scriptTypeRadioButtons selectCellWithTag: typeNum];
}

/*****************************************
 - Sets the app icon given the script type
*****************************************/

- (NSString *)getInterpreterFromShebang: (NSString *)path
{
	FILE			*file;
	static char		str[1024];
	char			filePath[1024];
	char			len,i;
	NSString		*interpreter;
	
	[path getCString: (char *)&filePath maxLength: sizeof(filePath)];
	
	//open file and read to the first newline character
	file = fopen(filePath, "r");
	fgets(str,1024,file);
	fclose(file);
	
	//doesn't seem to be a shebang line
	if (str[0] != '#' || str[1] != '!')
		return @"";
	
	//jump over the first two characters that indicate the shebang
	strcpy((char *)&str, (char *)&str[2]);
	
	len = strlen((char *)&str);

	for (i = 0; i < len; i++)
	{
		//terminate string when we reach a space or a newline
		if (str[i] == '\n'|| str[i] == ' ')
		{
			str[i] = NULL;
		}
	}
	interpreter = [NSString stringWithCString: (char *)&str];
	
	return interpreter;
}

- (void)setAppIconForType: (int)type
{
	switch(type)
	{
		case 0:
			[appIconImageWell setImage: [NSImage imageNamed:@"PlatypusDflt"]];
			[iconNameTextField setStringValue:@"Platypus Default"];
			break;
		case 1:
			[appIconImageWell setImage: [NSImage imageNamed:@"PythonSnake"]];
			[iconNameTextField setStringValue:@"Python Snake"];
			break;
		case 2:
			[appIconImageWell setImage: [NSImage imageNamed:@"AppleScript"]];
			[iconNameTextField setStringValue:@"AppleScript"];
			break;
		case 3:
			[appIconImageWell setImage: [NSImage imageNamed:@"ExpectMonkey"]];
			[iconNameTextField setStringValue:@"Hexley"];
			break;
		case 4:
			[appIconImageWell setImage: [NSImage imageNamed:@"PerlCamel"]];
			[iconNameTextField setStringValue:@"Perl Camel"];
			break;
		case 5:
			[appIconImageWell setImage: [NSImage imageNamed:@"RubyGoat"]];
			[iconNameTextField setStringValue:@"Ruby Goat"];
			break;
		case 6:
			[appIconImageWell setImage: [NSImage imageNamed:@"TclFlamingo"]];
			[iconNameTextField setStringValue:@"Tcl Flamingo"];
			break;
		case 7:
			[appIconImageWell setImage: [NSImage imageNamed:@"PHPBird"]];
			[iconNameTextField setStringValue:@"PHP Bird"];
			break;
		case 8:
			[appIconImageWell setImage: [NSImage imageNamed:@"PlatypusDroplet"]];
			[iconNameTextField setStringValue:@"Platypus Droplet"];
			break;
		case 9:
			[appIconImageWell setImage: [NSImage imageNamed:@"PlatypusPlate"]];
			[iconNameTextField setStringValue:@"Platypus Plate"];
			break;
	}
}

/*****************************************
 - Open panel to select script to load
*****************************************/

- (IBAction)selectScript:(id)sender
{
	int		result;
	
	//create open panel
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];
    [oPanel setAllowsMultipleSelection:NO];
	[oPanel setCanChooseDirectories: NO];
	
	//run open panel
    result = [oPanel runModalForDirectory:nil file:nil types:nil];
    if (result == NSOKButton) 
			[self loadScript: [oPanel filename]];
}

/*****************************************
 - Loads script data into platypus window
*****************************************/

- (void)loadScript:(NSString *)filename
{
	char		cStr[1024];
	NSString	*shebangInterpreter;
	int			i;

	//set script path
	[scriptPathTextField setStringValue: filename];

	//set app name
	NSString *appName = [filename lastPathComponent];
	[appName getCString: cStr maxLength: sizeof(cStr)];
	CutSuffix((char *)&cStr);
	appName = [NSString stringWithCString: cStr length: strlen(cStr)];
	[appNameTextField setStringValue: appName];
	
	//interpreter
	
	//read shebang line
	//if ([[NSUserDefaults standardUserDefaults] boolForKey:@"ParseShebangLineForInterpreter"] == 1)
	//{
		shebangInterpreter = [self getInterpreterFromShebang: filename];
	/*}
	else
	{
		shebangInterpreter = @"";
	}*/
	
	//if no interpreter can be retrieved from shebang line
	if ([shebangInterpreter caseInsensitiveCompare: @""] == NSOrderedSame)
	{
		//try to determine type from suffix
		[self setScriptType: (int)[self getFileTypeFromSuffix: filename] ];
	}
	else
	{
		//see if interpreter matches a preset
		NSArray *defaultInterpreters = [NSArray arrayWithObjects: @"/bin/sh",@"/usr/bin/python",@"/usr/bin/osascript",@"/usr/bin/expect",
																  @"/usr/bin/perl",@"/usr/bin/ruby",@"/usr/bin/tclsh",@"/usr/bin/php", nil];
		for (i = 0; i < [defaultInterpreters count]; i++)
		{
			if ([shebangInterpreter caseInsensitiveCompare: [defaultInterpreters objectAtIndex: i]] == NSOrderedSame)
			{
				[self setScriptType: i];
			}
		}
		
		//set shebang interpreter into interpreter field
		[interpreterTextField setStringValue: shebangInterpreter ];
	}
	
	//enable the edit/reveal/create buttons
	[createAppButton setEnabled: YES];
	[editScriptButton setEnabled: YES];
	[revealScriptButton setEnabled: YES];
	
}

/*****************************************
 - Toggles between advanced options mode
*****************************************/

- (IBAction)toggleAdvancedOptions:(id)sender
{
	#define		kWindowExpansionHeight  242
	NSWindow *theWindow = [showOptionsTextField window];
	NSRect winRect = [theWindow frame];
	
	if ([sender state] == NSOffState)
	{
		winRect.origin.y += kWindowExpansionHeight;
		winRect.size.height -= kWindowExpansionHeight;
		[showOptionsTextField setStringValue: @"Show advanced options"];
		[window setFrame: winRect display:TRUE];
	}
	else if ([sender state] == NSOnState)
	{
		winRect.origin.y -= kWindowExpansionHeight;
		winRect.size.height += kWindowExpansionHeight;
		[showOptionsTextField setStringValue: @"Hide advanced options"];
		[window setFrame: winRect display:TRUE];
	}
}


/*****************************************
 - Flip between icons
*****************************************/
- (IBAction)switchIcons:(id)sender
{
	[self setAppIconForType: [sender intValue]];
}

- (IBAction)nextIcon:(id)sender
{
	if ([iconToggleButton intValue] + 1 > [iconToggleButton maxValue])
		[iconToggleButton setIntValue: [iconToggleButton minValue]];
	else
		[iconToggleButton setIntValue: [iconToggleButton intValue] + 1];
	
	[self setAppIconForType: [iconToggleButton intValue]];
}

- (IBAction)previousIcon:(id)sender
{
	if ([iconToggleButton intValue] - 1 < [iconToggleButton minValue])
		[iconToggleButton setIntValue: [iconToggleButton maxValue]];
	else
		[iconToggleButton setIntValue: [iconToggleButton intValue] - 1];
	
	[self setAppIconForType: [iconToggleButton intValue]];
}

/*****************************************
 - Write an NSImage as icon to a path
*****************************************/

- (void)writeImageAsIcon: (NSImage *)img path: (NSString *)path
{
	IconFamily *iconFam = [[IconFamily alloc] initWithThumbnailsOfImage: img];
	[iconFam writeToFile: path];
}

/*****************************************
 - Generate dictionary for Info.plist in app bundle
*****************************************/

- (NSDictionary *)getInfoPropertyListDictionary: (NSString *)appName isDroppable: (BOOL)droppable
{
	
	NSDictionary *infoPlist;
	NSDictionary *fileTypesDict;

	if (!droppable)
	{
		infoPlist = [NSDictionary dictionaryWithObjectsAndKeys:
								@"English", @"CFBundleDevelopmentRegion",
								appName, @"CFBundleExecutable", 
								@"appIcon.icns", @"CFBundleIconFile",  
								@"1.0", @"CFBundleVersion",  
								@"org.platypus.app", @"CFBundleIdentifier",  
								@"MainMenu", @"NSMainNibFile",  
								@"NSApplication", @"NSPrincipalClass",  nil];

	}
	else
	{
		fileTypesDict = [NSDictionary dictionaryWithObjectsAndKeys:
								[NSArray arrayWithObject: @"*"], @"CFBundleTypeExtensions",
								[NSArray arrayWithObject: @"*"], @"CFBundleTypeMIMETypes",
								[NSArray arrayWithObject: @"****"], @"CFBundleTypeOSTypes",
								@"Editor", @"CFBundleTypeRole", nil];
		
		NSArray *ftArray = [NSArray arrayWithObject: fileTypesDict];
	
		infoPlist = [NSDictionary dictionaryWithObjectsAndKeys:
								@"English", @"CFBundleDevelopmentRegion",
								appName, @"CFBundleExecutable",
								ftArray,@"CFBundleDocumentTypes",
								@"appIcon.icns", @"CFBundleIconFile",  
								@"1.0", @"CFBundleVersion", 
								@"org.platypus.app", @"CFBundleIdentifier",  
								@"MainMenu", @"NSMainNibFile",  
								@"NSApplication", @"NSPrincipalClass", nil];
		
	//	[ftArray release];
								
	}
	return(infoPlist);
}


/*****************************************
 - Determine script type based on a file's suffix
*****************************************/

- (int)getFileTypeFromSuffix: (NSString *)fileName
{
	if ([fileName hasSuffix: @".sh"])
		return 0;
	else if ([fileName hasSuffix: @".py"])
		return 1;
	else if ([fileName hasSuffix: @".scpt"] || [fileName hasSuffix: @".applescript"])
		return 2;
	else if ([fileName hasSuffix: @".exp"] || [fileName hasSuffix: @".expect"])
		return 3;
	else if ([fileName hasSuffix: @".pl"] || [fileName hasSuffix: @".perl"])
		return 4;
	else if ([fileName hasSuffix: @".rb"] || [fileName hasSuffix: @".rbx"])
		return 5;
	else if ([fileName hasSuffix: @".tcl"])
		return 6;
	else if ([fileName hasSuffix: @".php"])
		return 7;
	else
		return 0;
}

#pragma mark -

- (IBAction)showPrefs:(id)sender
{
	[parseShebangCheckbox setState: [[NSUserDefaults standardUserDefaults] boolForKey:@"ParseShebangLineForInterpreter"]];
	[revealAppCheckbox setState: [[NSUserDefaults standardUserDefaults] boolForKey:@"RevealApplicationWhenCreated"]];
	[defaultEditorMenu setTitle: [[NSUserDefaults standardUserDefaults] stringForKey:@"DefaultEditor"]];
	
	[prefsWindow center];
	[prefsWindow makeKeyAndOrderFront: sender];
}

- (IBAction)applyPrefs:(id)sender
{
	[[NSUserDefaults standardUserDefaults] setBool: [parseShebangCheckbox state] forKey:@"ParseShebangLineForInterpreter"];
	[[NSUserDefaults standardUserDefaults] setBool: [revealAppCheckbox state]  forKey:@"RevealApplicationWhenCreated"];
	[[NSUserDefaults standardUserDefaults] setObject: [defaultEditorMenu titleOfSelectedItem]  forKey:@"DefaultEditor"];
	
/*	NSLog(@"Parse Shabang: %d", [[NSUserDefaults standardUserDefaults] boolForKey:@"ParseShebangLineForInterpreter"]);
	NSLog(@"Reveal App: %d", [[NSUserDefaults standardUserDefaults] boolForKey:@"RevealApplicationWhenCreated"]);
	NSLog(@"Editor: %@", [[NSUserDefaults standardUserDefaults] stringForKey:@"DefaultEditor"]);*/
	
	[prefsWindow performClose: sender];
}


#pragma mark -

/*****************************************
 - Dragging and dropping for Platypus window
*****************************************/

- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
{
	NSPasteboard *pboard = [sender draggingPasteboard];

    if ( [[pboard types] containsObject:NSFilenamesPboardType] ) 
	{
        NSArray *files = [pboard propertyListForType:NSFilenamesPboardType];
		[self loadScript: [files objectAtIndex: 0]];
    }

    return YES;

}

- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender 
{

    NSPasteboard *pboard;
    NSDragOperation sourceDragMask;

    sourceDragMask = [sender draggingSourceOperationMask];
    pboard = [sender draggingPasteboard];


    if ( [[pboard types] containsObject:NSFilenamesPboardType] ) 
	{
        if (sourceDragMask & NSDragOperationLink) 
            return NSDragOperationLink;
		else if (sourceDragMask & NSDragOperationCopy)
            return NSDragOperationCopy;
    }

    return NSDragOperationNone;
}

#pragma mark -

- (void)controlTextDidChange:(NSNotification *)aNotification
{
	BOOL	isDir;

	if ([[NSFileManager defaultManager] fileExistsAtPath: [scriptPathTextField stringValue] isDirectory:&isDir] && !isDir)
	{
		[createAppButton setEnabled: YES];
		[editScriptButton setEnabled: YES];
		[revealScriptButton setEnabled: YES];
	}
	else
	{
		[createAppButton setEnabled: NO];
		[editScriptButton setEnabled: NO];
		[revealScriptButton setEnabled: NO];
	}
}


- (BOOL)validateMenuItem:(NSMenuItem*)anItem 
{
	BOOL isDir;

	//edit script
	if ([[anItem title] isEqualToString:@"Edit Script"] && (![[NSFileManager defaultManager] fileExistsAtPath: [scriptPathTextField stringValue] isDirectory:&isDir] || isDir))
	{
		return NO;
	}
	//reveal script
	if ([[anItem title] isEqualToString:@"Reveal Script"] && (![[NSFileManager defaultManager] fileExistsAtPath: [scriptPathTextField stringValue] isDirectory:&isDir] || isDir))
	{
		return NO;
	}
	//create app menu
	if ([[anItem title] isEqualToString:@"Create App"] && (![[NSFileManager defaultManager] fileExistsAtPath: [scriptPathTextField stringValue] isDirectory:&isDir] || isDir))
	{
		return NO;
	}
	

	//File list menu items
	if ([[anItem title] isEqualToString:@"Add File To Bundle"] && [showAdvancedArrow state] == NSOffState)
		return NO;
		
    if ([[anItem title] isEqualToString:@"Remove File"] && ([fileList numFiles] == 0 || [showAdvancedArrow state] == NSOffState))
		return NO;

    if ([[anItem title] isEqualToString:@"Reveal File In File List"] && ([fileList numFiles] == 0 || [showAdvancedArrow state] == NSOffState))
		return NO;
	
	if ([[anItem title] isEqualToString:@"Clear File List"] && ([fileList numFiles] == 0 || [showAdvancedArrow state] == NSOffState))
		return NO;

    return YES;

}

@end
