#import "ScriptExecController.h"
#import "STUtil.h"
#include <Security/Authorization.h>
#include <sys/wait.h>

@implementation ScriptExecController



- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
	NSBundle	*appBundle = [NSBundle mainBundle];
	
	//load settings from app bundle
	[self loadSettings];
	
	//create progress window if so specified
	if (outputType == kProgressBarOutput)
	{
		//prepare progress bar
		[progressBar setUsesThreadedAnimation: YES];
		[progressBar startAnimation: self];
		//center and display window
		[progressWindow center];
		[progressWindow makeKeyAndOrderFront: NULL];
	}
	else if (outputType == kTextOutput)
	{   
		[textOutputWindow center];
		[textOutputWindow makeKeyAndOrderFront: NULL];
	}

	//create argument list
	arguments = [NSMutableArray arrayWithCapacity: 257];
	[arguments addObject: scriptPath];
	[arguments addObject: [appBundle bundlePath]]; //$1 is path of application bundle
	if (fileArgs != NULL)//if there are any dropped files, we add them as arguments after $1
		[arguments addObjectsFromArray: fileArgs];

	//start new thread for executing script
	if (execStyle == kPrivilegedExecution)
	{
		//Authentication mode
		[NSThread detachNewThreadSelector: @selector(executeScriptWithPrivileges) toTarget: self withObject: NULL];
	}
	else
	{
		//plain old regular
		[NSThread detachNewThreadSelector: @selector(executeScript) toTarget: self withObject: NULL];
	}
}



//launch task with NSTask
- (void)executeScript
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	
	NSData *outData;
	NSPipe *outPipe = [NSPipe pipe];
	NSFileHandle *outReader = [outPipe fileHandleForReading];
	
	//initalize task
	task = [[NSTask alloc] init];
	
	//apply settings for task
	[task setLaunchPath: interpreter];
	[task setArguments: arguments];
	[task setStandardOutput:outPipe];
	
	//set it off
	[task launch];
	
	while ([task isRunning])
	{
		outData = [outReader readDataToEndOfFile];
		NSString	*str = [[NSString alloc] initWithData: outData encoding:  NSASCIIStringEncoding];
		
		[textOutputTextField setStringValue: @"Hello"];
	}
	
	//[task waitUntilExit];
	
	close([outReader fileDescriptor]);
	[[NSApplication sharedApplication] terminate: self];
}


//launch task using Authentication Manager
- (void)executeScriptWithPrivileges
{

	OSErr					err = noErr;
    AuthorizationRef 		authorizationRef;
    short					i;
    char					*args[257];
	char					interpreterStr[1024];
	
	//interpreter
	[interpreter getCString: (char *)&interpreterStr maxLength: 1024];
	
	//arguments array
	for (i = 0; i < [arguments count]; i++)
	{
		args[i] = malloc(1024);
		[[arguments objectAtIndex:i] getCString: (char *)args[i] maxLength: 1024];
		if (i == [arguments count]-1)
		{
			args[i+1] = NULL;
		}
	}
    
    // Use Apple's Authentication Manager APIs to run script with root privileges
    err = AuthorizationCreate(NULL, kAuthorizationEmptyEnvironment, kAuthorizationFlagDefaults, &authorizationRef);
    if (err != errAuthorizationSuccess)
        [[NSApplication sharedApplication] terminate: self];

    if ((err = AuthorizationExecuteWithPrivileges(authorizationRef,(char *)&interpreterStr, kAuthorizationFlagDefaults, args, NULL)) != noErr)
    {
		[[NSApplication sharedApplication] terminate: self];
        //some kind of error took place -- most likely, authentication failed
    }
    else
    {
            //wait for child process to finish
            int child;
            wait(&child);
			[[NSApplication sharedApplication] terminate: self];
    }

}


//respond to AEOpenDoc
- (BOOL)application:(NSApplication *)theApplication openFile:(NSString *)filename
{
	//init array for receiving file list 
	if (fileArgs == NULL)
		fileArgs = [NSMutableArray arrayWithCapacity: 255];

	[fileArgs addObject: filename];

	return(TRUE);

}

- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender
{
		//terminate task
	if (task != NULL && [task isRunning])
	{
		[task terminate];
		[task release];
	}

	return(YES);
}

//load configuration files from application bundle
- (void)loadSettings
{
	NSBundle		*appBundle = [NSBundle mainBundle];
	NSFileManager   *fmgr = [NSFileManager defaultManager];
	
	//make sure all the config files are present
	if (![fmgr fileExistsAtPath: [appBundle pathForResource:@"interpreter" ofType:nil]] ||
		![fmgr fileExistsAtPath: [appBundle pathForResource:@"script" ofType:nil]] ||
		![fmgr fileExistsAtPath: [appBundle pathForResource:@"outputType" ofType:nil]] ||
		![fmgr fileExistsAtPath: [appBundle pathForResource:@"auth" ofType:nil]])
		{
			[STUtil fatalAlert: @"Corrupted app bundle" subText: @"Some configuration files were missing from the application bundle."];
		}
	
	//get interpreter
	interpreter = [NSString stringWithContentsOfFile: [appBundle pathForResource:@"interpreter" ofType:nil]];
	if (![fmgr fileExistsAtPath: interpreter])
	{
		[STUtil fatalAlert: @"Missing interpreter" subText: [NSString stringWithFormat:@"Your system lacks the required interpreter ('%@') to execute this program.", interpreter]];
	}
	
	//get path to script
	scriptPath = [appBundle pathForResource:@"script" ofType:nil];
	
	//determine output type
	NSString *outputTypeStr = [NSString stringWithContentsOfFile: [appBundle pathForResource:@"outputType" ofType:nil]];
	if ([outputTypeStr isEqualToString: @"ProgressBar"])
		outputType = kProgressBarOutput;
	else if ([outputTypeStr isEqualToString: @"Text"])
		outputType = kTextOutput;
	else
		outputType = kNoOutput;
		
	//determine execution style
	NSString *execStyleStr = [NSString stringWithContentsOfFile: [appBundle pathForResource:@"auth" ofType:nil]];
	if ([execStyleStr isEqualToString: @"Auth"])
		execStyle = kPrivilegedExecution;
	else
		execStyle = kNormalExecution;
}



// Respond to Cancel by exiting application
- (IBAction)cancel:(id)sender
{
	[[NSApplication sharedApplication] terminate: self];
}




@end
