#import "IconController.h"
#import "IconFamily.h"
#import "STUtil.h"

@implementation IconController

/*****************************************
 - init function
*****************************************/
- (id)init
{
	if (self = [super init]) 
	{
		usesIcnsFile = FALSE;
	}
    return self;
}

- (IBAction)copyIcon:(id)sender
{
	[[NSPasteboard generalPasteboard] declareTypes: [NSArray arrayWithObject: NSTIFFPboardType] owner: self];
	[[NSPasteboard generalPasteboard] setData: [[iconImageWell image] TIFFRepresentation] forType: NSTIFFPboardType];
}

- (IBAction)pasteIcon:(id)sender
{
	NSImage *pastedIcon = [[[NSImage alloc] initWithPasteboard: [NSPasteboard generalPasteboard]] autorelease];
	if (pastedIcon != NULL)
	{
		[iconImageWell setImage: pastedIcon];
		[self iconDidChange: self];
		usesIcnsFile = 0;
	}
}

/*****************************************
 - Responds when user drags custom icon on image well
*****************************************/

- (IBAction) iconDidChange:(id)sender
{
	[iconNameTextField setStringValue:@"Custom Icon"];
	[iconImageWell performClick: self];//prevents the background from graying out
}

- (IBAction)nextIcon:(id)sender
{
	if ([iconToggleButton intValue] + 1 > [iconToggleButton maxValue])
		[iconToggleButton setIntValue: [iconToggleButton minValue]];
	else
		[iconToggleButton setIntValue: [iconToggleButton intValue] + 1];
	
	[self setAppIconForType: [iconToggleButton intValue]];

}


- (IBAction)previousIcon:(id)sender
{
	if ([iconToggleButton intValue] - 1 < [iconToggleButton minValue])
		[iconToggleButton setIntValue: [iconToggleButton maxValue]];
	else
		[iconToggleButton setIntValue: [iconToggleButton intValue] - 1];
	
	[self setAppIconForType: [iconToggleButton intValue]];
}

/*****************************************
 - Each script type has a certain icon associated with it.
 - Set the icon according to the type specified
*****************************************/

- (void)setAppIconForType: (int)type
{
	switch(type)
	{
		case 0:
			[iconImageWell setImage: [NSImage imageNamed:@"PlatypusDflt"]];
			[iconNameTextField setStringValue:@"Platypus Default"];
			break;
		case 1:
			[iconImageWell setImage: [NSImage imageNamed:@"PythonSnake"]];
			[iconNameTextField setStringValue:@"Python Snake"];
			break;
		case 2:
			[iconImageWell setImage: [NSImage imageNamed:@"AppleScript"]];
			[iconNameTextField setStringValue:@"AppleScript"];
			break;
		case 3:
			[iconImageWell setImage: [NSImage imageNamed:@"ExpectMonkey"]];
			[iconNameTextField setStringValue:@"Expect Monkey"];
			break;
		case 4:
			[iconImageWell setImage: [NSImage imageNamed:@"PerlCamel"]];
			[iconNameTextField setStringValue:@"Perl Camel"];
			break;
		case 5:
			[iconImageWell setImage: [NSImage imageNamed:@"RubyGoat"]];
			[iconNameTextField setStringValue:@"Ruby Goat"];
			break;
		case 6:
			[iconImageWell setImage: [NSImage imageNamed:@"TclFlamingo"]];
			[iconNameTextField setStringValue:@"Tcl Flamingo"];
			break;
		case 7:
			[iconImageWell setImage: [NSImage imageNamed:@"PHPBird"]];
			[iconNameTextField setStringValue:@"PHP Bird"];
			break;
		case 8:
			[iconImageWell setImage: [NSImage imageNamed:@"PlatypusDroplet"]];
			[iconNameTextField setStringValue:@"Platypus Droplet"];
			break;
		case 9:
			[iconImageWell setImage: [NSImage imageNamed:@"PlatypusPlate"]];
			[iconNameTextField setStringValue:@"Platypus Plate"];
			break;
		case 10:
			[iconImageWell setImage: [NSImage imageNamed:@"installer"]];
			[iconNameTextField setStringValue:@"Platypus Installer"];
			break;
			
	}
	usesIcnsFile = 0;
}

- (IBAction)switchIcons:(id)sender
{
	[self setAppIconForType: [sender intValue]];

}

- (IBAction)selectIcnsFile:(id)sender
{
	NSOpenPanel *oPanel = [NSOpenPanel openPanel];
	[oPanel setPrompt:@"Select"];
    [oPanel setAllowsMultipleSelection:NO];
	[oPanel setCanChooseDirectories: NO];
	
	[window setTitle: @"Platypus - Select an icns file"];

	//run open panel
    [oPanel beginSheetForDirectory:nil file:nil types: [NSArray arrayWithObject: @"icns"] modalForWindow: window modalDelegate: self didEndSelector: @selector(selectIcnsFileDidEnd:returnCode:contextInfo:) contextInfo: nil];
}

- (void)selectIcnsFileDidEnd:(NSOpenPanel *)oPanel returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	if (returnCode == NSOKButton)
	{
		NSImage *customIcon = [[NSImage alloc] initWithContentsOfFile: [oPanel filename]];
		if (customIcon != NULL)
		{
			icnsFilePath = [[oPanel filename] retain];
			[iconImageWell setImage: customIcon];
			[customIcon autorelease];
			[self iconDidChange: self];
			usesIcnsFile = 1;
		}
		else
			[STUtil alert:@"Corrupt Image File" subText: @"The image file you selected appears to be damaged or corrupt."];
	}
	[window setTitle: @"Platypus"];
}

/*****************************************
 - Write an NSImage as icon to a path
*****************************************/

- (void)writeIconToPath: (NSString *)path
{
	if ([iconImageWell image] == NULL)
		[STUtil alert:@"Icon Error" subText: @"No icon could be found for your application.  Please set an icon to fix this."];

	IconFamily *iconFam = [[IconFamily alloc] initWithThumbnailsOfImage: [iconImageWell image]];
	[iconFam writeToFile: path];
	[iconFam release];
}

/*****************************************
 - Put up dialog prompting file to take icon from
*****************************************/
- (IBAction) importIcon:(id)sender
{
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];
	[oPanel setPrompt:@"Select"];
    [oPanel setAllowsMultipleSelection:NO];
	[oPanel setCanChooseDirectories: YES];
	
	[window setTitle: @"Platypus - Import a file's icon"];

	//run open panel
    [oPanel beginSheetForDirectory:nil file:nil types: nil modalForWindow: window modalDelegate: self didEndSelector: @selector(importIconDidEnd:returnCode:contextInfo:) contextInfo: nil];
	
}

- (void)importIconDidEnd:(NSOpenPanel *)oPanel returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	if (returnCode == NSOKButton)
	{
		IconFamily  *icon = [IconFamily iconFamilyWithIconOfFile: [oPanel filename]];
		if (icon != NULL)
		{
			[iconImageWell setImage: [icon imageWithAllReps]];
			[self iconDidChange: self];
			usesIcnsFile = 0;
		}
		else
			[STUtil alert:@"Error getting icon" subText: @"The icon of the file you selected appears to be damaged or corrupt."];
	}
	[window setTitle: @"Platypus"];
}


/*****************************************
 - Put up dialog prompting for image file to use as icon
*****************************************/
- (IBAction) selectIcon:(id)sender
{
	//file types acceptable as custom icon data -- I think that's the lot that Apple's APIs can handle
	NSArray *fileTypes = [NSArray arrayWithObjects: @"icns", @"jpg", @"jpeg",@"gif",@"png",@"tif",@"tiff",@"bmp",@"pict",@"psd",@"pdf",@"tga",@"sgi",@"pntg", nil];

    NSOpenPanel *oPanel = [NSOpenPanel openPanel];
	[oPanel setPrompt:@"Select"];
    [oPanel setAllowsMultipleSelection:NO];
	[oPanel setCanChooseDirectories: NO];
	
	[window setTitle: @"Platypus - Select an image file"];

	//run open panel
    [oPanel beginSheetForDirectory:nil file:nil types: fileTypes modalForWindow: window modalDelegate: self didEndSelector: @selector(selectIconDidEnd:returnCode:contextInfo:) contextInfo: nil];
		
}

- (void)selectIconDidEnd:(NSOpenPanel *)oPanel returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	if (returnCode == NSOKButton)
	{
		NSImage *customIcon = [[NSImage alloc] initWithContentsOfFile: [oPanel filename]];
		if (customIcon != NULL)
		{
			[iconImageWell setImage: customIcon];
			[customIcon autorelease];
			[self iconDidChange: self];
			usesIcnsFile = 0;
		}
		else
			[STUtil alert:@"Corrupt Image File" subText: @"The image file you selected appears to be damaged or corrupt."];
	}
	[window setTitle: @"Platypus"];
}

- (BOOL) hasCustomIcon
{
	if ([[iconNameTextField stringValue] isEqualToString: @"Custom Icon"])
		return YES;
	
	return NO;
}

- (void) setImage: (NSImage *)img
{
	[iconImageWell setImage: img];
	[iconNameTextField setStringValue: @"Custom Icon"];
}

- (NSImage *)getImage
{
	return [iconImageWell image];
}

- (BOOL) usesIcnsFile
{
	return (BOOL)usesIcnsFile;
}

- (NSString *) getIcnsFilePath
{
	if (![[NSFileManager defaultManager] fileExistsAtPath: icnsFilePath])
	{
		[STUtil alert: @"Missing icns file" subText: [NSString stringWithFormat: @"The icns file %@ could not be found at the location you specified", icnsFilePath]];
		return @"";
	}
	return icnsFilePath;
}

@end
