/*
    Platypus - program for creating Mac OS X application wrappers around scripts
    Copyright (C) 2005 Sveinbjorn Thordarson <sveinbt@hi.is>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#import "FileList.h"


@implementation FileList

- (id) init
{
		files = [[NSMutableArray alloc] init];
		return self;
}

-(void)dealloc
{
	[files release];
}



- (NSString *)getFileAtIndex:(int)index
{
	return [[files objectAtIndex: index] objectAtIndex: 0];
}

- (void) addFile: (NSString *)fileName
{
	NSImage *icon = [[NSWorkspace sharedWorkspace] iconForFile: fileName];
	[files addObject: [NSArray arrayWithObjects: fileName, icon, nil]];
}

- (void) addFiles: (NSArray *)fileNames
{
	NSImage *icon;
	int i;
	for (i = 0; i < [fileNames count]; i++)
	{
		icon = [[NSWorkspace sharedWorkspace] iconForFile: [fileNames objectAtIndex: i]];
		[files addObject: [NSArray arrayWithObjects: [fileNames objectAtIndex: i], icon, nil]];
	}
}

- (BOOL) hasFile: (NSString *)fileName
{
	int i;
	
	for (i = 0; i < [files count]; i++)
	{
		if ([[[files objectAtIndex: i] objectAtIndex: 0] isEqualToString: fileName])
			return YES;
	}
	return NO;
}

- (void) clearList
{
	[files removeAllObjects];
}

- (void) removeFile: (int)index
{
	[files removeObjectAtIndex: index];
}

- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
	return([files count]);
}

- (int)numFiles
{
	return([files count]);
}

- (NSArray *)getFilesArray
{
	NSMutableArray	*fileNames = [NSMutableArray arrayWithCapacity: 255];
	int				i;
	
	for (i = 0; i < [files count]; i++)
	{
		[fileNames addObject: [[files objectAtIndex: i] objectAtIndex: 0]];
	}

	return fileNames;
}


- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
{
	if ([[aTableColumn identifier] caseInsensitiveCompare: @"2"] == NSOrderedSame)
	{
		return([[files objectAtIndex: rowIndex] objectAtIndex: 0]);
	}
	else if ([[aTableColumn identifier] caseInsensitiveCompare: @"1"] == NSOrderedSame)
	{
		if (rowIndex == 0)
		{
			NSImageCell* iconCell;
			iconCell = [[[NSImageCell alloc] init] autorelease];
			[aTableColumn setDataCell:iconCell];
		}
        
        return [[files objectAtIndex: rowIndex] objectAtIndex: 1];
	}
	return(@"");
}


- (void)revealInFinder: (int)index
{
	BOOL		isDir;
	NSString	*path = [[files objectAtIndex: index] objectAtIndex: 0];
	
	if ([[NSFileManager defaultManager] fileExistsAtPath:path isDirectory:&isDir]) 
	{
            if (isDir)
				[[NSWorkspace sharedWorkspace] selectFile:nil inFileViewerRootedAtPath:path];
            else
				[[NSWorkspace sharedWorkspace] selectFile:path inFileViewerRootedAtPath:nil];
	}
}
@end
