/* Platypus */

#import <Cocoa/Cocoa.h>
#import "FileList.h"
#import "TypesList.h"
#import "IconFamily.h"
#import "STUtil.h"

@interface Platypus : NSObject
{
    IBOutlet id appIconImageWell;
    IBOutlet id appNameTextField;
    IBOutlet id fileListDataBrowser;
    IBOutlet id interpreterTextField;
    IBOutlet id isDroppableCheckbox;
    IBOutlet id outputTypeRadioButtons;
    IBOutlet id receivesAppPathCheckbox;
    IBOutlet id rootPrivilegesCheckbox;
    IBOutlet id scriptPathTextField;
    IBOutlet id scriptTypeRadioButtons;
    IBOutlet id showOptionsTextField;
	IBOutlet id iconNameTextField;
	IBOutlet id showAdvancedArrow;
	IBOutlet id editScriptButton;
	IBOutlet id revealScriptButton;
	IBOutlet id createAppButton;
	IBOutlet id iconToggleButton;
	IBOutlet id showInDockCheckbox;
	IBOutlet id editTypesButton;
	IBOutlet id numTypesTextField;
	
	//file list
	IBOutlet id addFileButton;
	IBOutlet id removeFileButton;
	IBOutlet id revealFileButton;
	IBOutlet id clearFileListButton;
	
	//types list
	IBOutlet id suffixTextField;
	IBOutlet id typesListDataBrowser;
	
	//windows
	IBOutlet id window;
	IBOutlet id prefsWindow;
	IBOutlet id typesWindow;
	
	//prefs
	IBOutlet id parseShebangCheckbox;
    IBOutlet id revealAppCheckbox;
    IBOutlet id defaultEditorMenu;
	
	FileList			*fileList;
	TypesList			*typesList;
	NSMutableArray		*defaultIcons;
	int					currentIcon;
}

- (IBAction)createApp:(id)sender;
- (IBAction)editScript:(id)sender;
- (IBAction)newScript:(id)sender;
- (IBAction)revealScript:(id)sender;

- (IBAction)scriptTypeSelected:(id)sender;
- (IBAction)selectScript:(id)sender;
- (IBAction)toggleAdvancedOptions:(id)sender;

- (IBAction)switchIcons:(id)sender;
- (IBAction)nextIcon:(id)sender;
- (IBAction)previousIcon:(id)sender;

- (IBAction)isDroppableWasClicked:(id)sender;


- (IBAction)showPrefs:(id)sender;
- (IBAction)applyPrefs:(id)sender;

- (NSDictionary *)getInfoPropertyListDictionary: (NSString *)appName isDroppable: (BOOL)droppable inBackground: (BOOL)background;

- (void)writeImageAsIcon: (NSImage *)img path: (NSString *)path;
- (void)setScriptType: (int)typeNum;
- (void)loadScript:(NSString *)filename;
- (int)getFileTypeFromSuffix: (NSString *)fileName;
- (void)setAppIconForType: (int)type;

- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender;
- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender;

- (NSString *)generateBundleIdentifier;
- (BOOL)validateMenuItem:(NSMenuItem*)anItem;

// File List
- (IBAction)addFileToFileList:(id)sender;
- (IBAction)clearFileList:(id)sender;
- (IBAction)removeFileFromFileList:(id)sender;
- (IBAction)revealFileInFileList:(id)sender;

// Types List
- (IBAction) addType:(id)sender;
- (IBAction) removeType:(id)sender;
- (IBAction) clearTypesList:(id)sender;
- (IBAction) setDefaultTypes:(id)sender;

//Help
- (IBAction) showHelp:(id)sender;
@end
