/* Platypus */

#import <Cocoa/Cocoa.h>
#import "FileList.h"
#import "SuffixList.h"
#import "TypesList.h"
#import "IconFamily.h"
#import "STUtil.h"

@interface Platypus : NSObject
{
    IBOutlet id appIconImageWell;
    IBOutlet id appNameTextField;
	IBOutlet id appIconView;
    
    IBOutlet id interpreterTextField;
    IBOutlet id isDroppableCheckbox;
    IBOutlet id outputTypeRadioButtons;
    IBOutlet id rootPrivilegesCheckbox;
	IBOutlet id remainRunningCheckbox;
    IBOutlet id scriptPathTextField;
    IBOutlet id scriptTypeRadioButtons;
    IBOutlet id showOptionsTextField;
	IBOutlet id iconNameTextField;
	IBOutlet id showAdvancedArrow;
	IBOutlet id editScriptButton;
	IBOutlet id revealScriptButton;
	IBOutlet id createAppButton;
	IBOutlet id iconToggleButton;
	IBOutlet id showInDockCheckbox;
	IBOutlet id editTypesButton;
	IBOutlet id appFunctionRadioButtons;
	IBOutlet id toggleAdvancedMenuItem;
	
	IBOutlet id versionTextField;
	IBOutlet id signatureTextField;
	
	IBOutlet id openRecentMenu;
	IBOutlet id profilesMenu;
	
	IBOutlet id bundleIdentifierTextField;
	IBOutlet id bundleSizeTextField;
	
	//file list
	IBOutlet id addFileButton;
	IBOutlet id removeFileButton;
	IBOutlet id revealFileButton;
	IBOutlet id clearFileListButton;
	IBOutlet id fileListDataBrowser;
	
	IBOutlet id typesErrorTextField;
	
	//suffix list
	IBOutlet id suffixTextField;
	IBOutlet id suffixListDataBrowser;
	IBOutlet id numSuffixesTextField;
	IBOutlet id removeSuffixButton;
	IBOutlet id addSuffixButton;
	
	//types list
	IBOutlet id typeCodeTextField;
	IBOutlet id typesListDataBrowser;
	IBOutlet id numTypesTextField;
	IBOutlet id removeTypeButton;
	IBOutlet id addTypeButton;
	
	//windows
	IBOutlet id window;
	IBOutlet id prefsWindow;
	IBOutlet id typesWindow;
	
	//prefs
	IBOutlet id parseShebangCheckbox;
    IBOutlet id revealAppCheckbox;
    IBOutlet id defaultEditorMenu;
	
	
	FileList			*fileList;
	TypesList			*typesList;
	SuffixList			*suffixList;
	NSMutableArray		*defaultIcons;
	NSMutableArray		*recentItems;
	int					currentIcon;
}

- (IBAction)createButtonPressed: (id)sender;
- (void)createApp:(NSSavePanel *)sPanel returnCode:(int)result contextInfo:(void *)contextInfo;
- (IBAction)editScript:(id)sender;
- (IBAction)newScript:(id)sender;
- (IBAction)revealScript:(id)sender;
- (IBAction)runScript:(id)sender;

- (IBAction)scriptTypeSelected:(id)sender;
- (IBAction)selectScript:(id)sender;
- (void)selectScriptPanelDidEnd:(NSOpenPanel *)oPanel returnCode:(int)returnCode contextInfo:(void *)contextInfo;
- (IBAction)toggleAdvancedOptions:(id)sender;
//- (IBAction)outputTypeChanged: (id)sender;

- (BOOL)verifyFieldContents;

- (IBAction)switchIcons:(id)sender;
- (IBAction)nextIcon:(id)sender;
- (IBAction)previousIcon:(id)sender;
- (IBAction)selectCustomIcon:(id)sender;
- (IBAction)importCustomIcon:(id)sender;

- (IBAction)isDroppableWasClicked:(id)sender;

- (IBAction)clearAllFields:(id)sender;
- (IBAction)showPrefs:(id)sender;
- (IBAction)applyPrefs:(id)sender;
- (void)setIconsForEditorMenu;

- (NSDictionary *)getInfoPropertyListDictionary: (NSString *)appName isDroppable: (BOOL)droppable inBackground: (BOOL)background;

- (void)writeImageAsIcon: (NSImage *)img path: (NSString *)path;
- (void)setScriptType: (int)typeNum;
- (void)loadScript:(NSString *)filename;
- (int)getFileTypeFromSuffix: (NSString *)fileName;
- (void)setAppIconForType: (int)type;

- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender;
- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender;

- (void)updateFileSizeField;
- (NSString *)generateBundleIdentifier;
- (BOOL)validateMenuItem:(NSMenuItem*)anItem;
-(void) constructOpenRecentMenu;

//profiles
- (IBAction) saveProfile:(id)sender;
- (void) profileMenuItemSelected: (id)sender;
- (IBAction) clearAllProfiles:(id)sender;
- (void) constructProfilesMenu;

// File List
- (IBAction)addFileToFileList:(id)sender;
- (void)addFilesPanelDidEnd:(NSOpenPanel *)oPanel returnCode:(int)returnCode contextInfo:(void *)contextInfo;
- (IBAction)clearFileList:(id)sender;
- (IBAction)removeFileFromFileList:(id)sender;
- (IBAction)revealFileInFileList:(id)sender;

// Types List
- (IBAction) addType:(id)sender;
- (IBAction) removeType:(id)sender;
- (IBAction) clearTypesList:(id)sender;

// Suffix List
- (IBAction) addSuffix:(id)sender;
- (IBAction) removeSuffix:(id)sender;
- (IBAction) clearSuffixList:(id)sender;

- (IBAction) setDefaultTypes:(id)sender;

//Help
- (IBAction) showHelp:(id)sender;

//Select editor
- (IBAction) selectScriptEditor:(id)sender;
@end
