//
//  AMShellWrapper.h
//  CommX
//
//  Created by Andreas on 2002-04-24.
//  Based on TaskWrapper from Apple
//
//  2002-06-17 Andreas Mayer
//  - used defines for keys in AMShellWrapperProcessFinishedNotification userInfo dictionary
//  2002-08-30 Andreas Mayer
//  - added setInputStringEncoding: and setOutputStringEncoding:

#import <Foundation/Foundation.h>

#define AMShellWrapperProcessFinishedNotification @"AMShellWrapperProcessFinishedNotification"
#define AMShellWrapperProcessFinishedNotificationTaskKey @"AMShellWrapperProcessFinishedNotificationTaskKey"
#define AMShellWrapperProcessFinishedNotificationTerminationStatusKey @"AMShellWrapperProcessFinishedNotificationTerminationStatusKey"


// implement this protocol to control your AMShellWrapper object:

@protocol AMShellWrapperController

// output from stdout
- (void)appendOutput:(NSString *)output;

// output from stderr
- (void)appendError:(NSString *)error;

// This method is a callback which your controller can use to do other initialization
// when a process is launched.
- (void)processStarted:(id)sender;

// This method is a callback which your controller can use to do other cleanup
// when a process is halted.
- (void)processFinished:(id)sender withTerminationStatus:(int)resultCode;

// AMShellWrapper posts a AMShellWrapperProcessFinishedNotification when a process finished.
// The userInfo of the notification contains the corresponding NSTask ((NSTask *), key @"task")
// and the result code ((NSNumber *), key @"resultCode")

@end


@interface AMShellWrapper : NSObject
{
    NSTask 				*task;
    id					<AMShellWrapperController>controller;
	NSString			*workingDirectory;
	NSDictionary		*environment;
	NSArray				*arguments;
	id					stdinPipe;
	id					stdoutPipe;
	id					stderrPipe;
	NSFileHandle		*stdinHandle;
	NSFileHandle		*stdoutHandle;
	NSFileHandle		*stderrHandle;
	NSStringEncoding	inputStringEncoding;
	NSStringEncoding	outputStringEncoding;
	BOOL 				stdoutEmpty;
	BOOL				stderrEmpty;
	BOOL				taskDidTerminate;
}

// This is the designated initializer - pass in your controller and any task arguments.
// The first argument should be the path to the executable to launch with the NSTask.
// Allowed for stdin/stdout and stderr are
// - values of type NSFileHandle or
// - NSPipe or
// - nil, in which case this wrapper class automatically connects to the callbacks
//   and appendInput: method and provides asynchronous feedback notifications.
// The environment may be nil in which case the environment is inherited from the calling
// process.

- (id)initWithController:(id <AMShellWrapperController>)controller
							  inputPipe:(id)input
							 outputPipe:(id)output
							  errorPipe:(id)error
					   workingDirectory:(NSString *)directoryPath
							environment:(NSDictionary *)env
							arguments:(NSArray *)args;

// If you need something else than UTF8, set the code type of the task's input here
- (void)setInputStringEncoding:(NSStringEncoding)newInputStringEncoding;

// If you need something else than UTF8, tell the task what coding to use for output here
- (void)setOutputStringEncoding:(NSStringEncoding)newOutputStringEncoding;

// This method launches the process, setting up asynchronous feedback notifications.
- (void)startProcess;

// This method stops the process, stoping asynchronous feedback notifications.
- (void)stopProcess;

// input to stdin
- (void)appendInput:(NSString *)input;


@end

