//their stuff
#include <stdio.h>
#include <unistd.h>
#include <Carbon/Carbon.h>
#include <string.h>
#include <fcntl.h>

//my stuff
#include "platypus.h"





// Platypus windows and controls
WindowRef	platypusWindow, aboutWindow;
ControlRef	appNameTextField, scriptPathTextField, typeRadioButtons, iconSelectIncrementor, appIcon, iconNameTextField;

// Rectangle occupied by icon
Rect	iconRect = { 46, 48, 110, 112 };

// paths needed
char		execPath[kMaxPathLength] = "\0";
char		resourcesPath[kMaxPathLength] = "\0";

// icons
short		currentIcon = 0;
PlatypusIconStruct	platypusIcons[9];


/*************************************
program start
*************************************/
int main(int argc, char* argv[])
{
	#pragma unused (argc)

    OSErr		err;

    // Get path to application resource from path to executable passed
    // as first argument.
    err = InitPaths(argv[0]);
    if (err)
        DoFatalAlert("\pInitialization error", "\pAn error occurred when getting resource paths.");

    err = InitIcons();
    if (err)
        DoFatalAlert("\pInitialization error", "\pAn error occurred when loading icons.");

    // Create window and menu bar from nib file
    err = InitNib(&platypusWindow);
    if (err)
        DoFatalAlert("\pInitialization error", "\pAn error occurred when loading window from Nib file.");
    
    // Get references to window controls
    err = InitControls();
    if (err)
        DoFatalAlert("\pInitialization error", "\pAn error occurred when getting control references.");
    
    // Install application event handlers
    err = InitHandlers();
    if (err)
        DoFatalAlert("\pInitialization error", "\pAn error occurred when initing application event handlers.");

    
    // Display
    NewPlatypusWindow(platypusWindow);
    
    // Run the event loop
    RunApplicationEventLoop();

    return 0;
}


/*************************************
Quit application
*************************************/
void QuitApp (void)
{
    ExitToShell();

}



#pragma mark -

/*************************************
Install all event handlers
*************************************/
OSErr InitHandlers (void)
{
    OSErr	err = noErr;

    const EventTypeSpec	commandEvents[] = { {kEventClassCommand, kEventProcessCommand} };
    const EventTypeSpec windowEvents[] = { {kEventClassWindow, kEventWindowUpdate}, {kEventClassWindow, kEventWindowDrawContent}, {kEventClassWindow, kEventWindowClose}  };
    const EventTypeSpec aboutWindowEvents[] = { {kEventClassWindow, kEventWindowClose} };
    
    //command handler -- control events and menu events -- target is application
    err = InstallEventHandler(GetApplicationEventTarget(), (EventHandlerUPP)NewEventHandlerUPP(PlatypusCommandHandler), 1, commandEvents, NULL, NULL );
    err = +InstallWindowEventHandler(platypusWindow, (EventHandlerUPP)NewEventHandlerUPP(PlatypusWindowHandler), 3, windowEvents, NULL, NULL );
    err = +InstallWindowEventHandler(aboutWindow, (EventHandlerUPP)NewEventHandlerUPP(AboutWindowHandler), 1, aboutWindowEvents, NULL, NULL );
	err = +AEInstallEventHandler( kCoreEventClass, kAEReopenApplication, NewAEEventHandlerUPP((AEEventHandlerProcPtr)AppAEReopenHandler), 0, false );

    return noErr;
}

/*************************************
Generate strings containing paths to
the components in the application package
required for creating the script app
*************************************/
OSErr InitPaths (char *argvZero)
{
    char	contentsPath[kMaxPathLength];
    
    if (argvZero == NULL)
        return true;

    //get path to Contents folder within application bundle
    strcpy((char *)&contentsPath, (char *)argvZero);
    CutFileFromPath((char *)&contentsPath);
    CutFileFromPath((char *)&contentsPath);
    
    //get path to executables i.e. MacOS folder
    strcpy((char *)&execPath, (char *)&contentsPath);
    strcat((char *)&execPath, (char *)&"/MacOS/");
    
    //get path to Resources folder
    strcpy((char *)&resourcesPath, (char *)&contentsPath);
    strcat((char *)&resourcesPath, (char *)&"/Resources/");
    
    return noErr;
}

/*************************************
Load window and menu bar from Nib file
*************************************/
OSErr InitNib (WindowRef *window)
{
    OSErr		err;
    IBNibRef 		nibRef;
    
    err = CreateNibReference(CFSTR("main"), &nibRef);
    if (err)
        return err;
    
    err = SetMenuBarFromNib(nibRef, CFSTR("MenuBar"));
    if (err)
        return err;
    
    err = CreateWindowFromNib(nibRef, CFSTR("MainWindow"), window);
    if (err)
        return err;
    
    err = CreateWindowFromNib(nibRef, CFSTR("AboutWindow"), &aboutWindow);
    if (err)
        return err;

    // We don't need the nib reference anymore.
    DisposeNibReference(nibRef);

    return noErr;
}

/*************************************
Get references to window controls
*************************************/
OSErr InitControls (void)
{
    ControlID		controlID;
    ControlFontStyleRec	fontStyle;
    
    fontStyle.flags = kControlUseFontMask;
    fontStyle.style = 1;
    fontStyle.font = kControlFontBigSystemFont;
    //fontStyle.style |= 0;
    
        //get references to editable text fields
        controlID.signature = 'Text';
        controlID.id = 0;
        GetControlByID (platypusWindow, &controlID, &appNameTextField);
        
        controlID.signature = 'Text';
        controlID.id = 1;
        GetControlByID (platypusWindow, &controlID, &scriptPathTextField);
        
        SetControlFontStyle (scriptPathTextField, &fontStyle);
    	
    
		//icon name text field
		controlID.signature = 'IcNm';
        controlID.id = 0;
        GetControlByID (platypusWindow, &controlID, &iconNameTextField);
		
        
        
        //get reference to type radio buttons
        controlID.signature = 'Type';
        controlID.id = 0;
        GetControlByID (platypusWindow, &controlID, &typeRadioButtons);
        
        return noErr;
}

/*************************************
Load shipping icons from icns files
*************************************/
OSErr InitIcons (void)
{
	// Default Platypus App
	platypusIcons[0].icon = RegisterIcon( 'pIcn', 'pIc1', "platypusDefault.icns" );
	strcpy((char *)&platypusIcons[0].iconName, (char *)&"Platypus Default");
	strcpy((char *)&platypusIcons[0].fileName, (char *)&"platypusDefault.icns");
	
	// Generic Application
	GetIconRef(kOnSystemDisk, 'macs', kGenericApplicationIcon, &platypusIcons[1].icon);
	strcpy((char *)&platypusIcons[1].iconName, (char *)&"Generic App");
	
	// Platypus Application Icon
	platypusIcons[2].icon = RegisterIcon( 'pIcn', 'pIc2', "platypus.icns" );
	strcpy((char *)&platypusIcons[2].iconName, (char *)&"Platypus App");
	strcpy((char *)&platypusIcons[2].fileName, (char *)&"platypus.icns");
	
	// Hexley Fork Icon
	platypusIcons[3].icon = RegisterIcon( 'pIcn', 'pIc3', "hexleyFork.icns" );
	strcpy((char *)&platypusIcons[3].iconName, (char *)&"Hexley Fork");
	strcpy((char *)&platypusIcons[3].fileName, (char *)&"hexleyFork.icns");
	
	// Hexley Key Icon
	platypusIcons[4].icon = RegisterIcon( 'pIcn', 'pIc4', "hexleyKey.icns" );
	strcpy((char *)&platypusIcons[4].iconName, (char *)&"Hexley Key");
	strcpy((char *)&platypusIcons[4].fileName, (char *)&"hexleyKey.icns");
	
	// Hexley Box Icon
	platypusIcons[5].icon = RegisterIcon( 'pIcn', 'pIc5', "hexleyBox.icns" );
	strcpy((char *)&platypusIcons[5].iconName, (char *)&"Hexley Box");
	strcpy((char *)&platypusIcons[5].fileName, (char *)&"hexleyBox.icns");
	
	// Hexley Sit Icon
	platypusIcons[6].icon = RegisterIcon( 'pIcn', 'pIc6', "hexleySit.icns" );
	strcpy((char *)&platypusIcons[6].iconName, (char *)&"Hexley Sits");
	strcpy((char *)&platypusIcons[6].fileName, (char *)&"hexleySit.icns");
	
	// Hexley Magic Icon
	platypusIcons[7].icon = RegisterIcon( 'pIcn', 'pIc7', "hexleyMagic.icns" );
	strcpy((char *)&platypusIcons[7].iconName, (char *)&"Hexley Magic");
	strcpy((char *)&platypusIcons[7].fileName, (char *)&"hexleyMagic.icns");
	
	// Hexley App Icon
	platypusIcons[8].icon = RegisterIcon( 'pIcn', 'pIc8', "hexleyApp.icns" );
	strcpy((char *)&platypusIcons[8].iconName, (char *)&"Hexley App");
	strcpy((char *)&platypusIcons[8].fileName, (char *)&"hexleyApp.icns");
    
    return noErr;
}


#pragma mark -



/*************************************
Set controls to default and show main window
*************************************/

OSErr NewPlatypusWindow (WindowRef window)
{
    SetControlData(appNameTextField, kControlNoPart, kControlEditTextTextTag, strlen(kDefaultAppName), kDefaultAppName);
    SetControlData(scriptPathTextField, kControlNoPart, kControlEditTextTextTag, 0, "");
    SetIcon(0);
    ShowWindow(window);
    SelectWindow(window);
    return noErr;

}

/*************************************
Hide main window
*************************************/
void ClosePlatypusWindow (WindowRef window)
{
    HideWindow(window);
}


#pragma mark -


/*************************************
Take data entered and create app with it
*************************************/
OSErr CreateScriptApplication (void)
{
	char		command[4096] = "\0";
    char		appName[kMaxFileNameLength] = "";
    static char	appPath[kMaxPathLength] = "";
    char		scriptPath[kMaxPathLength] = "";
    Str255		pascalStr;
    OSErr		err = noErr;

	//get path to script
    GetControlData (scriptPathTextField, kControlEntireControl, kControlEditTextTextTag, sizeof(scriptPath), &scriptPath, NULL);

	//check if script path is valid
    if (access(scriptPath, F_OK) == -1)
    {
        DoAlert("\pInvalid Path", "\pNo file exists at the specified path.");
        return true;
    }
    if (UnixIsFolder((char *)&scriptPath))
    {
    	DoAlert("\pInvalid Path", "\pSpecified path points to a folder.");
        return true;
    }
    

	//get app name
    GetControlData (appNameTextField, kControlEntireControl, kControlEditTextTextTag, sizeof(appName), &appName, NULL);
    if (strlen((char *)&appName) == 0 || strlen((char *)&appName) > kMaxFileNameLength)
    {
    	DoAlert("\pInvalid Name", "\pYou must specify a name for the application to be created.");
        return true;
    }
	CopyCStringToPascal ((char *)&appName, pascalStr);
	
	//get app path
	err = NavigationPutPath(pascalStr, "\pSelect place to create app:", (char *)&appPath);
	if (err)
		return true;
	CutFileFromPath((char *)&appPath);
	
	
	//ok, add the path to the perl script that creates the app with perl defined as an interpreter
    strcpy((char *)&command, (char *)"/usr/bin/perl '");
    strcat((char *)&command, (char *)&execPath);
    strcat((char *)&command, (char *)"createapp.pl' ");
    
    //add the arguments
    // 1) Root
    strcat((char *)&command, (char *)"'");
    strcat((char *)&command, (char *)&appPath);
    strcat((char *)&command, (char *)"'");
    strcat((char *)&command, (char *)" ");
    // 2)
    strcat((char *)&command, (char *)"'");
    strcat((char *)&command, (char *)&appName);
    strcat((char *)&command, (char *)"'");
    strcat((char *)&command, (char *)" ");
    // 3)
    strcat((char *)&command, (char *)"'");
    strcat((char *)&command, (char *)&scriptPath);
    strcat((char *)&command, (char *)"'");
    strcat((char *)&command, (char *)" ");
    // 4)
    strcat((char *)&command, (char *)"'");
    strcat((char *)&command, (char *)&execPath);
    strcat((char *)&command, (char *)"shexec' ");
    strcat((char *)&command, (char *)" ");
    // 5)
    strcat((char *)&command, (char *)"'");
    strcat((char *)&command, (char *)&resourcesPath);
    strcat((char *)&command, (char *)"'");
    strcat((char *)&command, (char *)" ");
    // 6)
    strcat((char *)&command, (char *)"'");
    strcat((char *)&command, (char *)&platypusIcons[currentIcon].fileName);
    strcat((char *)&command, (char *)"'");
    
    system((char *)&command);
    
    return noErr;
}


#pragma mark -

/*************************************
Switch icon
*************************************/
OSErr SetIcon (short iconNum)
{
	currentIcon = iconNum;
	SetWindowProxyIcon(platypusWindow, platypusIcons[iconNum].icon);
	SetControlData(iconNameTextField, kControlNoPart, kControlEditTextTextTag, strlen((char *)&platypusIcons[iconNum].iconName), &platypusIcons[iconNum].iconName);
	
	return noErr;
}



#pragma mark -

/*************************************
Get path to a file via Navigation dialog
*************************************/

OSErr NavigationGetPath (char *path)
{
    FSSpec			fileSpec;
    OSErr			err;

    err = NavigationGetFSSpec(&fileSpec);
    if (err == userCanceledErr)
    {
       	return true;
    }
    else if (err)
    {
        DoAlert("\pNavigation Error", "\pFunction NavigationGetFSSpec() returned an error when called from function NavigationGetPath().");
        return true;
    }
    
    err = FSMakePath(fileSpec, (unsigned char *)path, kMaxPathLength);
    

    return noErr;
}

/*************************************
Get path to a location via Nav dialog
*************************************/
OSErr NavigationPutPath (Str255 defaultName, Str255 promptMessage, char *path)
{
	OSErr			err;
	FSSpec			fileSpec;

	//Get the location in which the app is to be created
	
	err = NavigationPutFSSpec (promptMessage, defaultName, &fileSpec);
	if (err == userCanceledErr)
	{
            return true;
    }
    else if (err)
    {
    		DoAlert("\pNavigation Error", "\pError getting file specification from NavigationLib");
    		return true;
    }       
	
	//create dest file to get path to it, to fool Apple's stupid function
	err = FSpCreate(&fileSpec, 'Plat', 'Plat', smSystemScript);
		
	//get path in which to create app from FSSpec
	err = FSMakePath(fileSpec, (unsigned char *)path, kMaxPathLength);
	
	//delete teh placeholder file
	HDelete(fileSpec.vRefNum, fileSpec.parID, fileSpec.name);
	
	if (err)
	{
		DoAlert("\pPath error", "\pUnable to create path from FSSpec");
		return true;
	}

	return noErr;
}

/*************************************
Get filespec via navigation
*************************************/
OSErr NavigationGetFSSpec (FSSpec *file)
{
    NavDialogOptions    dialogOptions;
    NavObjectFilterUPP  filterProc = NewNavObjectFilterUPP(NavFilterProc);
    OSErr               anErr = noErr;
    
    //  Specify default options for dialog box
    anErr = NavGetDefaultDialogOptions(&dialogOptions);
    if (anErr == noErr)
    {
        // Adjust the options to fit our needs
        //Set default location option
        dialogOptions.dialogOptionFlags |= kNavSelectDefaultLocation;
        dialogOptions.dialogOptionFlags |= kNavAllFilesInPopup;
        //Clear preview option
        dialogOptions.dialogOptionFlags ^= kNavAllowPreviews;
        
        if (anErr == noErr)
        {
            NavReplyRecord reply;
            
            // Call NavGetFile() with specified options
            anErr = NavGetFile (NULL, &reply, &dialogOptions,NULL, nil, filterProc, NULL, nil);
            
            if (anErr == noErr && reply.validRecord)
            {
                // Deal with multiple file selection
                long    count;
                
                anErr = AECountItems(&(reply.selection), &count);
                // Set up index for file list
                if (anErr == noErr)
                {
                    long index;
                    
                    //loop through all the files selected and open them in turn
                    for (index = 1; index <= count; index++)
                    {
                        AEKeyword   theKeyword;
                        DescType    actualType;
                        Size        actualSize;
                        FSSpec      documentFSSpec;
                        
                        // Get a pointer to selected file
                        anErr = AEGetNthPtr(&(reply.selection), index,
                                            typeFSS, &theKeyword,
                                            &actualType,&documentFSSpec,
                                            sizeof(documentFSSpec),
                                            &actualSize);
                        if (anErr == noErr)
                        {
                            *file = documentFSSpec;
                        }
                    }
                }
                //  Dispose of NavReplyRecord, resources, descriptors
                anErr = NavDisposeReply(&reply);
            }
        }
    }

    //DisposeRoutineDescriptor(filterProc);
    return anErr;
}



/*************************************
Get file spec for placement
*************************************/

OSErr NavigationPutFSSpec (Str255 message, Str255 defaultName, FSSpec *theFile)
{
	OSErr				theErr = noErr;
	NavReplyRecord			theReply;
	NavDialogOptions		dialogOptions;

	CGrafPtr			savePort = NULL;
	GDHandle			saveDevice = NULL;
	
	GetGWorld(&savePort, &saveDevice);
	
	//set the Navigation dialog options
	theErr = NavGetDefaultDialogOptions(&dialogOptions);
	if(theErr)
		return theErr;
	
	dialogOptions.dialogOptionFlags -= kNavDontAddTranslateItems;
	dialogOptions.dialogOptionFlags += kNavNoTypePopup;
	
	//set the default name in editable text field
	BlockMove(defaultName, dialogOptions.savedFileName, sizeof(Str255));
	
	//same goes for message
	if(message != nil)
            BlockMove(message, dialogOptions.message, sizeof(Str255));
	else
            dialogOptions.message[0] = 0;

	//call function that pops up the dialog
	theErr = NavPutFile(NULL, &theReply, &dialogOptions, NULL, 'TEXT', 'ttxt', NULL);
	
	if(theErr)
		goto Bail;
	
	//if user selects a valid location
	if(theReply.validRecord) 
	{
		FSSpec	finalFSSpec;	
		AEDesc 	resultDesc;
		
		//get AE description
		if((theErr = AEGetNthDesc(&(theReply.selection), 1, typeFSS, NULL, &resultDesc)) == noErr)
       	{
       		//retrive the file spec from it
			AEGetDescData(&resultDesc, &finalFSSpec, sizeof(FSSpec));
			*theFile = finalFSSpec;
            
			AEDisposeDesc(&resultDesc);
		}
	}
	
	//close
	NavDisposeReply(&theReply);
	
Bail:

	SetGWorld(savePort, saveDevice);
	return theErr;
}



/*************************************
Stupid filter for openable files in 
Navigation dialog
*************************************/
pascal Boolean NavFilterProc (AEDesc* theItem, void* info,  NavCallBackUserData callBackUD, NavFilterModes filterMode)
{
	#pragma unused (theItem, info, callBackUD, filterMode)

    //all files are valid for opening
    return true;
}







#pragma mark -

/*************************************
Application ommand handler
*************************************/
pascal OSStatus PlatypusCommandHandler (EventHandlerCallRef inCallRef, EventRef inEvent, void * inUserData)
{
	#pragma unused (inCallRef, inUserData)

    OSStatus        	result  = eventNotHandledErr;
    HICommand       	command;
    OSErr				err;
    static char			path[kMaxPathLength];
    	
                                    
    GetEventParameter( inEvent, kEventParamDirectObject, typeHICommand, NULL, sizeof( HICommand ), NULL, &command );

        switch ( GetEventKind( inEvent ) )
        {
            //regular old menu bar command
            case kEventCommandProcess:
                
                switch (command.commandID)
                {
                    //create app
                    case 'Crte':
                    	if (!CreateScriptApplication())
                    	{
                        	ClosePlatypusWindow(platypusWindow);
                        }
                        break;
                    
                    //select script
                    case 'SSel':
                    
                        err = NavigationGetPath((char *)&path);
                        if (err)
                        	return 0;
                        
                        SetControlData(scriptPathTextField, kControlNoPart, kControlEditTextTextTag, strlen((char *)&path), path);
    					DrawControls(platypusWindow);
                        break;
                    
                    //forward icon arrow
                    case 'Forw':
                        currentIcon++;
                        if (currentIcon > kNumIcons)
                        {
                            currentIcon = 0;
                         }
                         SetIcon(currentIcon);
                         InvalWindowRect (platypusWindow, &iconRect);
                        break;
                        
                    //backward icon arrow
                    case 'Back':
                        currentIcon--;
                        if (currentIcon < 0)
                        {
                            currentIcon = kNumIcons;
                         }
                         SetIcon(currentIcon);
                         InvalWindowRect (platypusWindow, &iconRect);
                        break;
                    
                    //quit
                  	case 'quit':
                  		QuitApp();
                  		break;
                  	
                  	//New window
                  	case 'new ':
                  		NewPlatypusWindow(platypusWindow);
                  		break;
                  	
                  	//About Platypus
                  	case 'abou':
                  		ShowWindow(aboutWindow);
                  		SelectWindow(aboutWindow);
                  		break;
                  	
                  	//Platypus website
                  	case 'PlWb':
                  		OpenURL((char *)&kPlatypusWebsite);
                  		break;
                  	
                  	//Contact author
                  	case 'CoAu':
                  		SendMail((char *)&kAuthorEmail);
                  		break;
                }
                break;
        }
        
        return noErr;
}

/*************************************
Handler for main window
*************************************/
pascal OSStatus PlatypusWindowHandler (EventHandlerCallRef inCallRef, EventRef inEvent, void * inUserData)
{ 
    #pragma unused (inCallRef, inUserData)

    switch (GetEventKind(inEvent))
    {
        case kEventWindowUpdate:
            BeginUpdate(platypusWindow);
            SetPort(GetWindowPort(platypusWindow));
            SetThemeWindowBackground(platypusWindow, kThemeBrushDrawerBackground,1 );
            DrawControls(platypusWindow);
            PlotIconRef (&iconRect,kAlignAbsoluteCenter, kTransformNone,  kIconServicesNormalUsageFlag, platypusIcons[currentIcon].icon);
            EndUpdate(platypusWindow);
            break;
        case kEventWindowDrawContent:
            SetThemeWindowBackground(platypusWindow, kThemeBrushDrawerBackground, 1);
            DrawControls(platypusWindow);
            PlotIconRef (&iconRect,kAlignAbsoluteCenter, kTransformNone,  kIconServicesNormalUsageFlag, platypusIcons[currentIcon].icon);
            
            break;
        case kEventWindowClose:
            ClosePlatypusWindow(platypusWindow);
            break;
        }
    
    return noErr;
}

/*************************************
Handler for about window
*************************************/
pascal OSStatus AboutWindowHandler (EventHandlerCallRef inCallRef, EventRef inEvent, void * inUserData)
{ 
	#pragma unused (inCallRef, inUserData)

	switch (GetEventKind(inEvent))
    {
        case kEventWindowClose:
            HideWindow(aboutWindow);
            break;
        }
    
    return noErr;
}




/*************************************
Handler for when Dock icon is clicked
*************************************/

static OSErr AppAEReopenHandler( const AppleEvent *appleEvt, AppleEvent* reply, UInt32 refcon )
{
	#pragma unused (appleEvt, reply, refcon)

    if (IsWindowVisible(platypusWindow))
    	SelectWindow(platypusWindow);
    else
    	NewPlatypusWindow(platypusWindow);

    return noErr;
}










